/* Definizione dei metodi di Vector */

#include "Vector.h"
#include <math.h>

Vector::Vector(): x(0), y(0) {
}

Vector::Vector(double xx, double yy): x(xx), y(yy) {
}

Vector::Vector(const Vector &v): x(v.x), y(v.y) {
}

bool Vector::operator == (const Vector &v) const {
  return (x == v.x) && (y == v.y);
}

bool Vector::operator != (const Vector &v) const {
  return (x != v.x) || (y != v.y);
}

Vector Vector::operator + () const {
  return *this;
}

Vector Vector::operator - () const {
  return Vector(-x, -y);
}

Vector Vector::operator + (double n) const {
  return Vector(x+n, y+n);
}

Vector Vector::operator - (double n) const {
  return Vector(x-n, y-n);
}

Vector Vector::operator * (double n) const {
  return Vector(x*n, y*n);
}

Vector Vector::operator / (double n) const {
  return Vector(x/n, y/n);
}

Vector Vector::operator + (const Vector &v) const {
  return Vector(x+v.x, y+v.y);
}

Vector Vector::operator - (const Vector &v) const {
  return Vector(x-v.x, y-v.y);
}

double Vector::operator * (const Vector &v) const {
  return x*v.x + y*v.y;
}

const Vector &Vector::operator = (const Vector &v) {
  x = v.x;
  y = v.y;
}

const Vector &Vector::operator += (double n) {
  x += n;
  y += n;
}

const Vector &Vector::operator -= (double n) {
  x -= n;
  y -= n;
}

const Vector &Vector::operator *= (double n) {
  x *= n;
  y *= n;
}

const Vector &Vector::operator /= (double n) {
  x /= n;
  y /= n;
}

const Vector &Vector::operator += (const Vector &v) {
  x += v.x;
  y += v.y;
}

const Vector &Vector::operator -= (const Vector &v) {
  x -= v.x;
  y -= v.y;
}

double Vector::norm() const {
  return sqrt(x*x+y*y);
}

Vector Vector::normalized() const {
  return *this/norm();
}

Vector Vector::normal() const {
  return Vector(-y, x);
}

Vector Vector::project(const Vector &v) const {
  double k = (*this*v)/(v*v);
  return v*k;
}

Vector Vector::reflect(const Vector &v) const {
  return project(v)*2-*this;
}