/* Semplice implementazione di un vettore bidimensionale */

#ifndef VECTOR_H
#define VECTOR_H

class Vector {
  
  public:
    
    // costruttori
    Vector();
    Vector(double x, double y);
    Vector(const Vector &v);
    
    // operatori di confronto
    bool operator == (const Vector &v) const;
    bool operator != (const Vector &v) const; 
  
    // + e - unari
    Vector operator +() const;
    Vector operator -() const;
    
    // addizione con scalare
    Vector operator +(double n) const;
    
    // sottrazione con scalare
    Vector operator -(double n) const;

    // moltiplicazione con scalare
    Vector operator *(double n) const;

    // divisione con scalare
    Vector operator /(double n) const;
    
    // addizione con vettore
    Vector operator +(const Vector &v) const;

    // sottrazione con vettore
    Vector operator -(const Vector &v) const;

    // prodotto scalare
    double operator *(const Vector &v) const;
    
    // assegnazione
    const Vector &operator =(const Vector &v);
    
    // assegnazione con operazioni su scalari
    const Vector &operator +=(double n);
    const Vector &operator -=(double n);
    const Vector &operator *=(double n);
    const Vector &operator /=(double n);
    
    // assegnazione con operazioni su vettori
    const Vector &operator +=(const Vector &v);
    const Vector &operator -=(const Vector &v);

    // accesso ai membri
    inline double get_x() const { return x; }
    inline double get_y() const { return y; }
    
    // norma (lunghezza) del vettore
    double norm() const;
    
    // restituisce il versore
    Vector normalized() const;
    
    // restituisce il vettore ruotato di 90° antiorario (vett. perpendicolare)
    Vector normal() const;
    
    // calcola il vettore proiezione su un altro vettore
    Vector project(const Vector &v) const;
    
    // calcola il simmetrico rispetto ad un asse
    Vector reflect(const Vector &v) const;
  
  protected:
    double x, y;
    
  private:
};

#endif