/* Simulazione fisica */

// librerie standard
#include <stdlib.h>
#include <stdio.h>

// libreria SDL per la grafica
#include <SDL/SDL.h>
#include <SDL/SDL_gfxPrimitives.h>

// classe vector dela STL
#include <vector>

// classi di questo progetto
#include "Vector.h"
#include "Corpo.h"
#include "Universo.h"

// dimensioni della finestra
#define WIDTH   800
#define HEIGHT  600


// variabili globali
SDL_Surface *screen;
Universo uni;

//--------------------------------


// disegna un vettore v con punto di applicazione p
void draw_vector(const Vector &v, const Vector &p) {
  lineColor(screen, 
	    (WIDTH/2)+p.get_x(), (HEIGHT/2)-p.get_y(), 
	    (WIDTH/2)+p.get_x()+v.get_x(), (HEIGHT/2)-(p.get_y()+v.get_y()), 
	    0x00ff00ff);
}

// disegna un corpo
void draw_corpo(const Corpo &c) {
  Vector centro = c.get_pos();
  // cerchio
  circleColor(screen, (WIDTH/2)+centro.get_x(), (HEIGHT/2)-centro.get_y(), c.get_r(), 0x00ffffff);
  // vettore velocità
  draw_vector(c.get_vel(), centro);
}

// disegna tutta la scena
void draw_universe() {
  
  // cancello lo schermo
  SDL_Rect rec = {0,0,WIDTH,HEIGHT};
  SDL_FillRect(screen, &rec, 0x00000000);
  
  // disegno gli assi
  hlineColor(screen, 0, WIDTH-1, HEIGHT/2, 0xff0000ff);
  vlineColor(screen, WIDTH/2, 0, HEIGHT-1, 0xff0000ff);
  
  // disegno i corpi
  vector<Corpo>::iterator it = uni.corpi.begin();
  for ( ; it < uni.corpi.end(); it++ ) {
    draw_corpo(*it);
  }
  
  // visualizzo tutto
  SDL_Flip(screen);
}


// ciclo degli eventi principale
void main_loop() {
  SDL_Event ev;
  bool playing = true;
  bool contmode = true;
  
  while (playing) {
    int ret = (contmode)?SDL_PollEvent(&ev):SDL_WaitEvent(&ev);
    if (ret) {
      switch (ev.type) {
        case SDL_QUIT: 
          playing=0; 
          break;
        case SDL_KEYDOWN: {
          switch (ev.key.keysym.sym) {
            case SDLK_ESCAPE: playing = 0; break;
            case SDLK_BACKSPACE: contmode ^= 1; break;
            default: break;
            }
          }
          break;
        case SDL_MOUSEBUTTONDOWN: {
	  contmode ^= 1;
          break;
        }
        case SDL_MOUSEBUTTONUP: {
          //contmode = false;
          break;
        }
      }
    }
    
    // anima l'universo
    uni.animate();
    
    // disegna l'universo
    draw_universe();
    
    // ritardo
    SDL_Delay(10);
  }
  
}

int main() {
  
  
  // inizializzo la grafica
  SDL_Init(SDL_INIT_VIDEO);
  
  const SDL_VideoInfo* info = SDL_GetVideoInfo(); 
  int vidFlags = SDL_GL_DOUBLEBUFFER;
  vidFlags |= (info->hw_available)?SDL_HWSURFACE:SDL_SWSURFACE;
  int bpp = info->vfmt->BitsPerPixel;
  
  SDL_EventState(SDL_MOUSEMOTION, SDL_IGNORE);
  screen = SDL_SetVideoMode(WIDTH, HEIGHT, bpp, vidFlags);
  if (screen == NULL) return 1;
  
  // imposto i limiti dell'universo
  uni.set_limits(-(WIDTH/2), -(HEIGHT/2), (WIDTH/2), (HEIGHT/2));
  
  // popolo l'universo
  /*Corpo c1(20, 20, Vector(0,0), Vector(1,0));
  Corpo c2(200, 40, Vector(150,20), Vector(-1,0));
  Corpo c3(10, 10, Vector(-100,200), Vector(1,-2));
  Corpo c4(400, 60, Vector(-200,-200), Vector(2,2));*/
  
  Corpo c1(50, 30, Vector(-200,0), Vector(1,0));
  Corpo c2(50, 30, Vector(-40,0), Vector(0,0));
  Corpo c3(50, 30, Vector(+20,0), Vector(0,0));
  Corpo c4(50, 30, Vector(+80,0), Vector(0,0));
  uni.add_corpo(c1);
  uni.add_corpo(c2);
  uni.add_corpo(c3);
  uni.add_corpo(c4);
  
  // ciclo principale
  main_loop();
  
  return 0;
}
