/* 
 CROSSWORD
 
 A tool to create crosswords
 
 TODO: error checking
*/

#include <stdio.h>
#include <stdlib.h>
#include <SDL/SDL.h>
#include <SDL/SDL_ttf.h>
#include "../include/class_virscrsdl.h"

#define min(a,b)	(((a)<(b))?(a):(b))

#define ERR_SDLINIT	1
#define ERR_TTFINIT	2
#define ERR_NOLOAD	3
#define ERR_NOSAVE	4
#define ERR_NOIMAGE	5
#define ERR_ONECROSS	6
#define ERR_BADCMD	7
#define ERR_BADLOAD	8

const char *font = "verdana.ttf";

typedef struct {
  int n;
  char filled;
  char c;
} Cell;

SDL_Surface *screen = NULL;
VirtualScreenSDL *virscr = NULL;
char fBlended = 0;
char *filetoload = NULL;
int cellSize;
int startx = 10,
    starty = 10;
int sizex = 15,
    sizey = 15;
int sizepanel = 150;
int selx = -1, 
    sely = -1;
Cell *board = NULL;

void exitProc(int n) {
  if (virscr != NULL) delete virscr;
  SDL_Quit();
  TTF_Quit();  
  if (board != NULL) free(board);
  exit(n);
}

void errorProc(int n) {
  switch (n) {
    case ERR_SDLINIT: 
      fprintf(stderr, "Impossibile inizializzare SDL.\n");
      exitProc(n);
      break;
    case ERR_TTFINIT: 
      fprintf(stderr, "Impossibile inizializzare SDL TTF.\n");
      exitProc(n);
      break;
    case ERR_NOLOAD: 
      fprintf(stderr, "Impossibile caricare il file.\n");
      break;
    case ERR_NOSAVE: 
      fprintf(stderr, "Impossibile salvare il file.\n");
      break;
    case ERR_NOIMAGE: 
      fprintf(stderr, "Impossibile salvare l\'immagine.\n");
      break;
    case ERR_ONECROSS: 
      fprintf(stderr, "Viene aperto solo un cruciverba.\n");
      break;
    case ERR_BADCMD: 
      fprintf(stderr, "Opzione non riconosciuta.\n");
      break;
  }
}

void drawPanel() {
  int i;
  char buf[100];

  virscr->TextAlign = VS_TA_CENTER;
  int butx1 = startx+sizex*cellSize+20;
  int butx2 = startx+sizex*cellSize+sizepanel;
  int butheight = 20;
  int butspacing = 10;
  char *buttexts[8] = {"Numera", "Crittografa", "Rimuovi numeri", "Rimuovi lettere", "Ridimensiona", "Carica", "Salva", "Salva immagine"};
  int butcolors[8] = {0xFFFFFFCF, 0xFFFFFF8F, 0xFFFFCFCF, 0xFFFF8F8F, 0xFFCFFFCF, 0xFFCFCFFF, 0xFF8F8FFF, 0xFFFFCFFF};
  
  for (i=0; i<8; i++) {  
    virscr->FRect(butx1, starty+(butheight+butspacing)*i, butx2, starty+butheight+(butheight+butspacing)*i, butcolors[i]);
    virscr->Rect(butx1, starty+(butheight+butspacing)*i, butx2, starty+butheight+(butheight+butspacing)*i, 0xFF000000);
    virscr->PrintRect(2, butx1, starty+(butheight+butspacing)*i, butx2, starty+butheight+(butheight+butspacing)*i, buttexts[i], 0xFF000000, 0);  
  }
  
  i = 250;
  sprintf(buf, "Size: %i %i", sizex, sizey);
  virscr->Print(2, startx+cellSize*sizex+20, i, buf, 0xFF000000, 0);
  if ((selx != -1) && (sely != -1)) {
    sprintf(buf, "Cell: %i %i", selx, sely);
    virscr->Print(2, startx+cellSize*sizex+20, i+20, buf, 0xFF000000, 0);
    sprintf(buf, "Num: %i", (board+sely*sizex+selx)->n);
    virscr->Print(2, startx+cellSize*sizex+20, i+40, buf, 0xFF000000, 0);
    sprintf(buf, "Char: %c", (board+sely*sizex+selx)->c);
    virscr->Print(2, startx+cellSize*sizex+20, i+60, buf, 0xFF000000, 0);
  } else {
    virscr->Print(2, startx+cellSize*sizex+20, i+20, "Cell: none", 0xFF000000, 0);
  }
}

void redraw() {
  int i,j;
  char n[20];

  virscr->Clear(0xFFFFFFFF);
  // draws lines
  virscr->Rect(startx-1, starty-1, startx+sizex*cellSize+1, starty+sizey*cellSize+1, 0xFF000000);
  for (i=0; i<=sizex; i++) 
    virscr->Line(startx+i*cellSize, starty, startx+i*cellSize, starty+sizey*cellSize, 0xFF000000);
  for (i=0; i<=sizey; i++)
    virscr->Line(startx, starty+i*cellSize, startx+sizex*cellSize, starty+i*cellSize, 0xFF000000);

  // draws blocks
  if ((selx != -1) && (sely != -1)) {
    virscr->Rect(startx+selx*cellSize+1, starty+sely*cellSize+1, startx+(selx+1)*cellSize-1, starty+(sely+1)*cellSize-1, 0xFFFF0000);
    virscr->FRect(startx+selx*cellSize+2, starty+sely*cellSize+2, startx+(selx+1)*cellSize-2, starty+(sely+1)*cellSize-2, 0xFFFFEFEF);
  }
  for (j=0; j<sizey; j++) {
    for (i=0; i<sizex; i++) {
      if ((board+j*sizex+i)->filled) 
        virscr->FRect(startx+i*cellSize+2, starty+j*cellSize+2, startx+(i+1)*cellSize-2, starty+(j+1)*cellSize-2, 0xFF000000);
      else {
        if ((board+j*sizex+i)->n > 0) {
          sprintf(n, "%i", (board+j*sizex+i)->n);
          virscr->Print(0, startx+i*cellSize+1, starty+j*cellSize, n, 0xFF000000, 0);
        }
        if ((board+j*sizex+i)->c > 0) {
          sprintf(n, "%c", (board+j*sizex+i)->c);
          int cx, cy;
          virscr->GetPrintSize(1, n, &cx, &cy);
          cx = (cellSize-cx)>>1;
          cy = (cellSize-cy)>>1;
          virscr->Print(1, startx+i*cellSize+cx, starty+j*cellSize+cy, n, 0xFF000000, 0);
        }
      }
    }
  }

  // draws panel
  drawPanel();  
  virscr->Show();
}

int isFilled(int x, int y) {
  if ((unsigned)x >= sizex || (unsigned)y >= sizey) return (1);
  return (board+y*sizex+x)->filled;
}

void autoNumerate() {
  int i,j, curnum=1;
  for (j=0; j<sizey; j++) {
    for (i=0; i<sizex; i++) {
      if (!isFilled(i,j)) {
        if ( (isFilled(i-1,j) && !isFilled(i+1,j))||(isFilled(i,j-1) && !isFilled(i,j+1)) )
          (board+j*sizex+i)->n = curnum++;
        else
          (board+j*sizex+i)->n = 0;
      }
    }
  }
}

void encrypt() {
  char c, count=1;
  int i,j;
  int letters[26];
  memset(letters, 0, sizeof(int)*26);
  for (i=0; i<sizex*sizey; i++) {
    (board+i)->n = 0;
    c = (board+i)->c;
    if (c >= 'A' && c <= 'Z') {
      if (letters[c-'A'] == 0) letters[c-'A'] = count++;
      (board+i)->n = letters[c-'A'];
    }
  }
}

char *askBox(char *text) {
  int x = (screen->w-300)/2; 
  int y = (screen->h-50)/2; 
  int tostay = 1, len=0;
  char *string = NULL;
  SDL_Event ev;
  
  virscr->FRect(x,y, x+300, y+50, 0xFFDFDFDF);
  virscr->Rect(x,y, x+300, y+50, 0xFF000000);
  virscr->TextAlign = VS_TA_LEFT;
  virscr->PrintRect(2, x+1,y,x+300, y+20, text, 0xFF000000, 0);
  virscr->Rect(x+2, y+22, x+296, y+48, 0xFF000000);
  virscr->Show();
  
  SDL_EnableUNICODE(SDL_ENABLE);
  while (tostay) {
    if (SDL_WaitEvent(&ev)) {
      switch (ev.type) {
        case SDL_KEYDOWN:
          switch (ev.key.keysym.sym) {
            case SDLK_ESCAPE: tostay = 0; break;
            case SDLK_RETURN:
              return(string);
              break;
            case SDLK_BACKSPACE:
              if (string != NULL && (len > 0)) {
                string[--len] = '\0';
                virscr->FRect(x+3, y+23, x+295, y+47, 0xFFDFDFDF);
                virscr->PrintRect(2, x+3,y+22, x+300,y+48, string, 0xFF000000, 0);
                virscr->Show(x,y, x+300,y+50);
              }
              break;
            default:
              if (ev.key.keysym.sym <= 127) {
                char ch = ev.key.keysym.sym;
		if (!(ev.key.keysym.unicode & 0xFF80)) ch = ev.key.keysym.unicode & 0x7F;
                string = (char *)realloc(string, sizeof(char)*(len+2));
                string[len] = ch;
                string[len+1] = '\0';
                len++;
                virscr->PrintRect(2, x+3,y+22, x+300,y+48, string, 0xFF000000, 0);
                virscr->Show(x,y, x+300,y+50);
              }
          }
          break;
      }
    }
  }
  
  return(NULL);
}

void createVirscr(int x, int y) {
  if (screen != NULL) SDL_FreeSurface(screen);
  screen = SDL_SetVideoMode(x,y, 32, SDL_HWSURFACE | SDL_RESIZABLE);
  SDL_WM_SetCaption("Crossword",NULL);
  cellSize = min((x-10-sizepanel)/sizex-1, (y-10)/sizey-1);
  if (virscr == NULL) {
    virscr = new VirtualScreenSDL(screen);
    if (fBlended) virscr->TextMode = VS_TM_BLENDED;
    virscr->LoadFont((char*)font, cellSize/3);
    virscr->LoadFont((char*)font, 2*cellSize/3);
    virscr->LoadFont((char*)font, 16);
    virscr->SetFontStyle(1, TTF_STYLE_BOLD);
  } else {
    virscr->ChangeSurface(screen);
    virscr->ReloadFont(0, (char*)font, cellSize/3);
    virscr->ReloadFont(1, (char*)font, 2*cellSize/3);
  }
}

void resize(int nsizex, int nsizey) {
  if (nsizex == 0 || nsizey == 0) {
    char *newsizex = askBox("Nuova larghezza:");
    if (newsizex == NULL) return;
    char *newsizey = askBox("Nuova altezza:");
    if (newsizey == NULL) {
      free(newsizex);
      return;
    }
    nsizex = atoi(newsizex);
    nsizey = atoi(newsizey);
    free(newsizex);
    free(newsizey);
  }
  Cell *board2 = (Cell*)calloc(nsizex*nsizey, sizeof(Cell));
  for (int j=0; j<min(sizey, nsizey); j++) {
    for (int i=0; i<min(sizex, nsizex); i++) {
      *(board2+j*nsizex+i) = *(board+j*sizex+i);
    }
  }
  free(board);
  sizex = nsizex;
  sizey = nsizey;
  board = board2;
  createVirscr((cellSize+1)*sizex+10+sizepanel, (cellSize+1)*sizey+10);
  redraw();
}

void loadCross(char *file) {
  FILE *f;
  int i, newx, newy;
  
  if (file == NULL) {
    file = askBox("File da aprire: ");
    if (file != NULL) {
      f = fopen(file, "r");
      free(file);
    } else return;
  } else f = fopen(file, "r");

  if (f == NULL) {
    errorProc(ERR_NOLOAD);
    return;
  }

  if (fscanf(f, "%i %i\n", &newx, &newy) != 2) {
    errorProc(ERR_BADLOAD);
    fclose(f);
    return;
  }
  resize(newx, newy);
  for (i=0; i<sizex*sizey; i++) {
    int a,b,c;
    if (fscanf(f, "%i %i %i\n", &a, &b, &c) != 3) {
      errorProc(ERR_BADLOAD);
      fclose(f);
      return;
    }
    (board+i)->filled = a;
    (board+i)->n = b;
    (board+i)->c = c;
  }
  fclose(f);
}

void saveCross() {
  char *file = askBox("Nome per il salvataggio: ");
  if (file != NULL) {
    FILE *f = fopen(file, "w");
    if (f == NULL) {
      errorProc(ERR_NOSAVE);
      free(file);
      return;
    }
    fprintf(f, "%i %i\n", sizex, sizey);
    for (int i=0; i<sizex*sizey; i++) {
      fprintf(f, "%i %i %i\n", (board+i)->filled, (board+i)->n, (board+i)->c);
    }
    fclose(f);
    free(file);
  }
}

void saveImage() {
  char *file = askBox("Nome dell\'immagine: ");
  selx = sely = -1;
  redraw();
  if (file != NULL) {
    file = (char *)realloc(file, sizeof(char)*(strlen(file)+10));
    strcat(file, ".bmp");
    virscr->SaveImage(startx-2,starty-2, startx+sizex*cellSize+3,starty+sizey*cellSize+3, file);
    free(file);
  }
}

void showSyntax() {
  printf("\
Crossword - versione 1.0\n\
Sintassi:\n\
  crossword [opzioni] [file]\n\
Opzioni:\n\
  --sizex n    cambia la larghezza del cruciverba\n\
  --sizey n    cambia l\'altezza del cruciverba\n\
  --blended    scrive i caratteri in modo sfumato\n\n");
}

void readCmd(int argc, char *argv[]) {
  int i;
  for (i=1; i<argc; i++) {
    if (argv[i][0] == '-' && argv[i][1] == '-') {
      if (strcmp(argv[i]+2, "sizex") == 0) {
        sizex = atoi(argv[++i]);
      } else if (strcmp(argv[i]+2, "sizey") == 0) {
        sizey = atoi(argv[++i]);
      } else if (strcmp(argv[i]+2, "blended") == 0) {
        fBlended = 1;
      } else if (strcmp(argv[i]+2, "help") == 0) {
        showSyntax();
        exitProc(0);
      } else {
        errorProc(ERR_BADCMD);
        exitProc(ERR_BADCMD);
      }
    } else {
      if (filetoload != NULL) {
        errorProc(ERR_ONECROSS);
      } else {
        filetoload = strdup(argv[i]);
      }
    }
  }
}

int main(int argc, char *argv[]) {
  int tostay = 1;
  SDL_Event ev;
  
  readCmd(argc, argv);
  
  board = (Cell*)malloc(sizeof(Cell)*sizex*sizey);
  for (int i=0; i<sizex*sizey; i++) {
    (board+i)->filled = 0;
    (board+i)->n = 0;
    (board+i)->c = 0;
  }
  
  if (SDL_Init(SDL_INIT_VIDEO) < 0) errorProc(ERR_SDLINIT);
  if (TTF_Init() < 0) errorProc(ERR_TTFINIT);
  SDL_EnableKeyRepeat(250, 80);

  cellSize = 25;
  createVirscr((cellSize+1)*sizex+10+sizepanel, (cellSize+1)*sizey+10);
  redraw();
  SDL_ShowCursor(SDL_ENABLE);
  
  if (filetoload != NULL) {
    loadCross(filetoload);
    free(filetoload);
    redraw();
  }  
  
  while (tostay) {
    
    SDL_WaitEvent(&ev);
    switch (ev.type) {
      case SDL_KEYDOWN: 
        switch (ev.key.keysym.sym) {
          case SDLK_HOME:
            selx = 0;
            redraw();
            break;
          case SDLK_END:
            selx = sizex-1;
            redraw();
            break;
          case SDLK_LEFT:
            if (selx > 0) selx--;
            redraw();
            break;
          case SDLK_RIGHT:
            if (selx < sizex-1) selx++;
            redraw();
            break;
          case SDLK_UP:
            if (sely > 0) sely--;
            redraw();
            break;
          case SDLK_DOWN:
            if (sely < sizey-1) sely++;
            redraw();
            break;
          case SDLK_SPACE:
          case SDLK_BACKSPACE:
            if (selx != -1) (board+sely*sizex+selx)->c = 0;
            redraw();
            break;
          case SDLK_RETURN:
            if (selx != -1) (board+sely*sizex+selx)->filled ^= 1;
            redraw();
            break;
          default:
            if (selx != -1) {
              if (ev.key.keysym.sym >= SDLK_a && ev.key.keysym.sym <= SDLK_z) {
                (board+sely*sizex+selx)->c = ev.key.keysym.sym-'a'+'A';
                redraw();
              } else if (ev.key.keysym.sym >= SDLK_0 && ev.key.keysym.sym <= SDLK_9) {
                (board+sely*sizex+selx)->n = ev.key.keysym.sym-'0';
                redraw();
              }
            }
        }
        break;
      case SDL_QUIT: tostay = 0; break;
      case SDL_VIDEORESIZE: {
        createVirscr(ev.resize.w, ev.resize.h);
        redraw();
        break;
      }
      case SDL_MOUSEBUTTONDOWN: {
        int x = ev.button.x-startx;
        int y = ev.button.y-starty;
        if ((unsigned)x < cellSize*sizex && (unsigned)y < cellSize*sizey) {
          x /= cellSize;
          y /= cellSize;
          if (ev.button.button == SDL_BUTTON_RIGHT) {
            (board+y*sizex+x)->filled ^= 1;
            redraw(); 
          }
          if (ev.button.button == SDL_BUTTON_LEFT) {
            selx = x;
            sely = y;
            redraw(); 
          }
        } else {
          x = ev.button.x-(startx+sizex*cellSize+20);
          y = ev.button.y-(starty);
          if ((unsigned)x < (sizepanel-20)) {
            int i;
            for (i=0; i<8; i++) {
              if (y < 20) break;
              y -= 10+20;
            }
            switch (i) {
              case 0: autoNumerate(); redraw(); break;
              case 1: encrypt(); redraw(); break;
              case 2: for (int j=0; j<sizex*sizey; j++) (board+j)->n=0; redraw(); break;
              case 3: for (int j=0; j<sizex*sizey; j++) (board+j)->c=0; redraw(); break;
              case 4: resize(0,0); redraw(); break;
              case 5: loadCross(NULL); redraw(); break;
              case 6: saveCross(); redraw(); break;
              case 7: saveImage(); break;
            }
          }
        }
        break;
      }
    }
  }
  
  exitProc(0);
  return(0);
}
