#include <string>
#include <vector>
#include "dict.h"
#include "cryptosolver.h"

using std::string;
using std::vector;


// Load a dictionary
void CryptoSolver::load_dict(const Dict& d) {
  dic = d;
}

// Load a dictionary from file
void CryptoSolver::load_dict(const string& s) {
  dic = Dict(s);
}

// solve the cryptogram
void CryptoSolver::solve(const Cryptogram& cg) {
  vector<Cryptogram> words = cg.extract_words();
  for (int i=0; i<words.size(); i++) {
    std::cout << "\n" << words[i] << " :\n";
    std::cout << find_matching(words[i]);
  }
}


// ---- PRIVATE METHODS -------------------------------------------------------

// Returns a dic with matching words
Dict CryptoSolver::find_matching(const Cryptogram &word) const {
  Dict ret;
  for (int i=0; i<dic.length(); i++) {
    if (dic[i].length() != word.length()) continue;
    if (word_match(dic[i], word)) ret.insert(dic[i]);
  }
  return ret;
}

bool CryptoSolver::word_match(const string& s, const Cryptogram &word) {
  char numtab[26], chartab[26];
  
  memset(numtab, 0, sizeof(numtab));
  memset(chartab, 0, sizeof(chartab));
  
  for (int i=0; i<word.length(); i++) {
    int n = word[i];
    if (s[i] < 'a' || s[i] > 'z') continue;
    if (numtab[n] == 0) {
      if (chartab[s[i]-'a'] == 1) return false;
      numtab[n] = s[i];
      chartab[s[i]-'a'] = 1;
    } else {
      if (numtab[n] != s[i]) return false;
    }
  }
  return true;
}
