/* CRYPTOGRAM */
/* FREQLANG CLASS */

/*   Contiene le frequenze per lettere, digrammi e trigrammi per una lingua */

/* TODO: maiuscole, gestione errori */

#include <string.h>

int pow(int a, int b) {
  int r = 1;
  for (int i=0; i<b; i++) r *= a;
  return r;
}

class FreqLang {
 public:
  FreqLang(const char *lang);
  ~FreqLang();
  char *get_lang();
  void add_freq(char *token, double freq);
  void inc_freq(char *token, double amount);
  void div_freqs_by(int table, double tot);
  double get_freq(char *token);
  void get_next_freq(int table, char *token, double *freq);
  int atend(int table);
  void reset() { ptr = 0; }

 private:
  char lang[10];
  double *list[3];
  int ptr;

  long calc_pos(char *token);
  char *reverse_pos(long pos, int len, char *token);
};

/* Inizializza l'oggetto */
FreqLang::FreqLang(const char *lang) {
  ptr = 0;
  strncpy(this->lang, lang, 9);
  list[0] = list[1] = list[2] = NULL;
}

/* Libera la memoria occupata dalle tabelle delle frequenze */
FreqLang::~FreqLang() {
  for (int i=0; i<3; i++)
    if (list[i] != NULL) free(list[i]);
}

/* Restituisce la lingua delle tabelle */
char *FreqLang::get_lang() {
  return lang;
}

int FreqLang::atend(int table) {
 return (ptr >= pow(27, table+1));
}

void FreqLang::get_next_freq(int table, char *token, double *freq) {
  if (table < 0 || table > 2 || list[table] == NULL) return;
  reverse_pos(ptr, table+1, token);
  *freq = list[table][ptr++];
}

/* Calcola la posizione che avrà la frequenza nella tabella */
long FreqLang::calc_pos(char *token) {
  // TODO: maiuscole, controllo stringe malformate!
  long pos = 0;
  int i = 0;
  while (token[i] != '\0') {
    int p = 0;
    if (token[i] != ' ') p = (unsigned char)(token[i]-'a'+1);
    pos = 27*pos+p;
    i++;
  }
  return pos;
}

/* Calcola i caratteri corrispondenti alla posizione pos */
char *FreqLang::reverse_pos(long pos, int len, char *token) {
  for (int i=0; i<len; i++) {
   int r = pos % 27;
   token[len-i-1] = (r == 0)?' ':('a'+r-1);
   pos = pos/27;
  }
  token[len] = 0;
  return token;
}

/* Aggiunge una frequenza alla tabella opportuna */
void FreqLang::add_freq(char *token, double freq) {
  int l = strlen(token);
  if (l == 0 || l > 3) return;
  if (list[l-1] == NULL) {
    long size = pow(27, l);
    list[l-1] = (double *)malloc(size*sizeof(double)+4);
    for (int i=0; i<size; i++) list[l-1][i] = 0.0;
  }
  list[l-1][calc_pos(token)] = freq;
}

void FreqLang::inc_freq(char *token, double amount) {
  int l = strlen(token);
  if (l == 0 || l > 3) return;
  if (list[l-1] == NULL) {
    long size = pow(27, l);
    list[l-1] = (double *)malloc(size*sizeof(double)+4);
    for (int i=0; i<size; i++) list[l-1][i] = 0.0;
  }
  list[l-1][calc_pos(token)] += amount;
}

void FreqLang::div_freqs_by(int table, double tot) {
  if (table < 0 || table > 2) return;
  if (list[table] != NULL) {
    long size = pow(27, table+1);
    for (int j=0; j<size; j++) {
      list[table][j] /= tot;
    }
  }
}

double FreqLang::get_freq(char *token) {
  int l = strlen(token);
  if (l == 0 || l > 3 || list[l-1] == NULL) return 0.0;
  return list[l-1][calc_pos(token)];
}
