/* CRYPTOGRAM */
/* FREQTABLE CLASS */

/* TODO:
 *  support for encodings, case, error checking
*/

#ifndef _FREQTABLE_H
#define _FREQTABLE_H

#include <stdio.h>
#include <ctype.h>
#include <string.h>

#include "freqlang.h"
#include "string2.h"

#define FREQT_ERR_NONE            0
#define FREQT_ERR_IOOPEN          1
#define FREQT_ERR_IOREAD          2
#define FREQT_ERR_NOMEM           3

#define FREQT_MAX_TABLES         10


/* Classe FreqTable */
/*   Gestisce le frequenze di varie lingue */
class FreqTable {

 public:
  FreqTable();
  ~FreqTable();
  int calc_freqs_from_file(const char *filename, const char *lang);
  int calc_freqs(const char *text, const char *lang);
  //void get_next_freq(int table, char *token, double *freq, const char *lang);
  FreqLang *get_langtable(const char *lang);
  int enable_case(int onfoff);

 private:
  char flag_enable_case;
  int langs;
  FreqLang *tables[FREQT_MAX_TABLES];

  int lang_exists(const char *lang);
  int lang_add(const char *lang);
};

// METODI ------------------------------------------------------------------

FreqTable::FreqTable() {
  flag_enable_case = 0;
  langs = 0;
  for (int i=0; i<FREQT_MAX_TABLES; i++) tables[i] = NULL;
}

FreqTable::~FreqTable() {
  for (int i=0; i<FREQT_MAX_TABLES; i++)
    if (!tables[i]) free(tables[i]);
}

/* NON IMPLEMENTATA */
/* Imposta il flag per abilitare la distinzione tra maiuscole e minuscole */
int FreqTable::enable_case(int onoff) {
  flag_enable_case = onoff;
  return FREQT_ERR_NONE;
}

FreqLang *FreqTable::get_langtable(const char *lang) {
  int i;
  if ((i=lang_exists(lang)) >= 0) {
    return tables[i];
  }
  return NULL;
}

/*void FreqTable::get_next_freq(int table, char *token, double *freq, const char *lang) {
  for (int i=0; i<langs; i++)
    if (strcmp(tables[i]->get_lang(), lang) == 0) tables[i]->get_next_freq(table, token, freq);
}*/

/* Calcola le frequenze da una stringa */
int FreqTable::calc_freqs(const char *text, const char *lang) {
  int i, langn;
  char ngram[5] = "\0\0\0\0";
  int counter[3] = {0,0,0};

  langn = lang_exists(lang);
  if (langn < 0) langn = lang_add(lang);
  for (i=0,strncpy(ngram, text, 4); ngram[0] != '\0'; i++,strncpy(ngram, text+i, 4)) {
    // eventuale conversione in minuscole
    if (!flag_enable_case) str_tolower(ngram);
    // converte i caratteri separatori in spazi
    str_multi_replace(".:,;!?\"'()[]", " ", ngram);
    char c[4] = {ngram[0], 0, 0, 0};
    // conteggio dei monogrammi
    if (isalpha(c[0]) || c[0] == ' ') {
      tables[langn]->inc_freq(c, 1.0);
      counter[0]++;
      // conteggio dei digrammi
      c[1] = ngram[1];
      if (isalpha(c[1]) || c[1] == ' ') {
        if (c[0] == ' ' && c[1] == ' ') continue;
        tables[langn]->inc_freq(c, 1.0);
        counter[1]++;
        // conteggio dei trigrammi
        c[2] = ngram[2];
        if (isalpha(c[2]) || c[2] == ' ') {
          if (c[1] == ' ' && c[2] == ' ') continue;
          tables[langn]->inc_freq(c, 1.0);
          counter[2]++;
        }
      }
    }
  }
  for (i=0; i<3; i++) {
    if (counter[i]) tables[langn]->div_freqs_by(i, counter[i]);
  }
  return FREQT_ERR_NONE;
}

/* Calcola le frequenze leggendo da un file di testo */
int FreqTable::calc_freqs_from_file(const char *filename, const char *lang) {
/* TODO:
 *   usare anche buffer più piccoli
*/

  // apre il file richiesto
  FILE *f = fopen(filename, "r");
  if (f == NULL) return FREQT_ERR_IOOPEN;

  // calcola la dimensione del buffer
  fseek(f, 0, SEEK_END);
  long size = ftell(f);
  rewind(f);

  // alloca la memoria
  char *buffer = (char *)malloc(size+16);
  if (buffer == NULL) {
    fclose(f);
    return FREQT_ERR_NOMEM;
  }

  // legge dal file
  if (fread(buffer, size, 1, f) != 1) {
    free(buffer);
    fclose(f);
    return FREQT_ERR_IOREAD;
  }
  fclose(f);

  // esegue il calcolo delle frequenze
  calc_freqs(buffer, lang);

  // libera le risorse e termina
  free(buffer);
  return FREQT_ERR_NONE;
}

// METODI PRIVATI ----------------------------------------------------------

/* Restituisce se la lingua specificata esiste */
int FreqTable::lang_exists(const char *lang) {
  for (int i=0; i<langs; i++) {
    if (tables[i] != NULL) {
      if (strcmp(tables[i]->get_lang(), lang) == 0) return i;
    }
  }
  return -1;
}

/* Aggiunge una lingua */
int FreqTable::lang_add(const char *lang) {
  if (langs >= FREQT_MAX_TABLES) return -1;
  tables[langs++] = new FreqLang(lang);
  return langs-1;
}

#endif
