/* DEMO1.H */

#define FLOWERS		3

#define M_PI_3		1.04719755119659774615
#define M_PI_6		0.52359877559829887308

#define D1_PATHLEN	2000
#define D1_RADLEN	200
#define D1_MAXRAD	130
#define D1_MAXDIST	(512+384)
#define D1_STRINGS	5

short d1_paths[FLOWERS][D1_PATHLEN][2];
unsigned char d1_rads[FLOWERS][D1_RADLEN];

double d1_dists[D1_MAXRAD*2][D1_MAXRAD*2];
double d1_rhos[D1_MAXRAD*2][D1_MAXRAD*2];
double d1_ks[D1_MAXRAD*2][D1_MAXRAD*2];
double d1_newrad[D1_MAXRAD*2][D1_MAXRAD*2];
double d1_reduce[D1_MAXDIST][D1_MAXRAD];

char *d1_text[D1_STRINGS] = {"Fools", "will", "be", "rewarded", ".!."};

VirtualScreenSDL *d1_virimg, *d1_flower;

void pre_demo1() {
  int i,j;
  double p = 0.50;
  double pathv[FLOWERS][8] = {
    {50.0,100.0, 80.0,70.0, 30.0,80.0, 40.0,60.0},
    {50.0,90.0, 90.0,40.0, 60.0,70.0, 30.0,60.0},   
    {20.0,100.0, 30.0,30.0, 40.0,90.0, 50.0,30.0} };
  double radv[FLOWERS][3] = {{8.0,50.0, 80}, {5.0,60.0, 70}, {3.0,40.0, 60}};
  
  for (j=0; j<FLOWERS; j++) {
    for (i=0; i<D1_PATHLEN; i++) {
      d1_paths[j][i][0] = (short)(sin(p*i/pathv[j][0])*pathv[j][1]+cos(p*i/pathv[j][2])*pathv[j][3])+(sizex>>1);
      d1_paths[j][i][1] = (short)(cos(p*i/pathv[j][4])*pathv[j][5]+cos(p*i/pathv[j][6]+M_PI_2*(j&1))*pathv[j][7])+(sizey>>1);
    }
    for (i=0; i<D1_RADLEN; i++) {
      // must not be greater than maxrad
      d1_rads[j][i] = (unsigned char)(sin(i/radv[j][0])*radv[j][1]+radv[j][2]);
    }
  }
  
  for (j=-D1_MAXRAD; j<D1_MAXRAD; j++) {
    for (i=-D1_MAXRAD; i<D1_MAXRAD; i++) {
      double d,r, nr;
      d = d1_dists[j+D1_MAXRAD][i+D1_MAXRAD] = sqrt(i*i+j*j);
      r = d1_rhos[j+D1_MAXRAD][i+D1_MAXRAD] = atan2(j,i);
      d1_ks[j+D1_MAXRAD][i+D1_MAXRAD] = d/fabs(sin(3.0*r));
      if (r < -2*M_PI_3) {
        nr = -5*M_PI_6;
      } else if (r < -M_PI_3) {
        nr = -M_PI_2;
      } else if (r < 0) {
        nr = -M_PI_6;
      } else if (r < M_PI_3) {
        nr = M_PI_6;
      } else if (r < 2*M_PI_3) {
        nr = M_PI_2;
      } else {
        nr = 5*M_PI_6;
      }
      d1_newrad[j+D1_MAXRAD][i+D1_MAXRAD] = nr;
    }
  }
  
  for (j=0; j<D1_MAXDIST; j++) {
    d1_reduce[j][0] = 0.0;
    for (i=1; i<D1_MAXRAD; i++) {
      d1_reduce[j][i] = exp(-((double)j*j)/(i*i));
    }
  }
  
  d1_virimg = new VirtualScreenSDL("lava.jpg");
  d1_virimg->SetDisplayFormat();
  d1_virimg->DrawMode = VS_DM_ADD;
  for (j=0; j<d1_virimg->GetHeight(); j++) {
    for (i=0; i<d1_virimg->GetWidth(); i++) {
      d1_virimg->PutPixel(i,j, 0x00202020);
    }
  }
  d1_virimg->DrawMode = VS_DM_NORMAL;
  d1_flower = new VirtualScreenSDL(D1_MAXRAD*2,D1_MAXRAD*2);
  d1_flower->SetDisplayFormat();
}

void demo1(unsigned int time, unsigned int maxtime) {
  int f, x,y, rad;
  int dx,dy, nx,ny, col;

  if (time > 9500) {
    virscr->Clear(0xffffffff);
    return;
  }
  
  d1_virimg->CopyTo(0,0, virscr);
  
  int pathpos = D1_PATHLEN*time/maxtime;
  int radpos = D1_RADLEN*time/maxtime;
  
  for (f=0; f<FLOWERS; f++) {
    x = d1_paths[f][pathpos][0];
    y = d1_paths[f][pathpos][1];
    rad = d1_rads[f][radpos];
    for (int j=y-rad; j<y+rad; j++) {
      dy = j-y;
      if (abs(dy) > rad) continue;
      for (int i=x-rad; i<x+rad; i++) {
        dx = i-x;
        nx = dx; ny = dy;
        if (abs(dx) <= rad) {
          double d = d1_dists[dy+D1_MAXRAD][dx+D1_MAXRAD];
          double r = d1_rhos[dy+D1_MAXRAD][dx+D1_MAXRAD];
          double k = d1_ks[dy+D1_MAXRAD][dx+D1_MAXRAD];
          if (k < rad) {
            double nr = d1_newrad[dy+D1_MAXRAD][dx+D1_MAXRAD];
            double reduce = d1_reduce[(int)d][rad]; 
            r = nr*reduce + r*(1-reduce);
            d = k*reduce + d*(1-reduce);
            double s,c;
            sincos(r, &s, &c);
            nx = (int)(d*c); ny = (int)(d*s);
          }
        }
        col = virscr->GetPixel(nx+x, ny+y);
        d1_flower->PutPixel(i-x+rad,j-y+rad, col);
      }
    }
    d1_flower->CopyRectTo(0,0, rad*2,rad*2, x-rad,y-rad, virscr);
  }
  
  static int curstring = 0;
  static int textsize = 20;
  static int alpha = 0xff;
  int oldstring = curstring;
  if (time < 1500) curstring = 0;
  else if (time < 2500) curstring = 1;
  else if (time < 3500) curstring = 2;
  else if (time < 4500) curstring = 3;
  else if (time < 5500) curstring = 4;

  if (oldstring != curstring) {
    textsize = 20;
    alpha = 0xff;
  }
  if (time > 500) {
    virscr->ReloadFont(1, "battlefield.ttf", textsize);
    textsize = 18*textsize/16;
    alpha = ((alpha-10)<0)?0:(alpha-10);
    virscr->TextAlign = VS_TA_CENTER;
    virscr->TextMode = VS_TM_SOLID;
    if (textsize < 400) virscr->PrintRect(1, -1000,100, sizex+1000,100+textsize*2, d1_text[curstring], 0xf08040 | (alpha<<24), 0);
  }
  
  static int lighter = 1;
  if (time > 7500) {
    int col = 0xff000000 | ((lighter<<16)&0xff0000) | ((lighter<<8)&0xff00) | (lighter&0xff);
    virscr->DrawMode = VS_DM_ADD;
    for (y=0; y<sizey; y++) {
      for (x=0; x<sizex; x++) {
        virscr->PutPixel(x,y, col);
      }
    }
    virscr->DrawMode = VS_DM_NORMAL;        
    lighter += 4;
  }
}

void end_demo1() {
  delete d1_virimg;
  delete d1_flower;
}
