#include <stdlib.h>
#include <math.h>

#include "cell.h"
#include "defs.h"

Cell::Cell():
  cx(0), cy(0), cvalue(MAP_NONE) {
}

Cell::Cell(char x, char y, int value):
  cx(x), cy(y), cvalue(value) {
}

Cell::Cell(const Cell& c):
  cx(c.cx), cy(c.cy), cvalue(c.cvalue) {
}

Cell& Cell::operator =(const Cell& c) {
  cx = c.cx;
  cy = c.cy;
  cvalue = c.cvalue;
  return *this;
}

/* Returns true if cell has ALL of items specified */
bool Cell::has(int items) const { 
  bool it = true;
  if (items & MAP_MASK) it = (cvalue & MAP_MASK) == (items & MAP_MASK);
  bool m = true;
  if (items & MAP_MARK) m = cvalue & MAP_MARK;
  bool w = true;
  if (items & MAP_WALLS) w = (items & MAP_WALLS) == (cvalue & items & MAP_WALLS);
  return it && m && w;
}


// Manhattan distance
int cell_dist1(const Cell& c1, const Cell& c2) {
  return abs(c1.x()-c2.x())+abs(c1.y()-c2.y());
}

// Euclidean distance
int cell_dist2(const Cell& c1, const Cell& c2) {
  return (int)hypot(c1.x()-c2.x(), c1.y()-c2.y());
}
