#include "labydisplay.h"
#include "defs.h"

#define CELL_SIZE	30

LabyDisplay::LabyDisplay() {
  int ret = SDL_Init(SDL_INIT_VIDEO);
  screen = NULL;
  virscr = NULL;
  if (ret == 0) {
    screen = SDL_SetVideoMode(400,480,32, SDL_HWSURFACE);
    SDL_WM_SetCaption("Dungeons&Spiders", "Dungeons&Spiders");
    virscr = new VirtualScreenSDL(screen);
    virscr->LoadFont("ARIAL.ttf", 12);
    virscr->TextMode = VS_TM_SOLID;
    virscr->Clear(0);
  }
}

LabyDisplay::~LabyDisplay() {
  if (virscr) delete virscr;
  if (screen) SDL_Quit();
}

void LabyDisplay::displaycell(const Cell& cell) const {
  const int wall_color = 0xb0b0b0;
  int x = cell.x()*CELL_SIZE+10, y = cell.y()*CELL_SIZE+10;
  if (cell.has(MAP_WALL_N)) virscr->Line(x,y, x+CELL_SIZE-1, y, wall_color);
  if (cell.has(MAP_WALL_S)) virscr->Line(x,y+CELL_SIZE-1, x+CELL_SIZE-1, y+CELL_SIZE-1, wall_color);
  if (cell.has(MAP_WALL_E)) virscr->Line(x+CELL_SIZE-1,y, x+CELL_SIZE-1, y+CELL_SIZE-1, wall_color);
  if (cell.has(MAP_WALL_W)) virscr->Line(x,y, x, y+CELL_SIZE-1, wall_color);
  
  switch (cell.value() & MAP_MASK) {
    case MAP_GOLD: virscr->FCircle(x+CELL_SIZE/2,y+CELL_SIZE/2, CELL_SIZE/4, 0xb09010); break;
    case MAP_SPIDER: 
      virscr->Line(x+6,y+6, x+CELL_SIZE-7,y+CELL_SIZE-7, 0x406040);
      virscr->Line(x+CELL_SIZE-7,y+6, x+6,y+CELL_SIZE-7, 0x406040);  
      break;
    case MAP_LIFE:
      virscr->FCircle(x+CELL_SIZE/2,y+CELL_SIZE/2, 3, 0x30d040);
      virscr->FCircle(x+CELL_SIZE/2-6,y+CELL_SIZE/2, 2, 0x30d040);
      virscr->FCircle(x+CELL_SIZE/2+6,y+CELL_SIZE/2, 2, 0x30d040);
      virscr->FCircle(x+CELL_SIZE/2,y+CELL_SIZE/2-6, 2, 0x30d040);
      virscr->FCircle(x+CELL_SIZE/2,y+CELL_SIZE/2+6, 2, 0x30d040);
      break;
    case MAP_LOCK:
      virscr->Circle(x+CELL_SIZE/2, y+CELL_SIZE/2, CELL_SIZE/3, 0x505080);
      virscr->Circle(x+CELL_SIZE/2, y+CELL_SIZE/2, CELL_SIZE/5, 0x505080);
      break;
    case MAP_KEY:
      virscr->Circle(x+6,y+CELL_SIZE/2, 3, 0x905050);
      virscr->Line(x+6,y+CELL_SIZE/2, x+2*CELL_SIZE/3,y+CELL_SIZE/2, 0x905050);
      break;
    case MAP_SHIELD:
      virscr->Triangle(x+6,y+6, x+CELL_SIZE-6,y+6, x+CELL_SIZE/2,y+CELL_SIZE-6, 0xb03060);
      break;
    case MAP_NOSHIELD:
      virscr->Triangle(x+8,y+8, x+CELL_SIZE-8,y+8, x+CELL_SIZE/2,y+CELL_SIZE-8, 0x909090);
      break;
    case MAP_ONEWAY_N:
      virscr->Line(x+CELL_SIZE/2,y+6, x+CELL_SIZE/2,y+CELL_SIZE-6, 0xd0f0d0);
      virscr->Line(x+CELL_SIZE/2,y+6, x+6,y+CELL_SIZE/2, 0xd0f0d0);
      virscr->Line(x+CELL_SIZE/2,y+6, x+CELL_SIZE-6,y+CELL_SIZE/2, 0xd0f0d0);
      break;
    case MAP_ONEWAY_S:
      virscr->Line(x+CELL_SIZE/2,y+6, x+CELL_SIZE/2,y+CELL_SIZE-6, 0xd0f0d0);
      virscr->Line(x+CELL_SIZE/2,y+CELL_SIZE-6, x+6,y+CELL_SIZE/2, 0xd0f0d0);
      virscr->Line(x+CELL_SIZE/2,y+CELL_SIZE-6, x+CELL_SIZE-6,y+CELL_SIZE/2, 0xd0f0d0);
      break;
    case MAP_ONEWAY_W:
      virscr->Line(x+6,y+CELL_SIZE/2, x+CELL_SIZE-6,y+CELL_SIZE/2, 0xd0f0d0);
      virscr->Line(x+6,y+CELL_SIZE/2, x+CELL_SIZE/2,y+6, 0xd0f0d0);
      virscr->Line(x+6,y+CELL_SIZE/2, x+CELL_SIZE/2,y+CELL_SIZE-6, 0xd0f0d0);
      break;
    case MAP_ONEWAY_E:
      virscr->Line(x+6,y+CELL_SIZE/2, x+CELL_SIZE-6,y+CELL_SIZE/2, 0xd0f0d0);
      virscr->Line(x+CELL_SIZE-6,y+CELL_SIZE/2, x+CELL_SIZE/2,y+6, 0xd0f0d0);
      virscr->Line(x+CELL_SIZE-6,y+CELL_SIZE/2, x+CELL_SIZE/2,y+CELL_SIZE-6, 0xd0f0d0);
      break;
  }
}

void LabyDisplay::display(const Labyrinth* laby, const Path *p) const {
  virscr->Clear(0);
  // Labyrinth
  for (int y=0; y<laby->get_size_y(); y++) {
    for (int x=0; x<laby->get_size_x(); x++) {
      displaycell(laby->get(x,y));
    }
  }
  // Path
  if (p && p->length() > 0) {
    const int path_color = 0x2090d0;
    const int padx = 2, pady = 2;
    Cell last = p->get(0);
    virscr->FCircle(10+padx+last.x()*CELL_SIZE+CELL_SIZE/2, 10+pady+last.y()*CELL_SIZE+CELL_SIZE/2, 3, path_color);
    for (int i=1; i<p->length(); i++) {
      Cell c = p->get(i);
      virscr->Line(10+padx+last.x()*CELL_SIZE+CELL_SIZE/2, 10+pady+last.y()*CELL_SIZE+CELL_SIZE/2,
                   10+padx+c.x()*CELL_SIZE+CELL_SIZE/2, 10+pady+c.y()*CELL_SIZE+CELL_SIZE/2, path_color);
      last = c;
    }
    virscr->FCircle(10+padx+last.x()*CELL_SIZE+CELL_SIZE/2, 10+pady+last.y()*CELL_SIZE+CELL_SIZE/2, 3, path_color);
    
    char tx[100];
    sprintf(tx, "Score: %i  -  Lives: %i", p->get_score(), p->get_lives());
    virscr->Print(10, 16+(laby->get_size_y())*CELL_SIZE, tx, 0xffffffff, 0);
  }
  
  virscr->Show();
}
