#include <string.h>
#include <stdio.h>
#include "labyrinth.h"
#include "defs.h"

Labyrinth::Labyrinth(const char *filename) {
  sizex = sizey = startx = starty = endx = endy = 0;
  teleports = NULL;
  read_from_file(filename); 
}

Labyrinth::Labyrinth(const Labyrinth& l) {
  sizex = l.sizex;
  sizey = l.sizey;
  startx = l.startx;
  starty = l.starty;
  endx = l.endx;
  endy = l.endy;
  map = (Cell*)malloc(sizeof(Cell)*sizex*sizey);
  memcpy(map, l.map, sizeof(Cell)*sizex*sizey);
  teleports = new Path(*(l.teleports));
}

Labyrinth::~Labyrinth() {
  if (map) free(map);
  if (teleports) delete teleports;
}

Cell Labyrinth::get(int x, int y) const {
  if (x >= 0 && x < sizex && y >= 0 && y < sizey)
    return *(map+y*sizex+x);
  return Cell(0,0, MAP_INVALID);
}

void Labyrinth::set(int x, int y, const Cell &value) {
  if (x >= 0 && x < sizex && y >= 0 && y < sizey)
    *(map+y*sizex+x) = value;
}

void Labyrinth::mark(int x, int y) {
  if (x >= 0 && x < sizex && y >= 0 && y < sizey)
    (map+y*sizex+x)->mark();
}

void Labyrinth::unmark(int x, int y) {
  if (x >= 0 && x < sizex && y >= 0 && y < sizey)
    (map+y*sizex+x)->unmark();
}


/*
int Labyrinth::print(const Path *p) const {
  // use escape seq
  char s[16],pc[16],pc2[16], pc3[16];
  char s1[sizex*10+2], s2[sizex*40+2], s3[sizex*3+2];
  for (int j=0; j<sizey; j++) {
    s1[0] = s2[0] = s3[0] = '\0';
    for (int i=0; i<sizex; i++) {
      int v = get(i,j);
      switch (v & MAP_MASK) {
        case MAP_GOLD: strcpy(s,"\x1B[33mG\x1B[0m"); break;
        case MAP_SPIDER: strcpy(s,"\x1B[41mS\x1B[0m"); break;
        case MAP_KEY: strcpy(s,"\x1B[37mk\x1B[0m"); break;
        case MAP_LOCK: strcpy(s,"\x1B[44mL\x1B[0m"); break;
        case MAP_LIFE: strcpy(s,"\x1B[35mf\x1B[0m"); break;
        default: s[0] = ' '; s[1] = '\0'; break;
      }
      pc[0] = ' '; pc[1] = '\0';
      pc2[0] = ' '; pc2[1] = '\0';
      pc3[0] = (v&MAP_WALL_N)?'-':' '; pc3[1] = '\0';
      // "─ │ ┌ ┐ └ ┘" = 'e2 94 80'   'e2 94 82'   'e2 94 8c'  'e2 94 90'   'e2 94 94'   'e2 94 98'
      int x1,y1, x2,y2;
      if (p && p->is_in(i,j, &x1,&y1, &x2,&y2)) {
        strcpy(pc,"\x1B[36m\xe2\x94\x80\x1b[0m");
        if (x1 > x2) { int t=x1; x1=x2; x2=t; t=y1; y1=y2; y2=t; }
        if (x1 == x2 && y1 > y2) { int t=x1; x1=x2; x2=t; t=y1; y1=y2; y2=t; }
        bool pn = ((x1==i)&&(y1==(j-1))) || ((x2==i)&&(y2==(j-1)));
        bool ps = ((x1==i)&&(y1==(j+1))) || ((x2==i)&&(y2==(j+1)));
        bool pw = ((y1==j)&&(x1==(i-1))) || ((y2==j)&&(x2==(i-1)));
        bool pe = ((y1==j)&&(x1==(i+1))) || ((y2==j)&&(x2==(i+1)));
        if (pn && ps) { pc[7] = '\x82'; strcpy(pc3, "\x1B[36m\xe2\x94\x82\x1b[0m"); }
        if (pw && pe) { pc[7] = '\x80'; strcpy(pc2, "\x1B[36m\xe2\x94\x80\x1b[0m"); }
        if (ps && pe) pc[7] = '\x8c';
        if (ps && pw) { pc[7] = '\x90'; strcpy(pc2, "\x1B[36m\xe2\x94\x80\x1b[0m"); }
        if (pn && pe) { pc[7] = '\x94'; strcpy(pc3, "\x1B[36m\xe2\x94\x82\x1b[0m"); }
        if (pn && pw) { 
          pc[7] = '\x98'; strcpy(pc2, "\x1B[36m\xe2\x94\x80\x1b[0m");
          strcpy(pc3, "\x1B[36m\xe2\x94\x82\x1b[0m");
        }
      }
      sprintf(s1, "%s+%s%s", s1, (v&MAP_WALL_N)?"--":"  ", pc3);
      sprintf(s2, "%s%c%s%s%s", s2, (v&MAP_WALL_W)?'|':' ', pc2,s,pc);
    }
    printf("%s+\n%s|\n", s1,s2,s3);
  }
  return 0;
}
*/

int Labyrinth::read_from_file(const char *filename) {
  FILE *f = fopen(filename, "r");
  if (!f) return 1;
  fscanf(f, "%i %i\n", &sizex, &sizey);
  fscanf(f, "%i %i\n", &startx, &starty);
  fscanf(f, "%i %i\n", &endx, &endy);
  if (map) free(map);
  map = (Cell*)malloc(sizeof(Cell)*sizex*sizey);
  if (teleports) teleports->clear(); else teleports = new Path();
  for (int i=0; i<sizex*sizey; i++) {
    int r;
    fscanf(f, "%i", &r);
    Cell c = Cell(i%sizex, i/sizex, r);
    *(map+i) = c;
    if (c.has(MAP_TELEPORT)) teleports->add_tail(c);
  }
  fclose(f);
  return 0;
}
