#include <string.h>
#include "path.h"
#include "defs.h"

Path::Path(): 
  score(0), lives(0), len(0), cells_alloc(16), cells(NULL) {
}

Path::Path(const Path& p):
  score(p.score), lives(p.lives), len(p.len), cells_alloc(p.cells_alloc) {
  cells = (Cell *)malloc(sizeof(Cell)*cells_alloc*(len/cells_alloc+1));
  memcpy(cells, p.cells, sizeof(Cell)*cells_alloc*(len/cells_alloc+1));
}

Path::~Path() {
  if (cells) free(cells);
}

Path& Path::operator =(const Path& p) {
  score = p.score;
  lives = p.lives;
  len = p.len;
  cells_alloc = p.cells_alloc;
  cells = (Cell *)realloc(cells, sizeof(Cell)*cells_alloc*(len/cells_alloc+1));
  memcpy(cells, p.cells, sizeof(Cell)*cells_alloc*(len/cells_alloc+1));
  return *this;
}

Cell Path::get(int i) const {
  if (i >= 0 && i < len) return cells[i];
  return Cell(0,0, MAP_INVALID);
}

void Path::add_tail(const Cell& c) {
  if (len % cells_alloc == 0) cells = (Cell*)realloc(cells, sizeof(Cell)*(cells_alloc+len));
  cells[len++] = c;
  //if (c.has(MAP_GOLD)) score += 1;
  //if (c.has(MAP_LIFE)) ++lives;
}

void Path::del_tail() {
  if (len > 0) {
    if ((--len % cells_alloc) == 0) cells = (Cell*)realloc(cells, sizeof(Cell)*len);
  }
}

int Path::value() const {
  const int weight_len = 0;
  const int weight_gold = 2;
  const int weight_lives = 1;
  return weight_len*len + weight_gold*score + weight_lives*lives;
}

bool Path::is_in(const Cell& c, Cell *prev, Cell *next) const {
  for (int i=0; i<len; i++) 
    if (cells[i].x() == c.x() && cells[i].y() == c.y()) {
      if (prev && i>0) *prev = cells[i-1];
      if (next && i<(len-1)) *next = cells[i+1];
      return true;
    }
  return false;
}

void Path::clear() {
  if (cells) free(cells);
  cells = NULL;
  len = 0;
}
