#include "player.h"
#include "defs.h"

Player::Player(Labyrinth *l): laby(l) {
  posx = posy = 0;
  score = PLAYER_INIT_SCORE;
  lives = PLAYER_INIT_LIVES;
  keys = shield = 0;
  path = new Path();
  lastpath = new Path();
  ld = new LabyDisplay();
  if (l) {
    posx = l->get_start_x();
    posy = l->get_start_y();
  }
}

Player::~Player() {
  if (path) delete path;
  if (lastpath) delete lastpath;
  if (ld) delete ld;
}

bool Player::at_begin() const {
  if (laby) return (posx == laby->get_start_x()) && (posy == laby->get_start_y());
  return false;
}

bool Player::at_end() const {
  if (laby) return (posx == laby->get_end_x()) && (posy == laby->get_end_y());
  return false;
}

bool Player::can_go(int dir) const {
  // todo: jump, letters, breakwall, teleport
  if (laby) {
    Cell c,v;
    v = laby->get(posx,posy);
    bool ow = (v.has(MAP_ONEWAY_N) || v.has(MAP_ONEWAY_S) || v.has(MAP_ONEWAY_E) || v.has(MAP_ONEWAY_W));
    switch (dir) {
      case PLAYER_MOVE_UP: {
        if (posy <= 0 || (v.has(MAP_WALL_N))) return false;
        if (ow && !v.has(MAP_ONEWAY_N)) return false;
        c = laby->get(posx,posy-1);
        break;
      }
      case PLAYER_MOVE_DOWN: {
        if (posy >= (laby->get_size_y()-1) || (v.has(MAP_WALL_S))) return false;
        if (ow && !v.has(MAP_ONEWAY_S)) return false;
        c = laby->get(posx,posy+1);
        break;
      }
      case PLAYER_MOVE_LEFT: {
        if (posx <= 0 || (v.has(MAP_WALL_W))) return false;
        if (ow && !v.has(MAP_ONEWAY_E)) return false;
        c = laby->get(posx-1,posy);
        break;
      }
      case PLAYER_MOVE_RIGHT: {
        if (posx >= (laby->get_size_x()-1) || (v.has(MAP_WALL_E))) return false;
        if (ow && !v.has(MAP_ONEWAY_W)) return false;
        c = laby->get(posx+1,posy);
        break;
      }
      default: return false;
    }
    if (c.marked()) return false;
    if (shield == 0 && c.has(MAP_SPIDER) && lives <= 1) return false;
    if (c.has(MAP_LOCK) && keys < 3) return false;
    return true;
  }
  return false;
}

void Player::walk() {

  if (!laby) return;
  /*if (path->length() == 0) {
    // first run
    path->set_score(0);
    path->set_lives(5);
  }*/
  
  Cell cell = laby->get(posx, posy);
  
  laby->mark(posx, posy);
  path->add_tail(cell);
  
  int oldlives = lives;
  int oldkeys = keys;
  int oldscore = score;
  int oldshield = shield;
  if (cell.has(MAP_SPIDER) && !shield) --lives;
  if (cell.has(MAP_LIFE)) ++lives;
  if (cell.has(MAP_KEY)) ++keys;
  if (cell.has(MAP_GOLD)) score += 1;
  if (cell.has(MAP_SHIELD)) shield = 1;
  if (cell.has(MAP_NOSHIELD)) shield = 0;
  
  
  if (at_end()) {
    path->set_score(score);
    path->set_lives(lives);
    if (lastpath->value() < path->value()) {
      *lastpath = *path;
      ld->display(laby, path);
    }

  } else {
    // todo: lista di celle disponibili
    if (can_go_up()) { --posy; walk(); ++posy; }
    if (can_go_right()) { ++posx; walk(); --posx; }
    if (can_go_down()) { ++posy; walk(); --posy;}
    if (can_go_left()) { --posx; walk(); ++posx; }
    
  }
  
  lives = oldlives;
  keys = oldkeys;
  score = oldscore;
  shield = oldshield;

  path->del_tail();
  laby->unmark(posx, posy);
}
