/* PROJECTOR CLASS */

#include "class_projector.h"
#include "pointf.cpp"

Projector::Projector() {
  me.x = me.y = me.z = 0;
  eye.x = 0;
  eye.y = 0;
  eye.z = -100;
  unitx = unity = 10;
  sizex = 640;
  sizey = 480;
  enableBackPlane = 1;
  calcPlane();
}

Projector::Projector(Point *me, Point *core) {
  this->me = *me;
  this->eye = *core;
  sizex = 640;
  sizey = 480;
  unitx = unity = 10;
  enableBackPlane = 1;
  calcPlane();
}

void Projector::setMe(Point *m) {
  me = *m;
  calcPlane();
}

void Projector::setMe(PROJECTOR_TYPE x, PROJECTOR_TYPE y, PROJECTOR_TYPE z) {
  me.x = x; me.y = y; me.z = z;
  calcPlane();
}

void Projector::moveMe(Point *m) {
  me += *m;
  eye += *m;
}

void Projector::moveMe(PROJECTOR_TYPE x, PROJECTOR_TYPE y, PROJECTOR_TYPE z) {
  me.x += x;
  me.y += y;
  me.z += z;
  eye.x += x;
  eye.y += y;
  eye.z += z;
}

void Projector::moveMeRel(PROJECTOR_TYPE x, PROJECTOR_TYPE y, PROJECTOR_TYPE z) {
  Point delta;
  delta.x = z*dirz.x + x*dirx.x + y*diry.x;
  delta.y = z*dirz.y + x*dirx.y + y*diry.y;
  delta.z = z*dirz.z + x*dirx.z + y*diry.z;
  me += delta;
  eye += delta;
}

inline void Projector::setOrigin(Point *o) {
  eye = *o;
  calcPlane();
}

inline void Projector::setOrigin(PROJECTOR_TYPE x, PROJECTOR_TYPE y, PROJECTOR_TYPE z) {
  eye.x = x; eye.y = y; eye.z = z;
  calcPlane();
}

void Projector::moveOriginCloser(PROJECTOR_TYPE amount) {
  eye += dirz*amount;
  eyedist -= amount;
}

PROJECTOR_TYPE Projector::calcDistMe(Point *p) {
  return (me-*p).length();
}

PROJECTOR_TYPE Projector::calcDistMe(PROJECTOR_TYPE x, PROJECTOR_TYPE y, PROJECTOR_TYPE z) {
  Point t = Point(me.x-x, me.y-y, me.z-z);
  return t.length();
}

PROJECTOR_TYPE Projector::calcDistOrigin(Point p) {
  return (eye-p).length();
}

PROJECTOR_TYPE Projector::calcDistOrigin(Point *p) {
  return (eye-*p).length();
}

void Projector::calcMxRot(PROJECTOR_TYPE alpha, PROJECTOR_TYPE beta, PROJECTOR_TYPE gamma) {
  double s[3], c[3]; // sine/cosine tables
  static PROJECTOR_TYPE olda = 0, oldb = 0, oldc = 0;
  if (alpha == olda && beta == oldb && gamma == oldc) return;
  olda = alpha;
  oldb = beta;
  oldc = gamma;
  #ifdef _GNU_SOURCE
    sincos(alpha, s+0, c+0);
    sincos(beta, s+1, c+1);
    sincos(gamma, s+2, c+2);
  #else
    s[0] = sin(alpha); c[0] = cos(alpha);
    s[1] = sin(beta); c[1] = cos(beta);
    s[2] = sin(gamma); c[2] = cos(gamma);
  #endif
  mxRot[0][0] = c[1]*c[2];
  mxRot[0][1] = -(s[0]*s[1]*c[2]+c[0]*s[2]);
  mxRot[0][2] = s[0]*s[2]-c[0]*s[1]*c[2];
  mxRot[0][3] = c[1]*s[2];
  mxRot[0][4] = c[0]*c[2]-s[0]*s[1]*s[2];
  mxRot[0][5] = -(c[0]*s[1]*s[2]+s[0]*c[2]);
  mxRot[0][6] = s[1];
  mxRot[0][7] = s[0]*c[1];
  mxRot[0][8] = c[0]*c[1];
}

void Projector::rotateMe(Point *center, PROJECTOR_TYPE alpha, PROJECTOR_TYPE beta, PROJECTOR_TYPE gamma) {
  double rx,ry,rz;

  calcMxRot(alpha, beta, gamma);

  if (me.x != center->x || me.y != center->y || me.z != center->z) {
    rx = me.x - center->x;
    ry = me.y - center->y;
    rz = me.z - center->z;
    me.x = rx*mxRot[0][0] + ry*mxRot[0][1] + rz*mxRot[0][2] + center->x;
    me.y = rx*mxRot[0][3] + ry*mxRot[0][4] + rz*mxRot[0][5] + center->y;
    me.z = rx*mxRot[0][6] + ry*mxRot[0][7] + rz*mxRot[0][8] + center->z;
  }
  if (eye.x != center->x || eye.y != center->y || eye.z != center->z) {
    rx = eye.x - center->x;
    ry = eye.y - center->y;
    rz = eye.z - center->z;
    eye.x = rx*mxRot[0][0] + ry*mxRot[0][1] + rz*mxRot[0][2] + center->x;
    eye.y = rx*mxRot[0][3] + ry*mxRot[0][4] + rz*mxRot[0][5] + center->y;
    eye.z = rx*mxRot[0][6] + ry*mxRot[0][7] + rz*mxRot[0][8] + center->z;
  }
  rx = dirx.x;
  ry = dirx.y;
  rz = dirx.z;
  dirx.x = rx*mxRot[0][0] + ry*mxRot[0][1] + rz*mxRot[0][2];
  dirx.y = rx*mxRot[0][3] + ry*mxRot[0][4] + rz*mxRot[0][5];
  dirx.z = rx*mxRot[0][6] + ry*mxRot[0][7] + rz*mxRot[0][8];
  rx = diry.x;
  ry = diry.y;
  rz = diry.z;
  diry.x = rx*mxRot[0][0] + ry*mxRot[0][1] + rz*mxRot[0][2];
  diry.y = rx*mxRot[0][3] + ry*mxRot[0][4] + rz*mxRot[0][5];
  diry.z = rx*mxRot[0][6] + ry*mxRot[0][7] + rz*mxRot[0][8];
  rx = dirz.x;
  ry = dirz.y;
  rz = dirz.z;
  dirz.x = rx*mxRot[0][0] + ry*mxRot[0][1] + rz*mxRot[0][2];
  dirz.y = rx*mxRot[0][3] + ry*mxRot[0][4] + rz*mxRot[0][5];
  dirz.z = rx*mxRot[0][6] + ry*mxRot[0][7] + rz*mxRot[0][8];
}

void Projector::calcMxRot(Point *axis, PROJECTOR_TYPE alpha, int nmatrix) {
  double s, c;
  if (alphas[nmatrix] == alpha) return;
  #ifdef _GNU_SOURCE
    sincos(alpha, &s, &c);
  #else
    s = sin(alpha); c = cos(alpha);
  #endif
  alphas[nmatrix] = alpha;
  mxRot[nmatrix][0] = c + (1-c)*axis->x*axis->x;
  mxRot[nmatrix][1] = (1-c)*axis->x*axis->y - s*axis->z;
  mxRot[nmatrix][2] = (1-c)*axis->x*axis->z + s*axis->y;
  mxRot[nmatrix][3] = (1-c)*axis->y*axis->x + s*axis->z;
  mxRot[nmatrix][4] = c + (1-c)*axis->y*axis->y;
  mxRot[nmatrix][5] = (1-c)*axis->y*axis->z - s*axis->x;
  mxRot[nmatrix][6] = (1-c)*axis->z*axis->x - s*axis->y;
  mxRot[nmatrix][7] = (1-c)*axis->z*axis->y + s*axis->x;
  mxRot[nmatrix][8] = c + (1-c)*axis->z*axis->z;
}

void Projector::rotateMeRel(PROJECTOR_TYPE alpha, PROJECTOR_TYPE beta, PROJECTOR_TYPE gamma) {
  PROJECTOR_TYPE rx,ry,rz;
  PROJECTOR_TYPE betas[3] = {alpha, beta, gamma};
  Point *dirs[3] = {&dirx, &diry, &dirz};
  
  for (int i=0; i<3; i++) {
    if (betas[i] == 0) continue;
    calcMxRot(dirs[i], betas[i], i);
    rx = dirx.x;
    ry = dirx.y;
    rz = dirx.z;
    dirx.x = rx*mxRot[i][0] + ry*mxRot[i][1] + rz*mxRot[i][2];
    dirx.y = rx*mxRot[i][3] + ry*mxRot[i][4] + rz*mxRot[i][5];
    dirx.z = rx*mxRot[i][6] + ry*mxRot[i][7] + rz*mxRot[i][8];
    rx = diry.x;
    ry = diry.y;
    rz = diry.z;
    diry.x = rx*mxRot[i][0] + ry*mxRot[i][1] + rz*mxRot[i][2];
    diry.y = rx*mxRot[i][3] + ry*mxRot[i][4] + rz*mxRot[i][5];
    diry.z = rx*mxRot[i][6] + ry*mxRot[i][7] + rz*mxRot[i][8];
    rx = dirz.x;
    ry = dirz.y;
    rz = dirz.z;
    dirz.x = rx*mxRot[i][0] + ry*mxRot[i][1] + rz*mxRot[i][2];
    dirz.y = rx*mxRot[i][3] + ry*mxRot[i][4] + rz*mxRot[i][5];
    dirz.z = rx*mxRot[i][6] + ry*mxRot[i][7] + rz*mxRot[i][8];
  }

  eye = me-dirz*eyedist;
}

void Projector::calcPlane() {
  dirz = me-eye;
  eyedist = sqrt(dirz.x*dirz.x+dirz.y*dirz.y+dirz.z*dirz.z);
  dirz /= eyedist;
   
  dirx.x = dirx.y = dirx.z = 0;
  diry.x = diry.y = diry.z = 0;
  if (dirz.z != 0) {
    dirx.x = dirz.z;
    dirx.z = -dirz.x;
    diry.y = dirz.z;
    diry.z = -dirz.y;
  } else if (dirz.y != 0) {
    dirx.x = dirz.y;
    dirx.y = -dirz.x;
    diry.y = dirz.z;
    diry.z = -dirz.y;
  } else {
    dirx.x = dirz.y;
    dirx.y = -dirz.x;
    diry.x = dirz.z;
    diry.z = -dirz.x;
  }
}

int Projector::trace(Point p, int *posx, int *posy) {
  trace(&p, posx, posy);
}

int Projector::trace(Point *p, int *posx, int *posy) {
  Point pro, i1, t1,t2;
  PROJECTOR_TYPE x,y, a;
  
  // inters piano e retta eye-p
  PROJECTOR_TYPE tp = dirz * (eye-me);
  if (tp == 0) return(-1);
  PROJECTOR_TYPE t = (dirz * (*p-me))/tp;
  if (t == 1) return(-2);
  pro =  (*p - eye*t)/(1-t);

  i1 = pro-me;

  // controlla se il punto è dietro a plane
  Point i2 = *p - eye;
  if (i2.x != 0) {
    a = (pro.x-eye.x)/i2.x;
  } else if (i2.y != 0) {
    a = (pro.y-eye.y)/i2.y;
  } else {
    a = (pro.z-eye.z)/i2.z;
  }
  if (a <= 0) return(-3); // skip if behind eye
  if (!enableBackPlane)
    if (1 <= a) return(-4); // skip if behind plane

  x = (i1*dirx)/unitx;
  y = -(i1*diry)/unity;
 
  // scala x,y
  *posx = (int)(sizex*(x+unitx)/(unitx*2));
  *posy = (int)(sizey*(y+unity)/(unity*2));
  return(0);
}
