/* PROJECTOR CLASS (header) */

#ifndef CLASS_PROJECTOR_H
#define CLASS_PROJECTOR_H

#include <math.h>

#define PROJECTOR_TYPE		double

/* Definitions and declarations for Point */
#include "point.h"

class Projector {
 public:
  char enableBackPlane;

  Projector();
  Projector(Point *me, Point *core);

  void setSizes(int x, int y) { sizex = x; sizey = y; }

  void setMe(Point *m);
  void setMe(PROJECTOR_TYPE x, PROJECTOR_TYPE y, PROJECTOR_TYPE z);

  Point *getMe() { return( &me ); }
  Point *getOrigin() { return( &eye ); }

  void moveMe(Point *m);
  void moveMe(PROJECTOR_TYPE x, PROJECTOR_TYPE y, PROJECTOR_TYPE z);
  void moveMeRel(PROJECTOR_TYPE x, PROJECTOR_TYPE y, PROJECTOR_TYPE z);

  void setOrigin(Point *o);
  void setOrigin(PROJECTOR_TYPE x, PROJECTOR_TYPE y, PROJECTOR_TYPE z);
  void moveOriginCloser(PROJECTOR_TYPE amount);

  void setUnits(PROJECTOR_TYPE ux, PROJECTOR_TYPE uy) { unitx = ux; unity = uy; }
  void getUnits(PROJECTOR_TYPE *ux, PROJECTOR_TYPE *uy) { *ux = unitx; *uy = unity; }
  void moveUnits(PROJECTOR_TYPE ux, PROJECTOR_TYPE uy) { unitx += ux; unity += uy; }

  PROJECTOR_TYPE calcDistMe(Point *p);
  PROJECTOR_TYPE calcDistMe(PROJECTOR_TYPE x, PROJECTOR_TYPE y, PROJECTOR_TYPE z);
  PROJECTOR_TYPE calcDistOrigin(Point *p);
  PROJECTOR_TYPE calcDistOrigin(Point p);

  void rotateMe(Point *center, PROJECTOR_TYPE alpha, PROJECTOR_TYPE beta, PROJECTOR_TYPE gamma);
  void rotateMeRel(PROJECTOR_TYPE alpha, PROJECTOR_TYPE beta, PROJECTOR_TYPE gamma);

  int trace(Point *p, int *x, int *y);
  int trace(Point p, int *x, int *y);
  
 private:
  Point me, eye;
  Point dirx, diry, dirz;  // versori dell'occhio
  PROJECTOR_TYPE eyedist, unitx,unity;
  int sizex, sizey;
  
  PROJECTOR_TYPE mxRot[3][9];
  PROJECTOR_TYPE alphas[3];

  void calcPlane();
  void calcMxRot(PROJECTOR_TYPE alpha, PROJECTOR_TYPE beta, PROJECTOR_TYPE gamma);
  void calcMxRot(Point *axis, PROJECTOR_TYPE alpha, int nmatrix);

};

#endif
