/* CLASS VIRTUALSCREEN */

#include <stdlib.h>
#include "class_virscr.h"

#define max(a,b)	(((a)>(b))?(a):(b))
#define min(a,b)	(((a)<(b))?(a):(b))

int VirtualScreen::Line(int x1, int y1, int x2, int y2, int color) {
  int dx = x2-x1, dy = y2-y1;
  int ix = abs(dx), iy = abs(dy),
      inct = (ix>iy?ix:iy),
      plotX = x1, plotY = y1,
      x = 0, y = 0,
      i;
  char plot;

  PutPixel(plotX,plotY, color);
  for (i=0;i<=inct;i++) {
    x += ix;
    y += iy;
    plot = 0;
    if (x > inct) {
      plot = 1;
      x -= inct;
      if (dx > 0) plotX++;
      else if (dx < 0) plotX--;
    }
    if (y > inct) {
      plot = 1;
      y -= inct;
      if (dy > 0) plotY++;
      else if (dy < 0) plotY--;
    }
    if (plot) PutPixel(plotX,plotY, color);
  }
  return(0);
}

int VirtualScreen::Rect(int x1, int y1, int x2, int y2, int color) {
  Line(x1,y1, x2,y1, color);
  Line(x1,y2, x2,y2, color);
  Line(x1,y1, x1,y2, color);
  Line(x2,y1, x2,y2, color);
}

int VirtualScreen::Triangle(int x1, int y1, int x2, int y2, int x3, int y3, int color) {
  Line(x1,y1, x2,y2, color);
  Line(x2,y2, x3,y3, color);
  Line(x3,y3, x1,y1, color);
}

int VirtualScreen::Circle(int x, int y, int rad, int color) {
  register int d, nx,ny;
  int x1,x2,x3,x4, y1,y2,y3,y4;
  d = 3 - (rad<<1);
  nx = 0;
  ny = rad;
  x1 = x+nx;  x2 = x-nx;  x3 = x+ny;  x4 = x-ny;
  y1 = y+ny;  y2 = y-ny;  y3 = y+nx;  y4 = y-nx;
  while (nx <= ny) {
    PutPixel(x1, y1, color);
    PutPixel(x1, y2, color);
    if (x2 != x1) {
      PutPixel(x2, y1, color);
      PutPixel(x2, y2, color);
    }
    if (y1 != y3) {
      PutPixel(x3, y3, color);
      PutPixel(x4, y3, color);
      if (y3 != y4) {
        PutPixel(x3, y4, color);
        PutPixel(x4, y4, color);
      }
    }
    nx++;
    x1++;  x2--;
    y3++;  y4--;
    if (d < 0) {
      d += (nx<<2)+6;
    } else {
      d += ((nx-ny--)<<2)+10;
      x3--;  x4++;
      y1--;  y2++;
    }
  }
  return(0);
}

int VirtualScreen::FRect(int x1, int y1, int x2, int y2, int color) {
  if (y1 > y2) {
    int t = y2; y2 = y1; y1 = t;
    t = x2; x2 = x1; x1 = t;
  }
  for (int i=y1; i<=y2; i++) Line(x1,i, x2,i, color); 
}

// !!--Experimental--!!
int VirtualScreen::FTriangle(int x1, int y1, int x2, int y2, int x3, int y3, int color) {

  // y1 must not be aligned with others y
  if (y1 == y2 && y1 == y3) { 
    return Line(min(x1,min(x2,x3)),y1, max(x1,max(x2,x3)),y2, color);
  } else if (y1 == y2) {
    int a,b;
    a = x1; b = y1;
    x1 = x3; y1 = y3;
    x3 = a; y3 = b;
  } else if (y1 == y3) {
    int a,b;
    a = x1; b = y1;
    x1 = x2; y1 = y2;
    x2 = a; y2 = b;
  }
  
  if (x1 < 0 && x2 < 0 && x3 < 0) return(-1);
  if (x1 > GetWidth() && x2 > GetWidth() && x3 > GetWidth()) return(-1);
  
  if ((y1 > y2 && y1 < y3)||(y1 > y3 && y1 < y2)) {
    int a,b;
    a = x1; b = y1;
    x1 = x2; y1 = y2;
    x2 = a; y2 = b;
  }

  int dx[3] = {x2-x1, x3-x1, x3-x2},
      dy[3] = {y2-y1, y3-y1, y3-y2},
      plotX[3] = {x1,x1,x2},
      plotY[3] = {y1,y1,y2};

  if (y1 < 0) {
    if (y2 < 0 && y3 < 0) return(-1);
    /* TODO */
  }
  if (y1 > GetHeight()) {
    if (y2 > GetHeight() && y3 > GetHeight()) return(-1);
  }
  
  int ix[3] = {abs(dx[0]), abs(dx[1]), abs(dx[2])},
      iy[3] = {abs(dy[0]), abs(dy[1]), abs(dy[2])},
      inct[3] = {max(ix[0],iy[0]), max(ix[1],iy[1]), max(ix[2],iy[2])},
      x[3] = {0,0,0},
      y[3] = {0,0,0},
      addx[3] = {0,0,0},
      addy[3] = {0,0,0},
      ylimit0 = iy[0],
      ylimit1 = iy[1],
      putx1,puty1, putx2,puty2,
      i,j;
  char drawn = 0, goy[2] = {1,1};
  register int idx0 = 0, idx1 = 1;

  if (iy[0]>iy[1]) {
    dx[2] = -dx[2];
    dy[2] = -dy[2];
    plotX[2] = x3;
    plotY[2] = y3;
    ylimit0 = iy[1];
    ylimit1 = iy[0];
  }

  for (i=0; i<3; i++) {
    if (dx[i] > 0) addx[i]++;
    else if (dx[i] < 0) addx[i]--;
    if (dy[i] > 0) addy[i]++;
    else if (dy[i] < 0) addy[i]--;
  }

  putx1 = plotX[idx0];
  putx2 = plotX[idx1];
  int newlen, oldhlen = -1;
  for (i=0; i<ylimit1; ) {
    if (goy[0]) {
      x[idx0] += ix[idx0];
      y[idx0] += iy[idx0];
      if (x[idx0] > inct[idx0]) {
        x[idx0] -= inct[idx0];
        plotX[idx0] += addx[idx0];
        newlen = abs(plotX[idx0]-putx2);
        if (newlen > oldhlen) {
          putx1 = plotX[idx0];
          oldhlen = newlen;
        }
      }
      if (y[idx0] > inct[idx0]) {
        //oldhlen = -1;
        puty1 = plotY[idx0];
        y[idx0] -= inct[idx0];
        plotY[idx0] += addy[idx0];
        goy[0] = 0;
      }
    }
    
    if (goy[1]) {
      x[idx1] += ix[idx1];
      y[idx1] += iy[idx1];
      if (x[idx1] > inct[idx1]) {
        x[idx1] -= inct[idx1];
        plotX[idx1] += addx[idx1];
        newlen = abs(plotX[idx1]-putx1);
        if (newlen > oldhlen) {
          putx2 = plotX[idx1];
          oldhlen = newlen;
        }
        //putx2 = plotX[idx1];
      }
      if (y[idx1] > inct[idx1]) {
        //oldhlen = -1;
        puty2 = plotY[idx1];
        y[idx1] -= inct[idx1];
        plotY[idx1] += addy[idx1];
        goy[1] = 0;
      }
    }

    if (!goy[0] && !goy[1]) {
      oldhlen = -1;
      goy[0] = goy[1] = 1;
      int r = Line(putx1,puty1, putx2,puty2, color);
      if (drawn && r) return(0);
      if (!r) drawn = 1;
      putx1 = plotX[idx0];
      putx2 = plotX[idx1];
      if (i == ylimit0-1) {
        if (iy[0] < iy[1]) idx0 = 2; else idx1 = 2;
      }
      i++;
    }
  }
  //Line(plotX[idx0],plotY[idx0], plotX[idx1],plotY[idx1], color);
  return( (drawn==0)?(-1):0 );
}

int VirtualScreen::FCircle(int x, int y, int rad, int color) {
  register int d, nx,ny;
  int x1,x2,x3,x4, y1,y2,y3,y4;
  d = 3 - (rad<<1);
  nx = 0;
  ny = rad;
  x1 = x+nx;  x2 = x-nx;  x3 = x+ny;  x4 = x-ny;
  y1 = y+ny;  y2 = y-ny;  y3 = y+nx;  y4 = y-nx;
  int oldy1 = y1+1;
  while (nx <= ny) {
    if (y1 != oldy1) {
      Line(x2,y1, x1,y1, color);
      Line(x2,y2, x1,y2, color);
    }
    if (y3 != y1){
      Line(x3,y3, x4,y3, color);
      if (y3 != y4) Line(x3,y4, x4,y4, color);
    }
    oldy1 = y1;
    nx++;
    x1++;  x2--;
    y3++;  y4--;
    if (d < 0) {
      d += (nx<<2)+6;
    } else {
      d += ((nx-ny--)<<2)+10;
      x3--;  x4++;
      y1--;  y2++;
    }
  }
}
