/* MMX.H */

#ifndef MMX_H
#define MMX_H

typedef int BOOL;
typedef unsigned char BYTE;
typedef unsigned short WORD;
//typedef unsigned long DWORD;
typedef unsigned int DWORD; 
typedef unsigned long long QWORD;

#define MMX_RETTYPE	QWORD

union __m64 {
  /* unsigned types */
  BYTE    b[8];
  WORD    w[4];
  DWORD  dw[2];
  QWORD  qw;
  /* signed types */
  char 	     sb[8];
  short      sw[4];
  long      sdw[2];
  long long sqw;
  /* some useful constructors */
  __m64() { }
  __m64(char &n) { sqw = (long long)n; }
  __m64(int &n) { sqw = (long long)n; }
  __m64(DWORD &n) { qw = (QWORD)n; }
  __m64(long long &n) { sqw = n; }
  __m64(QWORD &n) { qw = n; }
}; // __attribute__ ((transparent_union));


#ifdef __cplusplus
extern "C" {
#endif

BOOL isMMX(void);
BOOL isMMX2(void);

void _mm_empty(void);
MMX_RETTYPE _mm_packs_pi16(__m64 m1, __m64 m2);
MMX_RETTYPE _mm_packs_pi32(__m64 m1, __m64 m2);
MMX_RETTYPE _mm_packs_pu16(__m64 m1, __m64 m2);
MMX_RETTYPE _mm_add_pi8(__m64 m1, __m64 m2);
MMX_RETTYPE _mm_add_pi16(__m64 m1, __m64 m2);
MMX_RETTYPE _mm_add_pi32(__m64 m1, __m64 m2);
MMX_RETTYPE _mm_adds_pi8(__m64 m1, __m64 m2);
MMX_RETTYPE _mm_adds_pi16(__m64 m1, __m64 m2);
MMX_RETTYPE _mm_adds_pu8(__m64 m1, __m64 m2);
MMX_RETTYPE _mm_adds_pu16(__m64 m1, __m64 m2);
MMX_RETTYPE _mm_and_si64(__m64 m1, __m64 m2);
MMX_RETTYPE _mm_andnot_si64(__m64 m1, __m64 m2);
MMX_RETTYPE _mm_avg_pu8(__m64 m1, __m64 m2);
MMX_RETTYPE _mm_avg_pu16(__m64 m1, __m64 m2);
MMX_RETTYPE _mm_cmpeq_pi8(__m64 m1, __m64 m2);
MMX_RETTYPE _mm_cmpeq_pi16(__m64 m1, __m64 m2);
MMX_RETTYPE _mm_cmpeq_pi32(__m64 m1, __m64 m2);
MMX_RETTYPE _mm_cmpgt_pi8(__m64 m1, __m64 m2);
MMX_RETTYPE _mm_cmpgt_pi16(__m64 m1, __m64 m2);
MMX_RETTYPE _mm_cmpgt_pi32(__m64 m1, __m64 m2);
DWORD _mm_extract_pi16(__m64 m1, const int n);
MMX_RETTYPE _mm_insert_pi16(__m64 m1, int dw, const int n);
MMX_RETTYPE _mm_madd_pi16(__m64 m1, __m64 m2);
MMX_RETTYPE _mm_max_pi16(__m64 a, __m64 b);
MMX_RETTYPE _mm_max_pu8(__m64 a, __m64 b);
MMX_RETTYPE _mm_min_pi16(__m64 a, __m64 b);
MMX_RETTYPE _mm_min_pu8(__m64 a, __m64 b);
DWORD _mm_movemask_pi8(__m64 a);
MMX_RETTYPE _mm_mulhi_pu16(__m64 a, __m64 b);
MMX_RETTYPE _mm_mulhi_pi16(__m64 a, __m64 b);
MMX_RETTYPE _mm_mullo_pi16(__m64 a, __m64 b);
MMX_RETTYPE _mm_or_si64(__m64 m1, __m64 m2);
MMX_RETTYPE _mm_sad_pu8(__m64 a, __m64 b);
/* MMX_RETTYPE _mm_shuffle_pi16(__m64 a, int n); */
MMX_RETTYPE _mm_sll_pi16(__m64 m, __m64 count);
/* MMX_RETTYPE _mm_slli_pi16(__m64 m, int count); */
MMX_RETTYPE _mm_sll_pi32(__m64 m, __m64 count);
/* MMX_RETTYPE _mm_slli_pi32(__m64 m, int count); */
MMX_RETTYPE _mm_sll_si64(__m64 m, __m64 count);
/* MMX_RETTYPE _mm_slli_si64(__m64 m, int count); */
MMX_RETTYPE _mm_sra_pi16(__m64 m, __m64 count);
/* MMX_RETTYPE _mm_srai_pi16(__m64 m, int count); */
MMX_RETTYPE _mm_sra_pi32(__m64 m, __m64 count);
/* MMX_RETTYPE _mm_srai_pi32(__m64 m, int count); */
MMX_RETTYPE _mm_srl_pi16(__m64 m, __m64 count);
/* MMX_RETTYPE _mm_srli_si64(__m64 m, int count); */
MMX_RETTYPE _mm_srl_pi32(__m64 m, __m64 count);
/* MMX_RETTYPE _mm_srli_pi16(__m64 m, int count); */
MMX_RETTYPE _mm_srl_si64(__m64 m, __m64 count);
/* MMX_RETTYPE _mm_srli_pi32(__m64 m, int count); */
MMX_RETTYPE _mm_sub_pi8(__m64 m1, __m64 m2);
MMX_RETTYPE _mm_sub_pi16(__m64 m1, __m64 m2);
MMX_RETTYPE _mm_sub_pi32(__m64 m1, __m64 m2);
MMX_RETTYPE _mm_subs_pi8(__m64 m1, __m64 m2);
MMX_RETTYPE _mm_subs_pi16(__m64 m1, __m64 m2);
MMX_RETTYPE _mm_sub_pu8(__m64 m1, __m64 m2);
MMX_RETTYPE _mm_sub_pu16(__m64 m1, __m64 m2);
MMX_RETTYPE _mm_unpckhi_pi8(__m64 m1, __m64 m2);
MMX_RETTYPE _mm_unpckhi_pi16(__m64 m1, __m64 m2);
MMX_RETTYPE _mm_unpckhi_pi32(__m64 m1, __m64 m2);
MMX_RETTYPE _mm_unpcklo_pi8(__m64 m1, __m64 m2);
MMX_RETTYPE _mm_unpcklo_pi16(__m64 m1, __m64 m2);
MMX_RETTYPE _mm_unpcklo_pi32(__m64 m1, __m64 m2);
MMX_RETTYPE _mm_xor_si64(__m64 m1, __m64 m2);

#ifdef __cplusplus
}
#endif


#endif
