/* POINT.CPP */

#ifndef POINT_H
  #error This file must be included from POINT.H
#endif

/* CONSTRUCTORS */

inline Point::Point() {
}

inline Point::Point(const Point &p) {
  this->x = p.x;
  this->y = p.y;
  this->z = p.z;
}

inline Point::Point(POINT_TYPE x, POINT_TYPE y, POINT_TYPE z) {
  this->x = x;
  this->y = y;
  this->z = z;
}

/* FUNCTIONS */

// See pointf.cpp


/* OPERATORS */

// Sum
inline Point operator +(const Point &p1, const Point &p2) {
  return Point(p1.x+p2.x, p1.y+p2.y, p1.z+p2.z);
}

inline Point operator +(const Point &p1, POINT_TYPE n) {
  return Point(p1.x+n, p1.y+n, p1.z+n);
}

// Difference
inline Point operator -(const Point &p1, const Point &p2) {
  return Point(p1.x-p2.x, p1.y-p2.y, p1.z-p2.z);
}

inline Point operator -(const Point &p1, POINT_TYPE n) {
  return Point(p1.x-n, p1.y-n, p1.z-n);
}

// Multiplication by constant
inline Point operator *(const Point &p1, POINT_TYPE n) {
  return Point(p1.x*n, p1.y*n, p1.z*n);
}

// Division by constant
inline Point operator /(const Point &p1, POINT_TYPE n) {
  return Point(p1.x/n, p1.y/n, p1.z/n);
}

// Scalar product
inline POINT_TYPE operator *(const Point &p1, const Point &p2) {
  return p1.x*p2.x + p1.y*p2.y + p1.z*p2.z;
}

// Unary minus
inline Point Point::operator -() {
  this->x = -this->x;
  this->y = -this->y;
  this->z = -this->z;
  return *this;
}

// Unary plus
inline Point Point::operator +() {
  return *this;
}

// Assignment
inline Point & Point::operator =(const Point &p1) {
  this->x = p1.x;
  this->y = p1.y;
  this->z = p1.z;
  return *this;
}


/* Comparison operators */

inline int Point::operator ==(const Point &p2) {
  return (x == p2.x) && (y == p2.y) && (z == p2.z);
}

inline int Point::operator !=(const Point &p2) {
  return (x != p2.x) || (y != p2.y) || (z != p2.z);
}


/* Compound operators */

// Sum
inline Point & Point::operator +=(const Point &p1) {
  this->x += p1.x;
  this->y += p1.y;
  this->z += p1.z;
  return *this;
}

inline Point & Point::operator +=(POINT_TYPE n) {
  this->x += n;
  this->y += n;
  this->z += n;
  return *this;
}

// Difference
inline Point & Point::operator -=(const Point &p1) {
  this->x -= p1.x;
  this->y -= p1.y;
  this->z -= p1.z;
  return *this;
}

inline Point & Point::operator -=(POINT_TYPE n) {
  this->x -= n;
  this->y -= n;
  this->z -= n;
  return *this;
}

// Product
inline Point & Point::operator *=(POINT_TYPE n) {
  this->x *= n;
  this->y *= n;
  this->z *= n;
  return *this;
}

// Division
inline Point & Point::operator /=(POINT_TYPE n) {
  this->x /= n;
  this->y /= n;
  this->z /= n;
  return *this;
}
