/* POINT.H */

#ifndef POINT_H
#define POINT_H

#ifdef PROJECTOR_TYPE
  #define POINT_TYPE		PROJECTOR_TYPE
#else
  #define POINT_TYPE		double
#endif

/* Definition of type Point */

struct Point {
 public:
  POINT_TYPE x, y, z;
  
  /* Constructors */
  Point();
  Point(const Point &p);
  Point(POINT_TYPE x, POINT_TYPE y, POINT_TYPE z);
  
  /* Functions */
  POINT_TYPE length();	// length of vector
  POINT_TYPE length2();	// length^2 of vector

  /* Unary operators */
  Point operator -();
  Point operator +();
  
  /* Binary operators */
  friend Point operator +(const Point &p1, const Point &p2);
  friend Point operator +(const Point &p1, POINT_TYPE n);
  friend Point operator -(const Point &p1, const Point &p2);
  friend Point operator -(const Point &p1, POINT_TYPE n);
  friend Point operator *(const Point &p1, POINT_TYPE n);
  friend Point operator /(const Point &p1, POINT_TYPE n);
  // Scalar product
  friend POINT_TYPE operator *(const Point &p1, const Point &p2);
  
  /* Assignment */
  Point & operator =(const Point &p);

  /* Comparison operators */
  int operator ==(const Point &p1);
  int operator !=(const Point &p1);
  
  /* Compound operators */
  Point & operator +=(const Point &p);
  Point & operator +=(POINT_TYPE n);
  Point & operator -=(const Point &p);
  Point & operator -=(POINT_TYPE n);
  Point & operator *=(POINT_TYPE n);
  Point & operator /=(POINT_TYPE n);
};

/* Functions */

Point *crossProduct(Point p1, Point p2, Point *ret);


/* Operator definitions */
#include "point.cpp"

#endif
