/* CLASS_KRYFILE.H */

#ifndef _CLASS_KRYFILE_H
#define _CLASS_KRYFILE_H

#define HEADER_SIGN		0x2E59524B
#define HEADER_SIGNEND		0x212E
#define FILEHEADER_SIGN		0x00
#define FILEHEADER_SIGNEND	0x2146

typedef struct {
  char signature[4];
  short size;
  short version;
  long int files;
  long int properties;
  long int table_ptr;
  short reserved;
  long int checksum;
  char signature_end[2];
} __attribute__((packed)) Kryheader_090;

typedef struct {
  char sign;
  long int pointers;
  long int files_ptr[1]; // must be extended dynamically
} __attribute__((packed)) Krytable_090;

typedef struct { // First part of File-Header
  char sign;
  short size;
  short namelength;
  char name[1]; // must be extended dynamically
} __attribute__((packed)) Kryfileheader_090_1;

typedef struct { // Second part of File-Header
  long int filesize;
  long int attributes;
  long long date_create; // modification time
  long long date_access;
  long int randval;
  long int properties;
  long int data_ptr;
  char md5sum[32];
  char sign_end[2];
} __attribute__((packed)) Kryfileheader_090_2;

class Kryfile {
 public:
  char flag_showPercent;
  char flag_useMD5Hash;

  Kryfile();
  ~Kryfile();
  int Createfile(const char *filename);
  int Openfile(const char *filename);
  int Writeheader(int count);
  int Readheader();
  int Cryptdata(char *filename, char *pwd);
  int Decryptdata(char *out_path, int out_patcount, char *patterns[], char *pwd);
  int List(int out_patcount, char *patterns[]);
  int Finalize();
  int Close();
  //long int Get_filesadded() { return(filesadded); }
 private:
  FILE *archive;
  char *arc_filename;
  long int filesadded;
  Kryheader_090 *arc_head;
  Krytable_090 *table;

  long int Checksum(int length, char *buffer);
  char *Getshortname(char *name);
  int Readfilehead(Kryfileheader_090_1 **fh1, Kryfileheader_090_2 **fh2, int file);
  int CalcMD5Hash(char *filename, char *hash);

  int Calccubes(char *pwd, int magic, long int filesize, int *ncubes, int **cubesizes);
  long int Get_edgeofcube(int datasize);
  long int Get_bytestoread(int edge);
};

#endif
