/* ERRORS.CPP */
/* Gestisce i messaggi di errore in Kry */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "errors.h"

#define MAX_STRING_LEN		100

const char *txt_errors[ERRORS_COUNT] = {
  "Nessun errore",
  "Opzione non valida: %s",
  "Specificare un'azione",
  "Specificare un archivio",
  "Specificare almeno un file",
  "Impossibile creare il file %s",
  "Il file non è aperto",
  "Scrittura fallita",
  "Impossibile aprire il file %s",
  "Nessun file corrisponde",
  "Memoria insufficiente",
  "Nessun file elaborato, archivio eliminato",
  "L\'archivio non è valido",
  "Le password non coincidono",
  "Impossibile calcolare l\'hash MD5 del file %s",
  "Il file %s non corrisponde all\'originale"};

const char *txt_warnings[WARNINGS_COUNT] = {
  "Nessun allarme",
  "La password è stata troncata alla lunghezza %i",
  "La password non è molto sicura",
  "Impossibile leggere lo stato del file %s",
  "Il file %s esiste già",
  "L\'archivio è stato creato con una versione sconosciuta di kry",
  "L\'archivio è danneggiato",
  "Directory multiple non ammesse, ignoro %s",
  "Impossibile aggiornare lo stato del file %s"};
  
const char *txt_asks[ASKS_COUNT] = {
  "Sovrascrivere il file? [s,n] "};

int error(int number, void *data) {
  char *errname;
  
  number %= ERRORS_COUNT;
  switch (number) {
    case ERR_BADOPT: 
    case ERR_ARCCREAT: 
    case ERR_OPENFILE:
    case ERR_NOMD5:
    case ERR_BADMD5:
      errname = (char*)malloc(MAX_STRING_LEN+strlen((char*)data));
      sprintf(errname, txt_errors[number], (char*)data); break;
    default: 
      errname = (char*)malloc(MAX_STRING_LEN);
      sprintf(errname, "%s", txt_errors[number]);
  }
  
  fprintf(stderr, "Errore #%i: %s.\n", number, errname);
  free(errname);
  return(number);
}

int warn(int number, void *data) {
  char *warnname;
  
  number %= WARNINGS_COUNT;
  switch (number) {
    case WARN_PWDCUT: 
      warnname = (char*)malloc(MAX_STRING_LEN+20);
      sprintf(warnname, txt_warnings[number], *((int*)data)); break;
    case WARN_NOSTAT: 
    case WARN_FILEEXISTS: 
    case WARN_MANYOUTDIR: 
    case WARN_NOWRITESTAT: 
      warnname = (char*)malloc(MAX_STRING_LEN+strlen((char*)data));
      sprintf(warnname, txt_warnings[number], (char*)data); break;
    default: 
      warnname = (char*)malloc(MAX_STRING_LEN);
      sprintf(warnname, "%s", txt_warnings[number]);
  }
  
  fprintf(stderr, "Attenzione! %s.\n", warnname);
  free(warnname);
  return(number);
}

int ask(int number, void *data) {
  char answer[MAX_STRING_LEN];
  int ret = -1;
  
  number %= ASKS_COUNT;
  
  printf("%s", txt_asks[number]);
  fgets(answer, 49, stdin);

  switch (number) {
    case ASK_OVERWRITE: if (answer[0] == 's') ret = 0; else if (answer[0] == 'n') ret = 1; break;
  }
  return(ret);
}
