/* EDITLEV.CPP */

#define VERSION       "2.0"
#define AUTHOR        "Daniele Ferro"
#define AUTHOR_EMAIL  "<ziodf12@tiscali.it>"

#include <stdlib.h>
#include <stdio.h>
#include <SDL/SDL.h>
#include <SDL/SDL_image.h>
#include <sge.h>

#include "editlev_args.h"
#include "labyr.h"

// Global vars
Labyrinth *laby;
SDL_Surface *screen;
int sizex = 640, sizey = 480;

SDL_Rect *SDL_SetRect(SDL_Rect *rect, Sint16 x, Sint16 y, Uint16 w, Uint16 h) {
  rect->x = x;
  rect->y = y;
  rect->w = w;
  rect->h = h;
  return rect;
}

void draw_panel() {
  SDL_Surface *icon;
  SDL_Rect rect;

  SDL_FillRect(screen, SDL_SetRect(&rect, 0,0, 26*6, 26*16), 0);
  for (int j=0; j<6; j++) sge_Line(screen, j*26,0, j*26, screen->h-1, 0x9f9f9f);
  for (int j=0; j<16; j++) sge_Line(screen, 0,j*26, 26*6,j*26, 0x9f9f9f);

  icon = IMG_Load("editlev_data/open.png");
  SDL_BlitSurface(icon, NULL, screen, SDL_SetRect(&rect, 2+26*0,2,24,24));
  SDL_FreeSurface(icon);
  icon = IMG_Load("editlev_data/save.png");
  SDL_BlitSurface(icon, NULL, screen, SDL_SetRect(&rect, 2+26*1,2,24,24));
  SDL_FreeSurface(icon);
  icon = IMG_Load("editlev_data/exit.png");
  SDL_BlitSurface(icon, NULL, screen, SDL_SetRect(&rect, 2+26*4,2,24,24));
  SDL_FreeSurface(icon);

  // Walls
  icon = IMG_Load("editlev_data/wall_w.png");
  SDL_BlitSurface(icon, NULL, screen, SDL_SetRect(&rect, 2+26*0,2+26*2,24,24));
  SDL_FreeSurface(icon);
  icon = IMG_Load("editlev_data/wall_n.png");
  SDL_BlitSurface(icon, NULL, screen, SDL_SetRect(&rect, 2+26*1,2+26*2,24,24));
  SDL_FreeSurface(icon);
  icon = IMG_Load("editlev_data/wall_e.png");
  SDL_BlitSurface(icon, NULL, screen, SDL_SetRect(&rect, 2+26*2,2+26*2,24,24));
  SDL_FreeSurface(icon);
  icon = IMG_Load("editlev_data/wall_s.png");
  SDL_BlitSurface(icon, NULL, screen, SDL_SetRect(&rect, 2+26*3,2+26*2,24,24));
  SDL_FreeSurface(icon);

  // Oneway
  icon = IMG_Load("editlev_data/oneway_e.png");
  SDL_BlitSurface(icon, NULL, screen, SDL_SetRect(&rect, 2+26*0,2+26*3,24,24));
  SDL_FreeSurface(icon);
  icon = IMG_Load("editlev_data/oneway_s.png");
  SDL_BlitSurface(icon, NULL, screen, SDL_SetRect(&rect, 2+26*1,2+26*3,24,24));
  SDL_FreeSurface(icon);
  icon = IMG_Load("editlev_data/oneway_w.png");
  SDL_BlitSurface(icon, NULL, screen, SDL_SetRect(&rect, 2+26*2,2+26*3,24,24));
  SDL_FreeSurface(icon);
  icon = IMG_Load("editlev_data/oneway_n.png");
  SDL_BlitSurface(icon, NULL, screen, SDL_SetRect(&rect, 2+26*3,2+26*3,24,24));
  SDL_FreeSurface(icon);

  icon = IMG_Load("editlev_data/start.png");
  SDL_BlitSurface(icon, NULL, screen, SDL_SetRect(&rect, 2+26*1,2+26*5,24,24));
  SDL_FreeSurface(icon);


  sge_Line(screen, 26*6,0, 26*6,screen->h-1, 0xffffff);
  SDL_UpdateRect(screen, 0,0,0,0);
}

void draw_all() {
  draw_panel();
}

int main(int argc, char *argv[]) {
  parse_args(argc, argv);
  if (!arguments.filename) {
    printf("Nome del file da aprire o creare: ");
    arguments.filename = (char *)malloc(256);
    fgets(arguments.filename, 255, stdin);
    arguments.filename[255] = '\0';
  }

  // Init labyrinth
  laby = new Labyrinth();
  FILE *f = fopen(arguments.filename, "r");
  if (f != NULL) { // file exists
    fclose(f);
    laby->readFromFile(arguments.filename);
  } else { // file is new
    laby->create(arguments.width, arguments.height);
  }

  // Init SDL
  SDL_Init(SDL_INIT_VIDEO);
  screen = SDL_SetVideoMode(sizex, sizey, 32, SDL_HWSURFACE | SDL_RESIZABLE | SDL_SRCALPHA);
  SDL_SetAlpha(screen, SDL_SRCALPHA, SDL_ALPHA_OPAQUE);
  SDL_WM_SetCaption("Laby level editor", NULL);
  SDL_ShowCursor(SDL_ENABLE);

  // Main Loop
  int inloop = 1;
  SDL_Event ev;
  while (inloop) {
    SDL_WaitEvent(&ev);
    switch (ev.type) {
      case SDL_QUIT:
        inloop = 0;
        break;
      case SDL_KEYDOWN:
        if (ev.key.keysym.sym == SDLK_ESCAPE) inloop = 0;
        break;
    }
    draw_all();
  }

  // Free memory
  delete laby;
  SDL_Quit();

  return 0;
}
