/*  LABY V2  */

#pragma option -k-    // no stack frame
#pragma option -N-    // no stack overflow check
#pragma option -1     // enable 286 instruction
//#pragma option -f87


#define VERSION_HI  2
#define VERSION_LO  0

#include <conio.h>
#include <stdio.h>
#include <dos.h>
#include <dir.h>
#include <time.h>
#include <math.h>
#include <crt.h>
#include <vga.h>
#include <vgapal.h>

#include "labyr.h"
#include "vgawin.h"
#include "vgafont.h"
#include "menu.h"
#include "topscore.h"

#define CELL_SIZE         10

#include "items.h"
#include "myfont.h"

#define GAME_DELAY        40
#define LEVELS             6
#define LIGHT_DEFRADIUS   50   // standard visual radius
#define LIGHT_MIDRADIUS   30   // radius without light decay
#define LIGHT_MAXRADIUS  150   // maximum visual radius


const char *levels[LEVELS] = {"tutor.lev","laby01.lev","laby02.lev","laby03.lev","laby04.lev","laby05.lev"};
char levToLoad[16] = "";

VirtualScreen *mainScr = 0;
Font *font = 0;
TopScores *scores = 0;
pal256 mainPal;

int lives = 1;
int curLevel = 1,
    hackLevel = 0;
int lightRadius = LIGHT_DEFRADIUS,
    lightCounter = 0;
char lightInitialized = 0;
char far *lightDeltaTab = 0; // quite big, doesn't fit into heap
struct {
  int rCounter, gCounter, bCounter;
} potions = {0,0,0};
struct {
  char up,dn,sx,dx;
} keys = {0x48,0x50,0x4B,0x4D};

short posX, posY;
int lastDirE = 1;
int yetRead = 0;

Labyrinth *laby = 0;
typedef struct {
  short x,y;
} TELEP;
TELEP tports[16][4];
int nports[16];

int n_remItems;
TELEP *remItems = 0;

extern "C" short cdecl dist(short x, short y);
extern "C" short cdecl dist2(short x, short y);
extern "C" char cdecl getLightRad(short x, short y);

int random(int min, int max) { return( rand()%(abs(max-min)+1)+min ); }

void postExit(void) {
  if (crt_getVideoMode() == 0x13) { // not restored
    crt_setVideoMode(0x03 | 0x80);
  }
}

void exitFunc(int code) {
  if (mainScr != 0) delete mainScr;
  if (laby != 0) delete laby;
  if (font != 0) delete font;
  if (scores != 0) delete scores;
  if (lightDeltaTab != 0) farfree(lightDeltaTab);
  if (remItems != 0) free(remItems);
  crt_restoreVideoMode();
  exit(code);
}

void precalcTPorts() {
  int i,j,n;
  memset(nports, 0, sizeof(int)*16);
  for (j=0; j<laby->getHeight(); j++)
    for (i=0; i<laby->getWidth(); i++) {
      BYTE it = laby->getItem(i,j);
      if ((it >= LABY_ITEM_TELEPORT)&&(it < LABY_ITEM_TELEPORT+16)) {
        n = it & 0x0F;
        if (nports[n] < 4) {
          tports[n][nports[n]].x = i;
          tports[n][nports[n]].y = j;
          nports[n]++;
        }
      }
    }
}

void precalcLightTab() {
  int i,j, rad;
  lightInitialized = 0;
  lightDeltaTab = (char far *)farmalloc((unsigned long)LIGHT_MAXRADIUS*LIGHT_MAXRADIUS+8);
  if (lightDeltaTab == 0) return; // no memory

  for (j=0; j<LIGHT_MAXRADIUS; j++) {
    for (i=j; i<LIGHT_MAXRADIUS; i++) {
      rad = dist(i,j);
      if (rad < LIGHT_MIDRADIUS) rad = 0; else rad = ((rad-LIGHT_MIDRADIUS)*16)/(LIGHT_MAXRADIUS-LIGHT_MIDRADIUS);
      *(lightDeltaTab+j*LIGHT_MAXRADIUS+i) = (char)rad;
      if (i != j) *(lightDeltaTab+i*LIGHT_MAXRADIUS+j) = (char)rad;
    }
  }
  lightInitialized = 1;
}

void glowText(VirtualScreen *virscr, VGACOORD x, VGACOORD y, char *text, char size, BYTE r1, BYTE g1, BYTE b1, BYTE r2, BYTE g2, BYTE b2, int time) {
  const int res = 16;
  BYTE or,og,ob, vr,vg,vb;
  int i, dlay;

  if (time == 0) return;
  virscr->text(x,y, text, size, 0xFF);
  pal_inRGB(0xFF, &or, &og, &ob);
  pal_outRGB(0xFF, r1, b1, g1);
  virscr->show();
  dlay = time/res;
  for (i=1; i<=res; i++) {
    vr = r1 - r1*i/res + r2*i/res;
    vg = g1 - g1*i/res + g2*i/res;
    vb = b1 - b1*i/res + b2*i/res;
    pal_outRGB(0xFF, vr, vg, vb);
    delay(dlay);
  }
  //pal_outRGB(0xFF, or, og, ob);
}

void drawItem(int x, int y, const BYTE *ptr, char reverse) {
  int i,j;
  BYTE b;
  for (j=0; j<CELL_SIZE; j++) {
    for (i=0; i<CELL_SIZE; i++) {
      switch (reverse) {
        case 1:
          b = *(ptr+j*CELL_SIZE+CELL_SIZE-i-1);
          break;
        case 2:
          b = *(ptr+(CELL_SIZE-j-1)*CELL_SIZE+CELL_SIZE+i);
          break;
        default: b = *(ptr+j*CELL_SIZE+i);
      }
      if (b != 0) mainScr->putPixel(x+i,y+j, b);
    }
  }
}

void drawSprite(int x, int y, int sizeX, int sizeY, const BYTE *ptr, char reverse) {
  int i,j;
  BYTE b;
  for (j=0; j<sizeY; j++) {
    for (i=0; i<sizeX; i++) {
      switch (reverse) {
        case 1:
          //b = *(ptr+j*sizeX+sizeX-i-1);
          break;
        case 2:
          //b = *(ptr+(sizeY-j-1)*sizeX+CELL_SIZE+i);
          break;
        default: b = *(ptr+j*sizeX+i);
      }
      if (b != 0) mainScr->putPixel(x+i,y+j, b);
    }
  }
}

int initMenu() {
  char buff[20];
  mainScr->clear(0);

  sprintf(buff, "v %i.%.2i", VERSION_HI, VERSION_LO);
  font->setTextColor(0x51);
  font->align = 2; //right
  font->printRect(250,190,319,199, buff);
  glowText(mainScr, 95,40, "ZioSoft presents", 1, 0,0,0, mainPal[1].r,mainPal[1].g,mainPal[1].b, 500);
  mainScr->text(95, 40, "ZioSoft presents", 1, 1);
  glowText(mainScr, 24,70, "LABYRINTH", 2, 0,0,0, mainPal[2].r,mainPal[2].g,mainPal[2].b, 500);
  mainScr->text(24, 70, "LABYRINTH", 2, 2);

  srand(time(0));
  Menu *initmenu = new Menu(mainScr, VGA_WIDTH/2-80, 109, VGA_WIDTH/2+80, 190);
  initmenu->setColors(0x54,0x52,0x54, 0x51);
  initmenu->addItem(0,"Gioca");
  initmenu->addItem(1,"Istruzioni");
  initmenu->addItem(2,"Carica partita");
  initmenu->addItem(3,"Punteggi");
  initmenu->addItem(4,"Esci");
  int choice = initmenu->choose();
  if (choice == -1) choice = 4; //abort
  delete initmenu;
  return(choice);
}

void initGame() { // choice by user
  atexit(postExit);
  crt_init();
  crt_setVideoMode(0x13);
  pal_inAll(&mainPal);
  pal_setGradient(&mainPal, 0x10,0x1F, 0,0,0, 63,63,63);  // white
  pal_setGradient(&mainPal, 0x20,0x27, 0,0,0, 0,0,63);    // blue
  pal_setGradient(&mainPal, 0x28,0x2F, 0,0,63, 63,63,63);
  pal_setGradient(&mainPal, 0x30,0x37, 0,0,0, 0,63,0);    // green
  pal_setGradient(&mainPal, 0x38,0x3F, 0,63,0, 63,63,63);
  pal_setGradient(&mainPal, 0x40,0x47, 0,0,0, 63,0,0);    // red
  pal_setGradient(&mainPal, 0x48,0x4F, 63,0,0, 63,63,63);
  pal_setGradient(&mainPal, 0x50,0x57, 0,0,0, 0,63,63);   // cyan
  pal_setGradient(&mainPal, 0x58,0x5F, 0,63,63, 63,63,63);
  pal_setGradient(&mainPal, 0x60,0x67, 0,0,0, 63,63,0);   // yellow
  pal_setGradient(&mainPal, 0x68,0x6F, 63,63,0, 63,63,63);
  pal_setGradient(&mainPal, 0x70,0x77, 0,0,0, 63,0,63);   // magenta
  pal_setGradient(&mainPal, 0x78,0x7F, 63,0,63, 63,63,63);
  pal_setGradient(&mainPal, 0x80,0x87, 0,0,0, 37,19,18);  // brown
  pal_setGradient(&mainPal, 0x88,0x8F, 37,19,18, 63,63,63);
  pal_outAll(&mainPal);
  mainScr = new VirtualScreen();
  laby = new Labyrinth();
  font = new Font(mainScr);
  for (int i=0; i<CHAR_COUNT; i++) font->addChar(fnts[i]);
}

void drawStatus() {
  int i, statY = 179;
  char buff[50];

  mainScr->line(20, 199, 20, 184, 0x54);
  mainScr->line(300, 199, 300, 184, 0x54);
  for (i=0; i<2; i++) {
    mainScr->putPixel((i==0)?21:299, 183, 0x54);
    mainScr->putPixel((i==0)?21:299, 182, 0x54);
    mainScr->putPixel((i==0)?22:298, 181, 0x54);
    mainScr->putPixel((i==0)?23:297, 180, 0x54);
    mainScr->putPixel((i==0)?24:296, 180, 0x54);
  }
  mainScr->line(25, statY, 295, statY, 0x54);
  mainScr->line(25, 180, 295, 180, 0x52);
  mainScr->line(23, 181, 297, 181, 0x52);
  mainScr->line(22, 182, 298, 182, 0x52);
  mainScr->line(22, 183, 298, 183, 0x52);
  mainScr->fRect(21, 184, 299, 199, 0x52);

  drawSprite(25,185, 13,10, sprite_heart, 0);
  sprintf(buff, "X %i", lives);
  font->setTextColor(0x46);
  font->print(40,185, buff);

  // 2do!! insert sprites below
  if (lightCounter > 0) {
    sprintf(buff, "Torcia (%i%%)", (lightRadius*100)/LIGHT_DEFRADIUS);
    font->setTextColor(0x6A);
    font->print(65,185, buff);
  } else {
    font->setTextColor(0x63);
    font->print(65,185, "Torcia (100%)");
  }

  if (potions.rCounter > 0) {
    sprintf(buff, "Poz R (%i)", potions.rCounter);
    font->setTextColor(0x4A);
    font->print(130,185, buff);
  }
  if (potions.gCounter > 0) {
    sprintf(buff, "Poz G (%i)", potions.gCounter);
    font->setTextColor(0x3A);
    font->print(190,185, buff);
  }
  if (potions.bCounter > 0) {
    sprintf(buff, "Poz B (%i)", potions.bCounter);
    font->setTextColor(0x5A);
    font->print(250,185, buff);
  }
}

void drawShadow() {
  BYTE col;
  int i,j, mx,my, rad;
  int cx = (VGA_WIDTH+CELL_SIZE)/2;
  int cy = (VGA_HEIGHT+CELL_SIZE)/2;

  if (!lightInitialized) precalcLightTab();
  if (!lightInitialized || !lightDeltaTab) return; // some errors, skip shadow

  //my = min(179, cy+lightRadius); // status bar, see in showStatus()
  my = cy+lightRadius; // buggy!!
  mx = min(VGA_WIDTH, cx+lightRadius);
  for (j=cy; j<my; j++) {
    for (i=cx; i<mx; i++) {
      rad = -1;
      col = mainScr->getPixel(i,j);
      if (col >= 0x10) {
        rad = getLightRad(i-cx, j-cy);
        if (col > 0x1F) if ((col & 0x0F) >= 0x08) rad++;
        col = max(col & 0xF0, col-rad);
        mainScr->putPixel(i,j, col);
      }
      if (i != cx) {
        col = mainScr->getPixel(2*cx-i,j);
        if (col >= 0x10) {
          if (rad == -1) rad = getLightRad(i-cx, j-cy);
          if (col > 0x1F) if ((col & 0x0F) >= 0x08) rad++;
          col = max(col & 0xF0, col-rad);
          mainScr->putPixel(2*cx-i,j, col);
        }
      }
      if (j != cy) {
        col = mainScr->getPixel(i,2*cy-j);
        if (col >= 0x10) {
          if (rad == -1) rad = getLightRad(i-cx, j-cy);
          if (col > 0x1F) if ((col & 0x0F) >= 0x08) rad++;
          col = max(col & 0xF0, col-rad);
          mainScr->putPixel(i,2*cy-j, col);
        }
        if (i != cx) {
          col = mainScr->getPixel(2*cx-i,2*cy-j);
          if (col >= 0x10) {
            if (rad == -1) rad = getLightRad(i-cx, j-cy);
            if (col > 0x1F) if ((col & 0x0F) >= 0x08) rad++;
            col = max(col & 0xF0, col-rad);
            mainScr->putPixel(2*cx-i,2*cy-j, col);
          }
        }
      }
    }
  }
  mainScr->fRect(cx+lightRadius, cy-lightRadius, cx+lightRadius+CELL_SIZE+2, cy+lightRadius, 0);
  mainScr->fRect(cx-lightRadius, cy+lightRadius, cx+lightRadius, cy+lightRadius+CELL_SIZE+2, 0);
}

void drawLaby() {
  const BYTE wallColor = 0x1C;
  int i,j, tx,ty;
  char w,it;
  int cx = (VGA_WIDTH+CELL_SIZE)/2;
  int cy = (VGA_HEIGHT+CELL_SIZE)/2;
  int stx = 160-posX*CELL_SIZE;
  int sty = 100-posY*CELL_SIZE;

  for (j=0; j<laby->getHeight(); j++) {
    ty = sty+j*CELL_SIZE;
    for (i=0; i<laby->getWidth(); i++) {
      tx = stx+i*CELL_SIZE;
      if (dist(tx-cx,ty-cy) >= lightRadius) continue;
      w = laby->getWall(i,j);
      if ((w & LABY_WALL_E) == LABY_WALL_E) mainScr->line(tx+CELL_SIZE-1, ty, tx+CELL_SIZE-1, ty+CELL_SIZE-1, wallColor);
      if ((w & LABY_WALL_N) == LABY_WALL_N) mainScr->line(tx, ty, tx+CELL_SIZE-1, ty, wallColor);
      if ((w & LABY_WALL_W) == LABY_WALL_W) mainScr->line(tx, ty, tx, ty+CELL_SIZE-1, wallColor);
      if ((w & LABY_WALL_S) == LABY_WALL_S) mainScr->line(tx, ty+CELL_SIZE-1, tx+CELL_SIZE-1, ty+CELL_SIZE-1, wallColor);
      it = laby->getItem(i,j);
      switch (it) {
        case LABY_ITEM_LIFE:
          drawItem(tx, ty, item_life,0);
          break;
        case LABY_ITEM_DEATH:
          drawItem(tx, ty, item_death,0);
          break;
        case LABY_ITEM_ENDLEV:
          drawItem(tx, ty, item_endlevel,0);
          break;
        case LABY_ITEM_MESSAGE:
          drawItem(tx, ty, item_msg,0);
          break;
        case LABY_ITEM_LIGHT:
          drawItem(tx, ty, item_light,0);
          break;
        case LABY_ITEM_POTIONR:
          drawItem(tx, ty, item_potionr,0);
          break;
        case LABY_ITEM_POTIONG:
          drawItem(tx, ty, item_potiong,0);
          break;
        case LABY_ITEM_POTIONB:
          drawItem(tx, ty, item_potionb,0);
          break;
        case LABY_ITEM_ONEWAY_E:
          drawItem(tx, ty, item_1waye,0);
          break;
        case LABY_ITEM_ONEWAY_N:
          drawItem(tx, ty, item_1wayn,0);
          break;
        case LABY_ITEM_ONEWAY_W:
          drawItem(tx, ty, item_1waye,1);
          break;
        case LABY_ITEM_ONEWAY_S:
          drawItem(tx, ty, item_1wayn,2);
          break;
        default:
          if ((it >= LABY_ITEM_TELEPORT)&&(it < LABY_ITEM_TELEPORT+16)) {
            drawItem(tx, ty, item_tport,0);
          }
          break;
      }
    }
  }
  drawItem(cx-CELL_SIZE/2, cy-CELL_SIZE/2, item_player, lastDirE);
  drawShadow();
}

void showMessage(char *text, char isplaying) {
  char keyloop;
  const int sizX = 200;
  int len = strlen(text);
  int lin = ((len*font->getCharWidth('A'))/sizX + 2);
  int sizY = 1+font->getCharHeight('A')*lin;
  int sx = (VGA_WIDTH-sizX)>>1,
      sy = (VGA_HEIGHT-sizY)>>1;

  if (isplaying) {
    drawLaby();
    drawStatus();
  }
  Window *msg = new Window(mainScr,sx,sy, sx+sizX,sy+sizY);
  msg->setColors(0x54,0x52);
  msg->show(1);
  font->align = 1;
  font->setTextColor(0x54);
  font->printRect(sx+3,sy+2, sx+sizX-3, sy+sizY-2, text);
  mainScr->show();
  keyloop = 1;
  do {
    char c = getch();
    if ((c == 0x0D)||(c == 0x20)||(c == 0x1B)) keyloop = 0;
  } while (keyloop);
  delete msg;
}

void saveGame() {
  int i, ret, fileCount, handle;
  char buff[40], filename[16];
  ffblk finds;
  TELEP rem;
  struct {
    short level;
    short x, y;
    short lives;
    short lightrad, lightcount;
    short rc, gc, bc;
    char reserved[16];
    short remitems;
  } savedata;

  Window *savewin = new Window(mainScr, 0,0, 200, 40);
  savewin->setColors(0x54,0x52);
  savewin->setCenter();
  savewin->show(1);
  font->setTextColor(0x54);
  font->setBackColor(0x52);
  font->print(savewin->getX1()+2, savewin->getY1()+2, "Inserisci il nome della partita:");
  font->readLine(savewin->getX1()+2, savewin->getY1()+2+font->getCharHeight('A'), 31, buff);
  delete savewin;
  if (buff[0] == '\0') {
    showMessage("Salvataggio annullato",1);
    return;
  }
  fileCount = 0;
  ret = findfirst("*.SAV", &finds, FA_ARCH);
  while (ret == 0) {
    fileCount++;
    ret = findnext(&finds);
  }
  sprintf(filename, "LABY%.4i.SAV", fileCount);

  handle = _creat(filename, FA_ARCH);
  if (handle == -1) return;
  if (_write(handle, "LABSAV", 6) == -1) return;
  *((short*)filename) = 0;
  _write(handle, filename, 2);
  _write(handle, buff, 32);
  savedata.level = curLevel;
  savedata.x = posX;
  savedata.y = posY;
  savedata.lives = lives;
  savedata.lightrad = lightRadius;
  savedata.lightcount = lightCounter;
  savedata.rc = potions.rCounter;
  savedata.gc = potions.gCounter;
  savedata.bc = potions.bCounter;
  memset(savedata.reserved,0,sizeof(savedata.reserved));
  savedata.remitems = n_remItems;
  _write(handle, &savedata, sizeof(savedata));
  for (i=0; i<n_remItems; i++) _write(handle, (remItems+i), sizeof(TELEP));
  buff[0] = 0xFF;
  _write(handle, buff, 1);
  _close(handle);
  showMessage("Salvataggio effettutato",1);
}

int getSaveName(const char *filename, char *buffer) { //ret: -1=error
  int handle, rd;
  char buff[10];

  handle = _open(filename, O_RDONLY);
  if (handle == -1) return(-1);
  rd = _read(handle, buff, 8);
  if (rd != 8) {
    _close(handle);
    return(-1);
  }
  buff[6] = '\0';
  if (strcmp(buff, "LABSAV") != 0) {
    _close(handle);
    return(-1);
  }
  rd = _read(handle, buffer, 32);
  if (rd != 32) {
    _close(handle);
    return(-1);
  }
  buffer[31] = '\0';
  _close(handle);
  return(0);
}

int loadGame() { // ret: 2=aborted, 3=loaded
  int ret, t, i;
  char buff[40];
  ffblk finds;
  char *fileNames[100]; // max 100 savegames

  Menu *loadmenu = new Menu(mainScr, 0,0, 200, 180);
  loadmenu->setColors(0x54,0x52,0x54, 0x51);
  loadmenu->setCenter();
  loadmenu->setFont(font);

  i = 0;
  ret = findfirst("*.SAV", &finds, FA_ARCH);
  while (ret == 0) {
    t = getSaveName(finds.ff_name, buff);
    if (t == 0) {
      fileNames[i] = (char*)malloc(sizeof(char)*16);
      strcpy(fileNames[i], finds.ff_name);
      loadmenu->addItem(i++,buff);
    }
    ret = findnext(&finds);
  }
  if (i == 0) {
    delete loadmenu;
    return(2);
  }
  ret = loadmenu->choose();
  if (ret != -1) {
    strcpy(levToLoad, fileNames[ret]);
    for (t=0; t<i; t++) free(fileNames[t]);
    ret = 3;
  } else ret = 2;
  delete loadmenu;
  return(ret);
}

void lostGame(char showMsg) {
  int it;
  if (showMsg) showMessage("Sei morto!",1);
  if (curLevel == 0) return;
  scores = new TopScores();
  scores->readItems();
  scores->sort();
  it = scores->getItems();
  if ((it < SCORE_MAXITEMS)||(curLevel >= scores->getItem(it-1)->level)) {
    char name[20];
    Window *scorewin = new Window(mainScr, 0,0, 220, 30);
    scorewin->setColors(0x54,0x52);
    scorewin->setCenter();
    scorewin->show(1);
    font->setTextColor(0x54);
    font->setBackColor(0x52);
    font->print(scorewin->getX1()+2, scorewin->getY1()+2, "Sei entrato nella classifica, inserisci il tuo nome:");
    font->readLine(scorewin->getX1()+2, scorewin->getY1()+2+font->getCharHeight('A'), 14, name);
    scores->addItem(curLevel, name);
    scores->writeItems();
    delete scorewin;
  }
  delete scores;
  scores = 0;
}

int pauseGame() { // 0=return game, 1=save, 2=exit
  int sizX = 100,
      sizY = 3*font->getCharHeight('A')+1;
  int ret;

  drawLaby();
  drawStatus();
  Menu *pause = new Menu(mainScr, (VGA_WIDTH-sizX)>>1,(VGA_HEIGHT-sizY)>>1,(VGA_WIDTH+sizX)>>1,(VGA_HEIGHT+sizY)>>1);
  pause->addItem(0, "Ritorna al gioco");
  pause->addItem(1, "Salva partita");
  pause->addItem(2, "Esci");
  pause->setColors(0x54,0x52,0x54, 0x51);
  pause->setFont(font);
  ret = pause->choose();
  if (ret == -1) ret = 0;
  delete pause;
  return(ret);
}

void moveLine(short xend, short yend) {
  int dx = xend-posX, dy = yend-posY,
      ix = abs(dx), iy = abs(dy),
      inct = (ix>iy?ix:iy),
      plotX = posX, plotY = posY,
      x = 0, y = 0,
      i;
  char plot;
  for (i=0;i<=inct;i++) {
    x += ix;
    y += iy;
    plot = 0;
    if (x > inct) {
      plot = 1;
      x -= inct;
      if (dx > 0) plotX++;
      if (dx < 0) plotX--;
     }
     if (y > inct) {
       plot = 1;
       y -= inct;
       if (dy > 0) plotY++;
       if (dy < 0) plotY--;
     }
     if (plot) {
       posX = plotX;
       posY = plotY;
       drawLaby();
       drawStatus();
       mainScr->show();
       delay(GAME_DELAY);
     }
  }
}

void removeItem(short x, short y) {
  laby->setItem(x,y, LABY_ITEM_NULL);
  remItems = (TELEP *)realloc(remItems, sizeof(TELEP)*(n_remItems+1));
  (remItems+n_remItems)->x = x;
  (remItems+n_remItems)->y = y;
  n_remItems++;
}

void mixKeys() {
  char k1[4] = {0xFF,0xFF,0xFF,0xFF},
       k2[4];
  int k;

  for (int i=0; i<4; i++) {
    do k = rand()%4; while (k1[k] != -1);
    k1[k] = i;
  }
  k2[k1[0]] = keys.up;
  k2[k1[1]] = keys.dn;
  k2[k1[2]] = keys.sx;
  k2[k1[3]] = keys.dx;
  keys.up = k2[0];
  keys.dn = k2[1];
  keys.sx = k2[2];
  keys.dx = k2[3];
}

int evalItem(BYTE item) { // 0=nothing, ret 1 if next level, 2 if dead
  switch (item) {
    case LABY_ITEM_LIFE:
      lives++;
      removeItem(posX, posY);
      break;
    case LABY_ITEM_DEATH:
      if (potions.bCounter == 0) {
        lives--;
        if (lives <= 0) {
          lostGame(1);
          return(2);
        }
      }
      break;
    case LABY_ITEM_MESSAGE:
      if (!yetRead) showMessage(laby->getMessage(posX, posY),1);
      yetRead = 1;
      break;
    case LABY_ITEM_LIGHT: {
      int d = min(80, LIGHT_MAXRADIUS-lightRadius);
      // because some pc's are very slow!
      lightCounter += d;
      lightRadius += d;
      drawLaby();
      drawStatus();
      mainScr->show();
      /*for (int i=1; i<=d; i++) {
        lightRadius++;
        drawLaby();
        drawStatus();
        mainScr->show();
        delay(GAME_DELAY/2);
      }*/
      removeItem(posX, posY);
      break;
    }
    case LABY_ITEM_POTIONR:
      potions.rCounter += 10;
      removeItem(posX, posY);
      break;
    case LABY_ITEM_POTIONG: {
      mixKeys();
      potions.gCounter += 50;
      removeItem(posX, posY);
      break;
    }
    case LABY_ITEM_POTIONB:
      potions.bCounter += 30;
      removeItem(posX, posY);
      break;
    case LABY_ITEM_ENDLEV:
      showMessage("Congratulazioni!!",1);
      return(1);
    default:
      if ((item >= LABY_ITEM_TELEPORT)&&(item < LABY_ITEM_TELEPORT+16)) {
        int idx, n = item & 0x0F;
        if (nports[n] > 1) { // choose random
          do {
            idx = random(0, nports[n]-1);
          } while ((tports[n][idx].x == posX)&&(tports[n][idx].y == posY));
        }
        moveLine(tports[n][idx].x, tports[n][idx].y);
        posX = tports[n][idx].x;
        posY = tports[n][idx].y;
      }
      break;
  }
  return(0);
}

void showScore();
#include "starscor.h"

int playGame(char *filename, char isSaveGame) { // ret 1 if next level, 2 if dead
  char key, playing = 1, moved = 0;
  char buff[100];
  int t = 0, ret = 0;
  int handle;

  if (laby == 0) return(0);
  mainScr->clear(0);
  if (isSaveGame) { // open saved game
    handle = _open(filename, O_RDONLY);
    if (handle == -1) return(2);
    lseek(handle, 0x28, SEEK_SET);
    _read(handle,  &t, 2);  // read level number
    curLevel = t;
    strcpy(filename, levels[curLevel]);
  }

  laby->readFromFile(filename);
  precalcTPorts();
  n_remItems = 0;
  if (remItems != 0) { free(remItems); remItems = 0; }

  if (isSaveGame) {
    struct {
      short x, y;
      short lives;
      short lightrad, lightcount;
      short rc, gc, bc;
      char reserved[16];
      short remitems;
    } savedata;
    t = _read(handle, &savedata, sizeof(savedata));
    if (t != sizeof(savedata)) exitFunc(1);
    posX = savedata.x;
    posY = savedata.y;
    lives = savedata.lives;
    lightRadius = savedata.lightrad;
    lightCounter = savedata.lightcount;
    potions.rCounter = savedata.rc;
    potions.gCounter = savedata.gc;
    if (potions.gCounter > 0) mixKeys();
    potions.bCounter = savedata.bc;
    for (t=0; t<savedata.remitems; t++) {
      unsigned long pos;
      _read(handle, &pos, sizeof(pos));
      removeItem(pos & 0xFFFFL, pos >> 16);
    }
    _close(handle);
  } else {
    posX = laby->getStartX();
    posY = laby->getStartY();
    lives = 1;
    lightRadius = LIGHT_DEFRADIUS;
    lightCounter = 0;
    potions.rCounter = 0;
    potions.bCounter = 0;
    if (potions.gCounter != 0) {
      keys.up = 0x48;
      keys.dn = 0x50;
      keys.sx = 0x4B;
      keys.dx = 0x4D;
      potions.gCounter = 0;
    }
  }

  sprintf(buff, "%i: %s", curLevel, laby->getName());
  showMessage(buff,1);

  do {
    mainScr->clear(0);
    if (kbhit()) {
      key = getch();
      if (key == 0) {
        key = getch();
        switch (key) {
          case 0x4B: key = keys.sx; break;
          case 0x4D: key = keys.dx; break;
          case 0x48: key = keys.up; break;
          case 0x50: key = keys.dn; break;
        }
        switch (key) {
	  case 0x4B: //sx
	    if ((((laby->getWall(posX, posY) & LABY_WALL_W) != LABY_WALL_W)&&
                (laby->getItem(posX-1, posY) != LABY_ITEM_ONEWAY_E))||
                (potions.rCounter > 0)) {
              posX--;
              if (posX < 0) posX = 0;
              moved = 1;
              yetRead = 0;
            }
            lastDirE = 0;
	    break;
	  case 0x4D: //dx
            if ((((laby->getWall(posX, posY) & LABY_WALL_E) != LABY_WALL_E)&&
                (laby->getItem(posX+1, posY) != LABY_ITEM_ONEWAY_W))||
                (potions.rCounter > 0)) {
              posX++;
              if (posX >= laby->getWidth()) posX = laby->getWidth()-1;
              moved = 1;
              yetRead = 0;
            }
            lastDirE = 1;
	    break;
	  case 0x48: //up
            if ((((laby->getWall(posX, posY) & LABY_WALL_N) != LABY_WALL_N)&&
                (laby->getItem(posX, posY-1) != LABY_ITEM_ONEWAY_S))||
                (potions.rCounter > 0)) {
              posY--;
              if (posY < 0) posY = 0;
              moved = 1;
              yetRead = 0;
            }
	    break;
	  case 0x50: //down
            if ((((laby->getWall(posX, posY) & LABY_WALL_S) != LABY_WALL_S)&&
                (laby->getItem(posX, posY+1) != LABY_ITEM_ONEWAY_N))||
                (potions.rCounter > 0)) {
              posY++;
              if (posY >= laby->getHeight()) posY = laby->getHeight()-1;
              moved = 1;
              yetRead = 0;
            }
            break;
        }
      } else {
        switch (key) {
          case 0x1B:
            t = pauseGame();
            switch (t) {
              default:
              case 0: break;  // return game
              case 1: if (curLevel > 0) saveGame(); break;  // save
              case 2: lostGame(0); ret = 2; playing = 0; break; // exit, like a death
            }
            break;
        }
      }
    }
    if (moved) {
      moved = 0;

      BYTE item = laby->getItem(posX, posY);
      if (item != LABY_ITEM_NULL) {
        t = evalItem(item);
        if (t == 1) { // winner!
          ret = 1;
          playing = 0;
        }
        if (t == 2) { // dead
          ret = 2;
          playing = 0;
        }
      }

      if (lightCounter > 0) {
        lightCounter--;
        lightRadius--;
        if (lightCounter < 0) lightCounter = 0;
        if (lightRadius < LIGHT_DEFRADIUS) lightRadius = LIGHT_DEFRADIUS;
      }
      if (potions.rCounter > 0) {
        potions.rCounter--;
      }
      if (potions.gCounter > 0) {
        potions.gCounter--;
        if (potions.gCounter == 0) {
          keys.up = 0x48;
          keys.dn = 0x50;
          keys.sx = 0x4B;
          keys.dx = 0x4D;
        }
      }
      if (potions.bCounter > 0) {
        potions.bCounter--;
      }
    }

    drawLaby();
    drawStatus();
    mainScr->show();
    delay(GAME_DELAY);
  } while (playing);
  laby->unload();
  return(ret);
}

void parseArgs(int argc, char *argv[]) {
  for (int i=1; i<argc; i++) {
    if (*(argv[i]) == '/') {
      switch (*(argv[i]+1)) {
        case 'l':
        case 'L': {
          hackLevel = max(1, min(atoi(argv[i]+2), LEVELS));
          break;
        }
      }
    }
  }

}

void main(int argc, char *argv[]) {
  int choice, ret;
  char restart = 0, domenu = 1;

  parseArgs(argc, argv);
  initGame();

  do {
    restart = 0;
    if (domenu) {
      if (hackLevel != 0) {
        curLevel = hackLevel;
        hackLevel = 0;
      } else curLevel = 1;
      choice = initMenu();
    }
    switch (choice) {
      case 0: // start game
        ret = playGame((char*)levels[curLevel],0);
        break;
      case 1: // tutorial
        curLevel = 0;
        playGame((char*)levels[0],0);
        ret = 2;
        restart = 1;
        break;
      case 2:  // load game
        ret = loadGame();
        if (ret == 3) {
          ret = playGame(levToLoad, 1);
        }
        break;
      case 3:  // 2do
        showScore();
        ret = 2;
        break;
      case 4:
      default: exitFunc(0);
    }
    switch (ret) {
      case 1: // go next level
        curLevel++;
        if (curLevel >= LEVELS) curLevel = LEVELS-1;
        restart = 1;
        domenu = 0;
        choice = 0;
        break;
      case 2: // continue loop, restart menu
        domenu = 1;
        restart = 1;
        break;
      case 3: // continue loop, leave menu
        domenu = 0;
        restart = 1;
        break;
      default:
        domenu = 1;
        break;
    }
  } while (restart);

  exitFunc(0);
}