/*
Labyrinth class
*/

#include <stdlib.h>
#include <alloc.h>
#include <string.h>
#include <fcntl.h>
#include <dos.h>
#include <io.h>


#define LABY_WALL_E  1
#define LABY_WALL_N  2
#define LABY_WALL_W  4
#define LABY_WALL_S  8

#define LABY_ITEM_NULL       0
#define LABY_ITEM_LIFE       1
#define LABY_ITEM_DEATH      2
#define LABY_ITEM_LIGHT      3
#define LABY_ITEM_POTIONR    4
#define LABY_ITEM_POTIONG    5
#define LABY_ITEM_POTIONB    6
#define LABY_ITEM_MESSAGE   10
#define LABY_ITEM_TELEPORT  16
#define LABY_ITEM_ONEWAY_E  32
#define LABY_ITEM_ONEWAY_N  33
#define LABY_ITEM_ONEWAY_W  34
#define LABY_ITEM_ONEWAY_S  35
#define LABY_ITEM_ENDLEV    50


typedef int ERRPROC(int, int);
typedef ERRPROC *PERRPROC;

typedef struct {
  short x,y;
  char text[256];
} LABY_MSG, *PLABY_MSG;

class Labyrinth {
 public:
  Labyrinth();
  ~Labyrinth();

  char *getName() { return(name); }
  void setName(char *);
  short getWidth() { return(width); }
  short getHeight() { return(height); }
  short getStartX() { return(startX); }
  short getStartY() { return(startY); }
  void setStart(short, short);
  char getWall(short, short);
  void setWall(short, short, char);
  char getItem(short, short);
  void setItem(short, short, char);
  //void addMessage(MSG);
  char *getMessage(short, short);
  void addMessage(short, short, char *);
  void delMessage(short, short);

  void resize(short, short);
  void create(short, short);
  void unload();                // unload from memory any resource allocated by Labyrinth
  void writeToFile(const char *);
  void readFromFile(const char *);
 private:
  short width, height;          // size of labyrinth
  short startX, startY;         // where player starts
  unsigned int n_msg;           // number of messages
  char name[32];                // name of the labyrinth

  char *walls;                  // matrix with walls
  char *items;                  // matrix with items
  LABY_MSG *messages;           // array with messages

  PERRPROC errFunc;

  char checkCoords(short, short);
  int findMessage(short, short);
  void delMessage(int);
  static int errorHandler(int, int);
};

Labyrinth::Labyrinth() {
  width = 0;
  height = 0;
  startX = 0;
  startY = 0;
  name[0] = '\0';
  walls = 0;
  items = 0;
  n_msg = 0;
  messages = 0;
  errFunc = errorHandler;
}

Labyrinth::~Labyrinth() {
  /*if (walls != 0) free(walls);
  if (items != 0) free(items);
  if (messages != 0) free(messages);*/
  unload();
}

void Labyrinth::setName(char *n) {
  strncpy(name, n, 32);
  name[31] = '\0';
}

void Labyrinth::setStart(short x, short y) {
  startX = max(0, min(x, width-1));
  startY = max(0, min(y, height-1));
}

char Labyrinth::getWall(short x, short y) {
  if ((checkCoords(x,y) != 1)||(walls == 0)) return(0);
  return( *(walls+y*width+x) );
}

void Labyrinth::setWall(short x, short y, char wall) {
  if ((checkCoords(x,y) != 1)||(walls == 0)) return;
  *(walls+y*width+x) = wall;
}

char Labyrinth::getItem(short x, short y) {
  if ((checkCoords(x,y) != 1)||(items == 0)) return(0);
  return( *(items+y*width+x) );
}

void Labyrinth::setItem(short x, short y, char item) {
  if (item == LABY_ITEM_MESSAGE) return;
  if ((checkCoords(x,y) != 1)||(items == 0)) return;
  if (*(items+y*width+x) == LABY_ITEM_MESSAGE) delMessage(x,y);
  *(items+y*width+x) = item;
}

//void Labyrinth::addMessage(MSG msg) { }

char *Labyrinth::getMessage(short x, short y) {
  int ret = findMessage(x,y);
  if (ret == -1) return(0);
  return( (messages+ret)->text );
}

void Labyrinth::addMessage(short x, short y, char *text) {
  LABY_MSG ms;
  if (checkCoords(x,y) != 1) return;
  if (*(items+y*width+x) == LABY_ITEM_MESSAGE) { // subst
    int i = findMessage(x,y);
    if (i == -1) return;
    strncpy((messages+i)->text, text, 256);
    (messages+i)->text[255] = '\0';
  } else {
    messages = (PLABY_MSG)realloc(messages, sizeof(LABY_MSG)*(n_msg+1));
    if (messages == 0) {
      errFunc(2, 6);
      return;
    }
    (messages+n_msg)->x = x;
    (messages+n_msg)->y = y;
    strncpy((messages+n_msg)->text, text, 256);
    (messages+n_msg)->text[255] = '\0';
    n_msg++;
    *(items+y*width+x) = LABY_ITEM_MESSAGE;
  }
}

void Labyrinth::delMessage(short x, short y) {
  int ret = findMessage(x,y);
  if (ret == -1) return;
  delMessage(ret);
}

void Labyrinth::resize(short x, short y) {
  int i;
  char *newWalls = (char *)malloc(sizeof(char)*x*y);
  char *newItems = (char *)malloc(sizeof(char)*x*y);
  if ((newWalls == 0)||(newItems == 0)) errFunc(2, 3);
  memset(newWalls, 0, sizeof(char)*x*y);
  memset(newItems, LABY_ITEM_NULL, sizeof(char)*x*y);
  for (int j=0; j<min(y, height); j++)
    for (i=0; i<min(x, width); i++) {
      *(newWalls+j*x+i) = getWall(i,j);
      *(newItems+j*x+i) = getItem(i,j);
    }
  for (i=0; i<n_msg; i++) {
    if ( ((messages+i)->x >= x) || ((messages+i)->y >= y) ) delMessage(i);
  }
  width = x;
  height = y;
  if (walls != 0) free(walls);
  if (items != 0) free(items);
  walls = newWalls;
  items = newItems;
  setStart(startX, startY);
}

void Labyrinth::create(short x, short y) {
  width = x;
  height = y;
  walls = (char *)realloc(walls, sizeof(char)*width*height);
  items = (char *)realloc(items, sizeof(char)*width*height);
  if ((walls == 0)||(items == 0)) errFunc(2, 3);
  memset(walls, 0, sizeof(char)*width*height);
  memset(items, LABY_ITEM_NULL, sizeof(char)*width*height);
}

void Labyrinth::unload() {
  if (walls != 0) { free(walls); walls = 0; }
  if (items != 0) { free(items); items = 0; }
  if (messages != 0) { free(messages); messages = 0; }
  n_msg = 0;
  width = 0;
  height = 0;
  name[0] = '\0';
}

void Labyrinth::writeToFile(const char *filename) {
  int handle;
  char buff[10];

  handle = _creat(filename, FA_ARCH);
  if (handle == -1) errFunc(2, 1);
  if (_write(handle, "LABLEV", 6) == -1) errFunc(2,2);
  *((short*)buff) = 0;
  _write(handle, buff, 2);
  _write(handle, &width, 2);
  _write(handle, &height, 2);
  _write(handle, &startX, 2);
  _write(handle, &startY, 2);
  _write(handle, name, 32);
  if (walls != 0) _write(handle, walls, width*height);
  buff[0] = 0xFF;
  _write(handle, buff, 1);
  if (items != 0) _write(handle, items, width*height);
  _write(handle, buff, 1);
  _write(handle, &n_msg, 2);
  for (int i=0; i<n_msg; i++) {
    int len = strlen((messages+i)->text)+1;
    _write(handle, &((messages+i)->x), 2);
    _write(handle, &((messages+i)->y), 2);
    _write(handle, &len, 2);
    _write(handle, (messages+i)->text, len);
  }
  _write(handle, buff, 1);
  _close(handle);
}

void Labyrinth::readFromFile(const char *filename) {
  int handle, rd;
  short i;
  char buff[64];

  handle = _open(filename, O_RDONLY);
  if (handle == -1) errFunc(2,4);
  rd = _read(handle, buff, 8);
  if (rd != 8) errFunc(2,5);
  buff[6] = '\0';
  if (strcmp(buff, "LABLEV") != 0) errFunc(2,5);
  unload(); // frees previously allocated labyr.
  rd = _read(handle, &width, 2); if (rd != 2) errFunc(2,5);
  rd = _read(handle, &height, 2); if (rd != 2) errFunc(2,5);
  rd = _read(handle, &startX, 2); if (rd != 2) errFunc(2,5);
  rd = _read(handle, &startY, 2); if (rd != 2) errFunc(2,5);
  rd = _read(handle, name, 32); if (rd != 32) errFunc(2,5);
  name[31] = '\0';
  create(width, height);
  rd = _read(handle, walls, width*height); if (rd != width*height) errFunc(2,5);
  rd = _read(handle, buff, 1); if (rd != 1) errFunc(2,5);
  //if (buff[0] != 0xFF) errFunc(2,5);
  rd = _read(handle, items, width*height); if (rd != width*height) errFunc(2,5);
  rd = _read(handle, buff, 1); if (rd != 1) errFunc(2,5);
  //if (buff[0] != 0xFF) errFunc(2,5);
  rd = _read(handle, &i, 2); if (rd != 2) errFunc(2,5);
  for (int j=0; j<i; j++) {
    short x,y,l;
    rd = _read(handle, &x, 2); if (rd != 2) errFunc(2,5);
    rd = _read(handle, &y, 2); if (rd != 2) errFunc(2,5);
    rd = _read(handle, &l, 2); if (rd != 2) errFunc(2,5);
    if (l != 0) {
      char *str = (char*)malloc(l+2);
      if (str == 0) errFunc(2,7);
      rd = _read(handle, str, l); if (rd != l) errFunc(2,5);
      setItem(x,y, LABY_ITEM_NULL); // remove msg if present
      addMessage(x,y, str);
      free(str);
    }
  }
  rd = _read(handle, buff, 1); if (rd != 1) errFunc(2,5);
  //if (buff[0] != 0xFF) errFunc(2,5);
  _close(handle);
}

char Labyrinth::checkCoords(short x, short y) { // ret 0 if not ok
  if ((x < 0)||(y < 0)||(x >= width)||(y >= height)) return(0);
  return(1);
}

void Labyrinth::delMessage(int i) {
  if (i >= n_msg) return;
  *(items+(messages+i)->y*width+(messages+i)->x) = LABY_ITEM_NULL;
  if (i < n_msg-1) *(messages+i) = *(messages+n_msg-1);
  messages = (PLABY_MSG)realloc(messages, sizeof(LABY_MSG)*(n_msg-1));
  n_msg--;
}

int Labyrinth::findMessage(short x, short y) { // ret -1 if not found
  int ret = -1;
  if (getItem(x,y) != LABY_ITEM_MESSAGE) return(-1);
  for (int i=0; i<n_msg; i++) {
    PLABY_MSG pm = (messages+i);
    if ((pm->x == x)&&(pm->y == y)) {
      ret = i;
      break;
    }
  }
  return(ret);
}

int Labyrinth::errorHandler(int src, int code) {
  src;
  code;
  abort();
  return(0);
}