/* menu class */

#ifndef MENU_H
#define MENU_H

#include <string.h>
#include <vga.h>
#include "vgafont.h"

typedef struct {
  int id, len;
  char *text;
} MENU_ITEM, *PMENU_ITEM;

class Menu {
 public:
  Menu(VirtualScreen *, VGACOORD, VGACOORD, VGACOORD, VGACOORD);
  ~Menu();

  void setTextColor(BYTE col) { textColor = col; }
  void setBackColor(BYTE col) { backColor = col; }
  void setBdrColor(BYTE col) { bdrColor = col; }
  void setSelColor(BYTE col) { selColor = col; }
  void setColors(BYTE, BYTE, BYTE, BYTE);
  void setFont(Font *);
  void setCenter();
  void addItem(int, char *);
  void setSelected(int);
  void show(char);
  int choose();
 private:
  char visible;
  VGACOORD x1,y1, x2,y2;
  short width, height, cellHeight;
  int selected, n_items;
  BYTE textColor, backColor, bdrColor, selColor; // -1 is transparent
  VirtualScreen *screen;
  Font *font;

  BYTE far *saveBackData;
  MENU_ITEM *items;

  PERRPROC errFunc;

  void drawPanel(int, char);
  int findItem(int);
  static int errorHandler(int, int);
};

Menu::Menu(VirtualScreen *virscr, VGACOORD x1, VGACOORD y1, VGACOORD x2, VGACOORD y2) {
  this->x1 = x1;
  this->y1 = y1;
  this->x2 = x2;
  this->y2 = y2;
  width = x2-x1+1;
  height = y2-y1+1;
  cellHeight = 16;
  visible = 0;
  textColor = 7;
  backColor = 0;
  bdrColor = 0;
  selColor = 15;
  font = 0;
  errFunc = errorHandler;
  screen = virscr;
  if (screen == 0) errFunc(4,1);
  saveBackData = (BYTE far *)farmalloc((long)width*height);
  if (saveBackData == 0) errFunc(4,2);
  screen->getRect(x1,y1,x2,y2, saveBackData);
  selected = -1;
  n_items = 0;
  items = 0;
}

Menu::~Menu() {
  show(0);
  if (items != 0) {
    for (int i=0; i<n_items; i++) free((items+i)->text);
    free(items);
  }
  if (saveBackData != 0) farfree(saveBackData);
}

void Menu::setColors(BYTE txt, BYTE back, BYTE bdr, BYTE sel) {
  textColor = txt;
  backColor = back;
  bdrColor = bdr;
  selColor = sel;
}

void Menu::setFont(Font *fnt) {
  font = fnt;
  cellHeight = font->getCharHeight('A');
}

void Menu::setCenter() {
  x1 = (VGA_WIDTH-width)>>1;
  x2 = x1+width-1;
  y1 = (VGA_HEIGHT-height)>>1;
  y2 = y1+height-1;
  screen->getRect(x1,y1,x2,y2, saveBackData);
}

void Menu::addItem(int id, char *text) {
  items = (PMENU_ITEM)realloc(items, sizeof(MENU_ITEM)*(n_items+1));
  if (items == 0) errFunc(4,2);
  (items+n_items)->id = id;
  (items+n_items)->len = strlen(text);
  (items+n_items)->text = strdup(text);
  n_items++;
}

void Menu::setSelected(int id) {
  int idx = findItem(id);
  if (idx == -1) return;
  selected = idx;
}

void Menu::show(char onoff) {
  int i;
  if (onoff) {
    if (backColor != 0xFF) screen->fRect(x1,y1, x2,y2, backColor);
    else screen->putRect(x1,y1,x2,y2, saveBackData);
    if (bdrColor != 0xFF) {
      screen->line(x1,y1, x2,y1, bdrColor);
      screen->line(x1,y2, x2,y2, bdrColor);
      screen->line(x1,y1, x1,y2, bdrColor);
      screen->line(x2,y1, x2,y2, bdrColor);
    }
    for (i=0; i<n_items; i++) drawPanel(i, 0);
    visible = 1;
  } else {
    if (visible) {
      screen->putRect(x1,y1,x2,y2, saveBackData);
      visible = 0;
    }
  }
}

int Menu::choose() {
  char key, quit = 0, useFastCopy = 0;
  int oldsel, ch = -1;
  BYTE *saveBackPanel;

  if (n_items == 0) return(-1);
  saveBackPanel = (BYTE *)malloc((width-2)*(cellHeight+1));
  useFastCopy = (saveBackPanel != 0);

  oldsel = selected;
  selected = -1;
  show(1);
  selected = (oldsel==-1)?0:oldsel;
  if (useFastCopy) screen->getRect(x1+1, y1+1+cellHeight*selected, x2-1, y1+cellHeight*(1+selected), saveBackPanel);
  drawPanel(selected, 0);
  screen->show();
  do {
    key = getch();
    if (key == 0) {
      key = getch();
      switch (key) {
        case 72:  //up
          if (useFastCopy) {
            screen->putRect(x1+1, y1+1+cellHeight*selected, x2-1, y1+cellHeight*(1+selected), saveBackPanel);
          }
          selected = (selected+n_items-1)%n_items;
          if (useFastCopy) {
            screen->getRect(x1+1, y1+1+cellHeight*selected, x2-1, y1+cellHeight*(1+selected), saveBackPanel);
            drawPanel(selected,0);
          } else show(1);
          break;
        case 80:  //down
          if (useFastCopy) {
            screen->putRect(x1+1, y1+1+cellHeight*selected, x2-1, y1+cellHeight*(1+selected), saveBackPanel);
          }
          selected = (selected+1)%n_items;
          if (useFastCopy) {
            screen->getRect(x1+1, y1+1+cellHeight*selected, x2-1, y1+cellHeight*(1+selected), saveBackPanel);
            drawPanel(selected,0);
          } else show(1);
          break;
      }
    } else {
      switch (key) {
        case 0x0D:
          ch = (items+selected)->id;
          quit = 1;
          break;
        case 0x1B:
          quit = 1;
          ch = -1; // abort
          break;
      }
    }
    screen->show();
  } while (!quit);
  if (useFastCopy) free(saveBackPanel);
  return(ch);
}

void Menu::drawPanel(int index, char drawBack) {
  if (selected == index) screen->fRect(x1+1,y1+1+cellHeight*selected, x2-1,y1+cellHeight*(1+selected), selColor);
  else if (drawBack) screen->fRect(x1+1,y1+1+cellHeight*index, x2-1,y1+cellHeight*(1+index), backColor);
  if (font == 0) {
    screen->text(x1+(width-8*(items+index)->len)/2,y1+1+cellHeight*index, (items+index)->text, 1, textColor);
   } else {
     BYTE oldc = font->getTextColor();
     int oldalign = font->align;
     font->setTextColor(textColor);
     font->align = 1; // center
     font->printRect(x1+1,y1+1+cellHeight*index, x2-1,y1+cellHeight*(1+index), (items+index)->text);
     font->setTextColor(oldc);
     font->align = oldalign;
   }
}

int Menu::findItem(int id) {
  int ret = -1;
  for (int i=0; i<n_items; i++) {
    if ((items+i)->id == id) {
      ret = i;
      break;
    }
  }
  return(ret);
}

int Menu::errorHandler(int src, int code) {
  src;
  code;
  abort();
  return(0);
}

#endif