#include <dos.h>
#include <mem.h>


#define SCORE_FILENAME   "laby.top"
#define SCORE_MAXITEMS   30

typedef struct {
  short level;
  char name[14];
} SCORE, *PSCORE;

int cmpScore(const void *s1, const void *s2) {
  if (((PSCORE)s1)->level < ((PSCORE)s2)->level) return(-1);
  return( ((PSCORE)s1)->level > ((PSCORE)s2)->level );
}

class TopScores {
 public:
  TopScores();
  ~TopScores();
  int getItems() { return(countItems); }
  PSCORE getItem(int n);
  void readItems();
  void writeItems();
  void addItem(int level, char *name);
  void sort();
 private:
  int countItems;
  char sorted;
  SCORE *items;
  int createEmpty();
};

TopScores::TopScores() {
  countItems = 0;
  sorted = 0;
  items = (PSCORE)malloc(SCORE_MAXITEMS*sizeof(SCORE));
}

TopScores::~TopScores() {
  free(items);
}

PSCORE TopScores::getItem(int n) {
  if (n >= countItems) n = countItems;
  return(items+n);
}

void TopScores::readItems() {
  struct {
    char sign[6];
    short version, items, size;
  } head;
  char *rec;
  int l1, l2;

  int handle = _open(SCORE_FILENAME, O_RDONLY);
  if (handle == -1) {
    handle = createEmpty();
    if (handle == -1) return;
  }
  _read(handle, &head,sizeof(head));
  if (strcmp(head.sign, "LABTOP") != 0) {
    _close(handle);
    return;
  }
  rec = (char*)malloc(head.size);
  countItems = min(SCORE_MAXITEMS, head.items);
  sorted = 1;
  l2 = 1000;
  for (int i=0; i<countItems; i++) {
    _read(handle, rec, head.size);
    memcpy(items+i, rec, min(sizeof(SCORE),head.size));
    l1 = (items+i)->level;
    if (l1 > l2) sorted = 0;
    l2 = l1;
  }
  free(rec);
  _close(handle);
}

int TopScores::createEmpty() {
  char buff[30] = "LABTOP";
  int t;
  int handle = _creat(SCORE_FILENAME, FA_ARCH);
  if (handle == -1) return(handle);
  _write(handle, buff, 6);
  buff[0] = VERSION_LO;
  buff[1] = VERSION_HI;
  _write(handle, buff, 2);
  t = 0;
  _write(handle, &t, 2);
  t = sizeof(SCORE);
  _write(handle, &t, 2);
  buff[0] = 0xFF;
  _write(handle, buff, 1);
  lseek(handle, 0, SEEK_SET);
  return(handle);
}

void TopScores::writeItems() {
  struct {
    char sign[6];
    short version, items, size;
  } head = {"LABTOP",(VERSION_HI<<8)|VERSION_LO,countItems,sizeof(SCORE)};

  if (!sorted) sort();
  int handle = _creat(SCORE_FILENAME, FA_ARCH);
  _write(handle, &head, sizeof(head));
  for (int i=0; i<countItems; i++) _write(handle, items+i, sizeof(SCORE));
  head.sign[0] = 0xFF;
  _write(handle, head.sign, 1);
  _close(handle);
}

void TopScores::addItem(int level, char *name) {
  if (countItems < SCORE_MAXITEMS) {
    (items+countItems)->level = level;
    strncpy((items+countItems)->name, name, 13);
    (items+countItems++)->name[13] = '\0';
    sorted = 0;
  } else {
    if (!sorted) sort();
    if (level > (items+countItems-1)->level) {
      (items+countItems-1)->level = level;
      strncpy((items+countItems-1)->name, name, 13);
      (items+countItems-1)->name[13] = '\0';
      sort();
    }
  }
}

void TopScores::sort() {
  if ((countItems == 0)||sorted) return;
  qsort(items, countItems, sizeof(SCORE), cmpScore);
  sorted = 1;
}