/* MASTER.H */

/* Functions for managing mastermind */

#ifndef MASTER_H
#define MASTER_H

#include <string.h>

#define MM_MAX_LENGTH		10
#define MM_MAX_SYMBOLS		20
#define MM_HASHTYPE		unsigned long long

int MM_SetLength(int l);
int MM_SetSymbols(char *symbols);
char *MM_GetSymbols();
void MM_SetCheckSymbols(char *symbols);
int MM_Check(char *master, char *slave, char *result);
MM_HASHTYPE MM_CountAll();
MM_HASHTYPE MM_Hash(char *slave);
char *MM_ReverseHash(MM_HASHTYPE hash, char *result);

/* ----------------- */
/* next funcs will be private */

int MM_Length = 4;
int MM_SymbolLength = 9;
char MM_Symbols[MM_MAX_SYMBOLS+1] = "1234567890ABCDEFGHIJ";
char MM_CheckSymbols[4] = "XO-";

int MM_SetLength(int l) {
  MM_Length = (l>MM_MAX_LENGTH) ? MM_MAX_LENGTH : l;
  return MM_Length;
}

int MM_SetSymbols(char *symbols) {
  int i = 0;
  if (!symbols) return MM_SymbolLength;
  while (*symbols && i<MM_MAX_SYMBOLS) MM_Symbols[i++] = *(symbols++);
  MM_SymbolLength = i;
  MM_Symbols[i] = '\0';
  return i;
}

char *MM_GetSymbols() {
  return(MM_Symbols);
}

void MM_SetCheckSymbols(char *symbols) {
  int i = 0;
  if (!symbols) return;
  while (*symbols && i<3) MM_CheckSymbols[i++] = *(symbols++);
}

int MM_Check(char *master, char *slave, char *result) {
  int i,j, countok=0, countmv=0;
  char workboard[MM_Length], userboard[MM_Length];

  if (!master || !slave) return 0;
  memcpy(workboard, master, MM_Length);
  memcpy(userboard, slave, MM_Length);    

  for (i=0; i<MM_Length; i++) {
    if (slave[i] == workboard[i]) {
      countok++;
      userboard[i] = workboard[i] = 0;
    }
  }

  for (i=0; i<MM_Length; i++) {
    if (userboard[i] == 0) continue;
    char v = userboard[i];
    for (j=0; j<MM_Length; j++) {
      if (workboard[j] == v) {
        countmv++;
        userboard[i] = workboard[j] = 0;
        break;
      }
    }
  }
  
  for (i=0,j=0; i<countok; i++,j++) result[j] = MM_CheckSymbols[0];
  for (i=0; i<countmv; i++,j++) result[j] = MM_CheckSymbols[1];
  for (i=0; i<MM_Length-(countok+countmv); i++,j++) result[j] = MM_CheckSymbols[2];
  result[MM_Length] = '\0';
  return (countok == MM_Length);
}

MM_HASHTYPE MM_CountAll() {
  int i;
  MM_HASHTYPE r = 1;
  for (i=0; i<MM_Length; i++) r *= MM_SymbolLength;
  return r;
}

MM_HASHTYPE MM_Hash(char *slave) {
  int i,j;
  MM_HASHTYPE h = 0;
  if (!slave) return 0;
  for (i=0; i<MM_Length; i++) {
    for (j=0; j<MM_SymbolLength; j++) if (MM_Symbols[j] == slave[i]) break;
    if (j == MM_SymbolLength) return 0;
    h = h*MM_SymbolLength+j;
  }
  return h+1;
}

char *MM_ReverseHash(MM_HASHTYPE hash, char *result) {
  int i;
  if (!result) return result;
  if (hash == 0) {
    *result = '\0';
    return result;
  } else hash--;
  
  for (i=MM_Length-1; i>=0; i--) {
    result[i] = MM_Symbols[hash % MM_SymbolLength];
    hash /= MM_SymbolLength;
  }
  result[MM_Length] = '\0';
  return result;
}

#endif
