#include "telefonata.h"
#include "bolletta.h"

Bolletta::Nodo::Nodo(): next(0) {
}

Bolletta::Nodo::Nodo(const Telefonata& t, Nodo *nex): 
  next(nex), data(t) {
}

Bolletta::Bolletta():
  head(0), pointer(0) {
}

Bolletta::Bolletta(const Bolletta& b):
  head(copy(b.head)), pointer(0) {
}

Bolletta::~Bolletta() {
  destroy(head);
}

bool Bolletta::is_empty() const {
  return (head == 0);
}

void Bolletta::add_head(const Telefonata& tel) {
  head = new Nodo(tel, head);
  pointer = head;
}

Telefonata Bolletta::del_head() {
  //undefined if empty
  //if (is_empty()) return 0;
  Nodo *oldhead = head;
  head = oldhead->next;
  Telefonata t = oldhead->data;
  delete oldhead;
  pointer = head;
  return t;
}

void Bolletta::del_call(const Telefonata& tel) {
  Nodo *p = head, *prec = 0;
   while (p && !(p->data == tel)) {
    prec = p;
    p = p->next;
  }
  if (p) {
    if (prec) {
      prec->next = p->next;
    } else {
      head = p->next;
    }
    delete p;
  }
  pointer = head;
}

Telefonata* Bolletta::get_next() {
  Telefonata *t = 0;
  if (pointer) {
    t = &(pointer->data);
    pointer = pointer->next;
  }
  return t;
}

Bolletta::Nodo *Bolletta::copy(Nodo *start) {
  if (!start) return 0;
  Nodo* first = new Nodo();
  first->data = start->data;
  Nodo* tail = first;
  while (start->next) {
    tail->next = new Nodo();
    start = start->next;
    tail = tail->next;
    tail->data = start->data;
  }
  tail->next = 0;
  return first;
}

void Bolletta::destroy(Nodo *start) {
  while (start) {
    Nodo *n = start->next;
    delete start;
    start = n;
  }
}

Bolletta& Bolletta::operator=(const Bolletta& b) {
  if (this != &b) {
    destroy(head);
    head = copy(b.head);
  }
  return *this;
}

ostream& operator<<(ostream& os, Bolletta& b) {
  Telefonata* t = b.get_next();
  os << "BOLLETTA:\n";
  while (t) {
    os << (*t);
    t = b.get_next();
  }
  return os;
}
