#ifndef _POLINOMIO_H
#define _POLINOMIO_H

#include <iostream>
#include "monomio.h"

using std::ostream;

class Polinomio {
 private:
  class Nodo;
 public:
  class Iterator;
  friend class Iterator;
  
  Polinomio();
  Polinomio(int coeff, int esp);
  Polinomio(const Monomio& m);
  bool is_null() const;
  Iterator begin() const;
  Iterator end() const;
  
  void debug_printlist();
  
  Monomio& operator [] (Iterator it) const;
  int operator () (int x) const;
  friend Polinomio operator + (const Polinomio& p1, const Polinomio& p2);
  friend Polinomio operator - (const Polinomio& p1, const Polinomio& p2);
  friend Polinomio operator * (int a, const Polinomio& p);
  
  class Iterator {
   public:
    bool operator == (const Iterator& it) const;
    bool operator != (const Iterator& it) const;
    Iterator& operator ++ ();
    Iterator operator ++ (int);
   private:
    friend class Polinomio;
    Nodo *ptr;
  };
  
 private:
  
  class SmartP {
   public:
    SmartP(Nodo *p);
    SmartP(const SmartP& sp);
    ~SmartP();
    SmartP& operator=(const SmartP& sp);
    Nodo& operator*() const;
    Nodo* operator->() const;
    bool operator==(const SmartP& sp) const;
    bool operator!=(const SmartP& sp) const;
    Nodo *ptr;
   private:
  };
  
  class Nodo {
   public:
    Nodo();
    Nodo(const Monomio& m, const SmartP& n);
    Monomio mon;
    int refs;
    SmartP next;
   private:
  };
   
  SmartP head;

  void reduct();
  Monomio del_head();
  Monomio del_monomio(SmartP);
  void add_head(const Monomio&);
  void add_tail(const Monomio&);
};

ostream& operator << (ostream& os, const Polinomio& p);

#endif
