/* Contenitore.h */

#ifndef CONTENITORE_H
#define CONTENITORE_H

#include <iostream>
#include <stdexcept>

// dichiarazioni incomplete
template <class T> class Contenitore;

// dichiarazioni incomplete per classi friend
template <class T> std::ostream &operator << (std::ostream& os, const Contenitore<T> &c);


/*
  Gestisce una collezioni di oggetti tramite una lista a doppio link
*/
template <class T>
class Contenitore {
 private:

  
  // ---- SOTTO CLASSI ----
  
  // un elemento della lista
  class ListNode {
   public:
    // amicizie con gli iteratori
    friend class Contenitore;
    friend class Contenitore::iterator;
    friend class Contenitore::const_iterator;
    // costruttori
    ListNode(T *poly = NULL, ListNode *prev = 0, ListNode *next = 0);
    ListNode(const ListNode& ln, ListNode *prev = 0, ListNode *next = 0);
    // distruttore
    ~ListNode();
    // metodi statici
    static void delete_forward(ListNode *start);
    static void delete_backward(ListNode *start);
   private:
    // dato da memorizzare
    T *poly;
    // precedente e prossimo elemento
    ListNode *prev, *next;
  };

  // classe virtuale base per gli iteratori
  class base_iterator {
   public:
    // costruttori
    base_iterator(ListNode *n = 0): node(n) {};
    // operatori
    bool operator == (const base_iterator &i) const { return node == i.node; }
    bool operator != (const base_iterator &i) const { return node != i.node; }
    // metodi
    bool is_null() const { return (node == NULL); }
   protected:
    // puntatore ad un nodo nella lista
    ListNode *node;
  };

 public:


  // ---- COSTRUTTORI ----
  
  // vuoto
  Contenitore();
  
  // copia (profonda)
  Contenitore(const Contenitore<T>& c);


  // ---- DISTRUTTORE ----
  ~Contenitore();


  // ---- CLASSI ITERATORI ----
  
  // iteratore read/write
  class iterator: public base_iterator {
   public:
    // dichiarazioni di amicizia
    friend class Contenitore;
    // costruttori
    iterator(ListNode *n = 0): base_iterator(n) {};
    // operatori
    T& operator *() const { return *(this->node->poly); }
    T* operator ->() const { return this->node->poly; }
    iterator &operator ++() { // prefisso
      if (this->node) this->node = this->node->next;
      return *this;
    }
    iterator  operator ++(int) { // postfisso
      iterator ret = *this;
      if (this->node) this->node = this->node->next;
      return ret;
    }
    iterator &operator --() { // prefisso
      if (this->node) this->node = this->node->prev;
      return *this;
    }
    iterator  operator --(int) { // postfisso
      iterator ret = *this;
      if (this->node) this->node = this->node->prev;
      return ret;
    }
  };
  
  // iteratore read-only
  class const_iterator : public base_iterator {
   public:
    // dichiarazioni di amicizia
    friend class Contenitore;
    // costruttori
    const_iterator(ListNode *n = 0): base_iterator(n) {};
    // operatori
    const T& operator *() const { return *(this->node->poly); }
    const T* operator ->() const { return this->node->poly; }
    const_iterator &operator ++() { // prefisso
      if (this->node) this->node = this->node->next;
      return *this;
    }
    const_iterator  operator ++(int) { // postfisso
      const_iterator ret = *this;
      if (this->node) this->node = this->node->next;
      return ret;
    }
    const_iterator &operator --() { // prefisso
      if (this->node) this->node = this->node->prev;
      return *this;
    }
    const_iterator  operator --(int) { // postfisso
      const_iterator ret = *this;
      if (this->node) this->node = this->node->prev;
      return ret;
    }
  };
  

  // ---- OPERATORI ----
  
  // assegnazione (profonda)
  Contenitore<T>& operator = (const Contenitore<T>& c);
  
  // indexing con iteratori
        T& operator [] (      iterator i)       { return *i; }
  const T& operator [] (const_iterator i) const { return *i; }
  
  // output su ostream
  friend std::ostream &operator << <T>(std::ostream& os, const Contenitore<T> &c);
  

  // ---- METODI ----
  
  // iteratore all'inizio della lista
        iterator begin()        { return       iterator(head); }
  const_iterator cbegin() const { return const_iterator(head); }
  
  // iteratore alla fine della lista
        iterator end()          { return       iterator(tail); }
  const_iterator cend()   const { return const_iterator(tail); }
  
  // elementi nella lista
  inline int length() const { return size; }
  
  // svuota la lista
  void clear();
  
  // inserisce in testa
  void add_head(const T &p);

  // inserisce in coda
  void add_tail(const T &p);
  
  // restituisce la testa
        T& get_head()       throw(std::out_of_range);
  const T& get_head() const throw(std::out_of_range);
  
  // restituisce la coda
        T& get_tail()       throw(std::out_of_range);
  const T& get_tail() const throw(std::out_of_range);
  
  // cancella la testa e la restiruisce
  T del_head() throw(std::out_of_range);

  // cancella la coda e la restiruisce
  T del_tail() throw(std::out_of_range);

  // cancella l'elemento puntato da it
  void del(const iterator& it) throw(std::invalid_argument);
  void del(const const_iterator& it) throw(std::invalid_argument);
  
  // converte la lista in un array di elementi
  // chi chiama il metodo deve poi liberare la memoria
  T* to_array() const;
  
  // esegue una ricerca usando una funzione di callback
  Contenitore<iterator> find( bool(*matchf)(const T*, const void *), const void *);
  
 private:
 
  // puntatori alla testa e alla coda
  ListNode *head, *tail;
  
  // numero di elementi
  int size;
  
};

// definizioni dei membri
#include "Contenitore-def.h"

#endif
