/* Matrix3D.h */

#ifndef MATRIX3D_H
#define MATRIX3D_H

#include <iostream>
#include <stdexcept>

#include "Point3D.h"

// dichiarazioni incomplete
template <class Type> class Matrix3D;

// dichiarazioni incomplete per funzioni friend
template <class Type>
std::ostream &operator << (std::ostream& os, const Matrix3D<Type> &);


/*
  Classe Matrix3D
    Una matrice 3x3, con funzioni minimali, utile per gestire trasformazioni
    nello spazio affine/euclideo.
*/
template <class Type>
class Matrix3D {
 
 public:
  

  // ---- COSTRUTTORI ----
  
  // vuoto o lista di punti
  Matrix3D(const Type* list = NULL);
  
  // lista di punti come parametri
  Matrix3D(Type, Type, Type, Type, Type, Type, Type, Type, Type);
  
  // le 3 colonne
  Matrix3D(const Point3D<Type>&, const Point3D<Type>&, const Point3D<Type>&);
  
  // copia, anche con tipo diverso
  template <class Type2>
  Matrix3D(const Matrix3D<Type2>&);
  

  // ---- DISTRUTTORE ----
  // ... standard
  

  // ---- OPERATORI ----
  
  // confronto
  bool operator == (const Matrix3D<Type> &) const;
  bool operator != (const Matrix3D<Type> &) const;

  // somma, differenza e prodotto
  Matrix3D<Type> operator +(const Matrix3D<Type> &) const;
  Matrix3D<Type> operator -(const Matrix3D<Type> &) const;
  Matrix3D<Type> operator *(const Matrix3D<Type> &) const;
  
  // prodotto con punto
  Point3D<Type> operator *(const Point3D<Type> &) const;
  
  // assegnazione, anche con tipo diverso
  template <class Type2>
  Matrix3D<Type>& operator =(const Matrix3D<Type2> &);
  
  // output su ostream
  friend std::ostream &operator << <Type>(std::ostream& os, const Matrix3D<Type> &);
  

  // ---- METODI ----
  
  // restituisce l'elemento in posizione (i,j)
  Type get(int i, int j) const throw(std::invalid_argument);
  
  // calcola il determinante
  Type det() const;
  
  // traspone la matrice
  Matrix3D<Type>& transpose();
  
  // esegue il complemento algebrico
  Matrix3D<Type>& complement();
  
  // inverte la matrice (se possibile)
  Matrix3D<Type>& invert() throw(std::domain_error);
  

  // ---- METODI STATICI ----
  
  // calcola la matrice di rotazione
  static Matrix3D<Type> rotation_matrix(double, double, double);
 
 protected:
 
  // gli elementi della matrice
  Type data[9];
  
  // imposta a v l'elemento in posizione (i,j)
  void set(int i, int j, Type v) throw(std::invalid_argument);
 
 private:
  
};

// definizioni dei membri
#include "Matrix3D-def.h"

// qualche tipo comodo
typedef Matrix3D<int> Matrix3Di;
typedef Matrix3D<double> Matrix3Dd;

// matrice nulla e unitaria (interi)
const Matrix3Di M3Di_ZERO(0,0,0, 0,0,0, 0,0,0);
const Matrix3Di M3Di_IDENTITY(1,0,0, 0,1,0, 0,0,1);

// matrice nulla e unitaria (double)
const Matrix3Dd M3Dd_ZERO(0,0,0, 0,0,0, 0,0,0);
const Matrix3Dd M3Dd_IDENTITY(1,0,0, 0,1,0, 0,0,1);

#endif
