/* QColorButton.cpp */

#include "QColorButton.h"

#include <QPixmap>
#include <QPainter>
#include <QColorDialog>

// ---- COSTRUTTORI ----------------------------------------------------------
QColorButton::QColorButton(QColor color, QWidget *parent): 
 QPushButton(parent), color(color) {
  
  setIcon(gen_icon());
  setText(color.name());
  connect(this, SIGNAL(clicked()), this, SLOT(choosecolor()));
  colorChanged(color);
}

// ---- METODI PUBBLICI ------------------------------------------------------

// imposta il colore
void QColorButton::setColor(QColor c) {
  if (c != color) {
    color = c;
    setIcon(gen_icon());
    setText(color.name()); 
    colorChanged(c);
  }
}


// ---- METODI PRIVATI -------------------------------------------------------

// apre la finestra di dialogo per la scelta del colore
void QColorButton::choosecolor() {
  QColor newcol = QColorDialog::getColor(color, this);
  if (newcol.isValid()) setColor(newcol);
}

// genera l'icona
QIcon QColorButton::gen_icon() const {
  QPixmap pm(iconSize());
  pm.fill(Qt::transparent);
  QPainter p(&pm);
  p.setPen(QPen(Qt::black, 0.0, Qt::SolidLine, Qt::FlatCap));
  p.setBrush(color);
  // this is bad! don't works like I want!
  //p.drawRoundRect(4,4, pm.width()-8, pm.height()-8);
  int border = 3;
  QPoint plist[] = {
    QPoint(border+2,border), QPoint(pm.width()-border-3,border),
    QPoint(pm.width()-border-1,border+2), QPoint(pm.width()-border-1,pm.height()-border-3),
    QPoint(pm.width()-border-3,pm.height()-border-1), QPoint(border+2,pm.height()-border-1),
    QPoint(border,pm.height()-border-3), QPoint(border,border+2)
  };
  p.drawConvexPolygon(plist, 8);
   
  return QIcon(pm);
}
