/* QDrawSolid.h */

#ifndef QDRAWSOLID_H
#define QDRAWSOLID_H

#include <QtGui>
#include "Polyhedron.h"

/* 
  Classe QDrawSolid:
   Widget che disegna un poliedro (vertici) rotante.
   E' possibile ruotare il solido trascinandolo con il tasto sinistro del 
   mouse, oppure variare lo zoom trascinando verticalmente col tasto destro.
*/
class QDrawSolid: public QWidget {
 
 // macro standard per i QObject
 Q_OBJECT
 
 public:
  
  // ---- COSTRUTTORI ----
  QDrawSolid(QWidget *parent = 0);
  
  
  // ---- DISTRUTTORI ----
  ~QDrawSolid();
  
  
  // ---- METODI ----

  // imposta la prospettiva
  // Se vero viene usata una (pseudo)proiezione rispetto ad un punto, 
  // se falso viene usata una proiezione parallela
  inline void setPerspective(bool ps) { perspective = ps; };
  
  // legge e imposta il fattore di scala
  inline double getScale() const { return scale; }
  inline void setScale(double s) { scale = s; };
  
  // restituisce le dimensioni consigliate per il Widget
  QSize sizeHint() const;
  
 
 public slots:
  // ---- SLOT PUBBLICI ----
   
  // imposta il poliedro
  void setPolyhedron(const Polyhedron& p);
  
  // imposta il colore
  void setColor(const QColor& c);
  
  // cancella il poliedro
  void clear();
 
 
 protected:
  // ---- METODI PROTETTI ----
  
  // disegna il solido
  void paintEvent(QPaintEvent *event);
  
  // gestisce il timer (ogni quanto ruotare il solido)
  void timerEvent(QTimerEvent *event);
  
  // gestiscono i movimenti del mouse
  void mousePressEvent(QMouseEvent *event);
  void mouseMoveEvent(QMouseEvent *event);
  void mouseReleaseEvent(QMouseEvent *event);
  
 
 private:
  // ---- MEMBRI PRIVATI ----
  
  // flag vari
  bool perspective, paused, mouse_scaling;
  
  // fattore di scala
  double scale;
  
  // altre variabili interne
  QPoint clickpos;
  QColor polycolor;
  
  // il poliedro
  Polyhedron *poly;
  
  // il centro del poliedro
  Point3Dd center;
  
  // il timer
  QBasicTimer timer;
  
  // converte in modo molto grezzo (ma efficace) punti tridimensionali
  // in punti nel piano
  void to2D(const Point3Dd &p, int *x, int *y) const;
  
  // confronta 2 punti usando la coordinata z
  static int point_zcomp(const void*, const void*);
  
};

#endif
