/* Contenitore-def.h */


// ---- CLASSE LISTNODE ------------------------------------------------------

// costruttore vuoto o standard
template <class T>
Contenitore<T>::ListNode::ListNode(T *poly, Contenitore::ListNode *prev, Contenitore::ListNode *next):
 poly(poly), prev(prev), next(next) {
}

// costruttore di copia
template <class T>
Contenitore<T>::ListNode::ListNode(const ListNode& ln, Contenitore::ListNode *prev, Contenitore::ListNode *next):
 poly(NULL), prev(prev), next(next) {
  if (ln.poly) poly = new T(*(ln.poly));
}

// distruttore 
template <class T>
Contenitore<T>::ListNode::~ListNode() {
  if (poly) delete poly;
  // aggiorna i nodi precedente e successivo
  if (prev) prev->next = next;
  if (next) next->prev = prev;
}

// distrugge in avanti
template <class T>
void Contenitore<T>::ListNode::delete_forward(ListNode *start) {
  if (start) {
    delete_forward(start->next);
    delete start;
  }
}

// distrugge all'indietro
template <class T>
void Contenitore<T>::ListNode::delete_backward(ListNode *start) {
  if (start) {
    delete_backward(start->prev);
    delete start;
  }
}


// ---- CLASSE CONTENITORE ---------------------------------------------------


// ---- COSTRUTTORI ----------------------------------------------------------

// vuoto
template <class T>
Contenitore<T>::Contenitore():
 head(0), tail(0), size(0) {
}

// di copia (profonda)
template <class T>
Contenitore<T>::Contenitore(const Contenitore<T>& c):
 head(0), tail(0), size(c.size) {
  if (c.head) {
    ListNode *cr, *l;
    head = new ListNode(*(c.head));
    cr = c.head->next;
    l = head;
    while (cr) {
      l->next = new ListNode(*cr, l);
      l = l->next;
      cr = cr->next;
    }
    tail = l;
  }
}


// ---- DISTRUTTORE ----------------------------------------------------------

template <class T>
Contenitore<T>::~Contenitore() {
  ListNode::delete_forward(head);
}


// ---- OPERATORI ------------------------------------------------------------

// assegnazione profonda
template <class T>
Contenitore<T>& Contenitore<T>::operator = (const Contenitore<T>& cont) {
  if (this != &cont) {
    clear();
    const_iterator ci = cont.cbegin();
    for (; !ci.is_null(); ++ci) add_tail(*ci);
  }
  return *this;
}

// output su ostream
template <class T>
std::ostream &operator << (std::ostream& os, const Contenitore<T> &c) {
  typename Contenitore<T>::const_iterator i = c.cbegin();
  os << "CONTENITORE:\n";
  for (; !i.is_null(); ++i) os << *i;
  os << "\n";
  return os;
}


// ---- METODI PUBBLICI ------------------------------------------------------

// svuota il contenitore
template <class T>
void Contenitore<T>::clear() {
  ListNode::delete_forward(head);
  head = tail = NULL;
  size = 0;
}

// aggiunge in testa
template <class T>
void Contenitore<T>::add_head(const T &p) {
  T *tp = new T(p);
  if (head) { // non vuoto
    head->prev = new ListNode(tp, 0, head);
    head = head->prev;
  } else { // vuoto
    head = tail = new ListNode(tp);
  }
  size++;
}

// aggiunge in coda
template <class T>
void Contenitore<T>::add_tail(const T &p) {
  T *tp = new T(p);
  if (tail) { // non vuoto
    tail->next = new ListNode(tp, tail);
    tail = tail->next;
  } else { // vuoto
    head = tail = new ListNode(tp);
  }
  size++;
}

// ritorna la testa
template <class T>
T& Contenitore<T>::get_head() throw(std::out_of_range) {
  if (head) return *(head->poly);
  else throw std::out_of_range("Class Contenitore: Vuoto.");
}

// ritorna la testa (costante)
template <class T>
const T& Contenitore<T>::get_head() const throw(std::out_of_range) {
  if (head) return *(head->poly);
  else throw std::out_of_range("Class Contenitore: Vuoto.");
}

// ritorna la coda
template <class T>
T& Contenitore<T>::get_tail() throw(std::out_of_range) {
  if (tail) return *(tail->poly);
  else throw std::out_of_range("Class Contenitore: Vuoto.");
}

// ritorna la coda (costante)
template <class T>
const T& Contenitore<T>::get_tail() const throw(std::out_of_range) {
  if (tail) return *(tail->poly);
  else throw std::out_of_range("Class Contenitore: Vuoto.");
}

// ritorna la testa e la cancella
template <class T>
T Contenitore<T>::del_head() throw(std::out_of_range) {
  if (head) { // non vuoto
    T data = *(head->poly);
    ListNode *next = head->next;
    delete head;
    head = next;
    if (next == NULL) { // un solo elemento
      tail = NULL;
    }
    size--;
    return data;
  } else {
    throw std::out_of_range("Class Contenitore: Vuoto.");
  }
}

// ritorna la coda e la cancella
template <class T>
T Contenitore<T>::del_tail() throw(std::out_of_range) {
  if (tail) { // non vuoto
    T data = *(head->poly);
    ListNode *prev = tail->prev;
    delete tail;
    tail = prev;
    if (prev == NULL) { // un solo elemento
      head = NULL;
    }
    size--;
    return data;
  } else {
    throw std::out_of_range("Class Contenitore: Vuoto.");
  }
}

// cancella l'elemento puntato da it
template <class T>
void Contenitore<T>::del(const iterator& it) throw(std::invalid_argument) {
  ListNode *p = it.node;
  if (!p) throw std::invalid_argument("Class Contenitore: Iteratore non valido.");
  if (p == head) del_head();
  else if (p == tail) del_tail();
  else {
    delete p;
    size--;
  }
}

// cancella l'elemento puntato da it (const_iterator)
template <class T>
void Contenitore<T>::del(const const_iterator& it) throw(std::invalid_argument) {
  ListNode *p = it.node;
  if (!p) throw std::invalid_argument("Class Contenitore: Iteratore non valido.");
  if (p == head) del_head();
  else if (p == tail) del_tail();
  else {
    delete p;
    size--;
  }
}

// restituisce un array contenente tutti gli elementi; è compito del
// chiamante liberare la memoria allocata
template <class T>
T* Contenitore<T>::to_array() const {
  T *buf = (T*)malloc(sizeof(T)*size);
  const_iterator ci = cbegin();
  for (int i=0; !ci.is_null(); ++ci,++i) buf[i] = *ci;
  return buf;
}

// esegue una ricerca utilizzando matchf come funzione per il confronto;
// userdata è passato a matchf. Ritorna un Contenitore di iteratori agli
// elementi trovati
template <class T>
Contenitore<typename Contenitore<T>::iterator> Contenitore<T>::find( bool(*matchf)(const T*, const void*), const void *userdata) {
  Contenitore<typename Contenitore<T>::iterator> cont;
  iterator i = begin();
  for (; !i.is_null(); ++i) if (matchf(&(*i), userdata)) cont.add_tail(i);
  return cont;
}

