/* Dialog.h */

#ifndef DIALOG_H
#define DIALOG_H

#include <QMainWindow>
#include <QtGui>
#include <string>

#include "Contenitore.h"
#include "QDrawSolid.h"
#include "QColorButton.h"

/* Finestra principale */
class Dialog : public QMainWindow {
 Q_OBJECT
 

 // ---- TIPI ----
 
 // contenitore di puntatori a Polyhedron
 typedef Contenitore<Polyhedron*> Polycont;
 
 // enum di cambiamenti alla lista
 enum chType {CLEAR, ADD, CHANGE, DEL};
 
 // enum di tipi di poliedri
 enum polyType {POLY, PRISM, RECTPRISM, PYRAMID, RECTPYRAMID, POLYGON, REGPOLYGON};


 public:
  // ---- COSTRUTTORI ----
  Dialog();


 public slots:
  // ---- SLOT PUBBLICI ----

  // svuota il database
  void cleardb();

  // inserisce un poliedro
  void insert(polyType);
  inline void insert_poly()           { insert(POLY);        }
  inline void insert_prism()          { insert(PRISM);       }
  inline void insert_rectprism()      { insert(RECTPRISM);   }
  inline void insert_pyramid()        { insert(PYRAMID);     }
  inline void insert_rectpyramid()    { insert(RECTPYRAMID); }
  inline void insert_polygon()        { insert(POLYGON);     }
  inline void insert_regularpolygon() { insert(REGPOLYGON);  }
  
  // cerca
  void search();
  
  // cancella un elemento
  void deleteitem();
  
  // salva i dati per un elemento
  void saveitem();
  
  // annulla i cambiamenti ad un elemento
  void cancelitem();
  
  // mostra un elemento
  void showitem(QTreeWidgetItem *cur);
  
  // mostra la finestra di informazioni
  void about();
  
 
 private slots:
  // ---- SLOT PRIVATI ----
  
  // aggiorna la lista dopo qualche cambiamento
  void updateList(chType type, Polycont::iterator *it, QTreeWidgetItem * = NULL);
  
  // seleziona un elemento dalla lista principale se si seleziona nella ricerca
  void syncSearchList(QTreeWidgetItem *);
 

 signals:
  // ---- SEGNALI ----
  
  // il contenuto del contenitore è cambiato
  void contChanged(chType type, Polycont::iterator *it, QTreeWidgetItem * = NULL);
 

 private:
  // il contenitore
  Polycont cont;

  // metodi di set-up di Dialog
  void createActions();
  void createConnections();
  void createMenu();
  void createSolids();

  // Componenti grafiche di Dialog
  QMenuBar* menubar;
  QMenu *menu_file, *menu_help, *menu_additem;
  QToolBar* toolbar;
  QDockWidget *searchdock;
  QCheckBox *searchcb_name, *searchcb_color, *searchcb_surface, *searchcb_volume;
  QComboBox *searchcombo_surface, *searchcombo_volume;
  QDoubleSpinBox *searchsb_surface, *searchsb_volume;
  QStatusBar* statusbar;
  QTreeWidget *list, *searchlist;
  QPushButton *dbnew, *dbadd, *dbdel, 
              *itemsave, *itemcancel,
              *dosearch;
  QColorButton *itemcolor, *search_color;
  QLineEdit *item_name, *search_item_name;
  QLabel *item_surface, *item_volume;
  QDrawSolid *preview;
  QGroupBox *grp_item, *grp_db;
  
  // azioni
  QAction *act_cleardb, *act_showsearch, *act_search, *act_exit, *act_about,
          *act_insert, *act_insertpoly, *act_insertprism, *act_insertpyramid,
          *act_insertrectprism, *act_insertrectpyramid, *act_insertpolygon,
          *act_insertregularpolygon,
          *act_delete, *act_itemsave;
  
  // struttura dati per la ricerca
  typedef struct {
    bool find_name, find_color, find_surface, find_volume;
    std::string name;
    unsigned int color;
    char cmp_surface, cmp_volume;
    double surface, volume;
  } poly_find_data;
  
  // funzione di callback per la ricerca
  static bool poly_compare(Polyhedron* const*, const void *);
};

#endif
