/* Point3D-def.h */

#include <math.h>
#include <typeinfo>


// ---- COSTRUTTORI ----------------------------------------------------------

// costruttore vuoto
template <class Type>
Point3D<Type>::Point3D(): x(0), y(0), z(0) {
}

// costruttore con le 3 componenti
template <class Type>
Point3D<Type>::Point3D(Type xi, Type yi, Type zi):
 x(xi), y(yi), z(zi) {
}

// costruttore di copia E di conversione di tipo
// può portare a perdita di precisione
template <class Type>
template <class Type2>
Point3D<Type>::Point3D(const Point3D<Type2> &p):
 x(p.get_x()), y(p.get_y()), z(p.get_z()) {
}


// ---- OPERATORI ------------------------------------------------------------

// == : uguale
template <class Type>
bool Point3D<Type>::operator == (const Point3D<Type> &p) const {
  return (x == p.x && y == p.y && z == p.z);
}

// != : diverso
template <class Type>
bool Point3D<Type>::operator != (const Point3D<Type> &p) const {
  return (x != p.x || y != p.y || z != p.z);
}

// + : più unario (invariato)
template <class Type>
Point3D<Type> Point3D<Type>::operator +() const {
  return *this;
}

// - : meno unario (cambio di segno)
template <class Type>  
Point3D<Type> Point3D<Type>::operator -() const {
  return Point3D<Type>(-x, -y, -z);
}

// + : somma punto-scalare
template <class Type>
Point3D<Type> Point3D<Type>::operator +(Type v) const {
  return Point3D<Type>(x+v, y+v, z+v);;
}

// + : somma scalare-punto
template <class Type>
Point3D<Type> operator +(Type v, const Point3D<Type> &p) {
  return Point3D<Type>(p.x+v, p.y+v, p.z+v);
}

// - : differenza punto-scalare
template <class Type>
Point3D<Type> Point3D<Type>::operator -(Type v) const {
  return Point3D<Type>(x-v, y-v, z-v);
}

// * : prodotto punto-scalare
template <class Type>
Point3D<Type> Point3D<Type>::operator *(Type v) const {
  return Point3D<Type> (x*v, y*v, z*v);
}

// * : prodotto scalare-punto
template <class Type>
Point3D<Type> operator *(Type v, const Point3D<Type> &p) {
  return Point3D<Type> (p.x*v, p.y*v, p.z*v);
}

// / : rapporto punto-scalare
template <class Type>
Point3D<Type> Point3D<Type>::operator /(Type v) const throw(std::domain_error) {
  if (v == 0) throw std::domain_error("Class Point3D: Divisione per zero.");
  return Point3D<Type> (x/v, y/v, z/v);
}

// + : somma punto-punto (traslazione)
template <class Type>
Point3D<Type> Point3D<Type>::operator +(const Point3D<Type> &p) const {
  return Point3D<Type>(x+p.x, y+p.y, z+p.z);
}

// - : differenza punto-punto
template <class Type>
Point3D<Type> Point3D<Type>::operator -(const Point3D<Type> &p) const {
  return Point3D<Type>(x-p.x, y-p.y, z-p.z);
}

// * : prodotto scalare
template <class Type>
Type Point3D<Type>::operator *(const Point3D<Type> &p) const {
  return (x*p.x + y*p.y+ z*p.z);
}

// = : assegnazione E conversione di tipo
// può portare a perdita di precisione
template <class Type>
template <class Type2>
Point3D<Type>& Point3D<Type>::operator =(const Point3D<Type2> &p) {
  x = p.get_x();
  y = p.get_y();
  z = p.get_z();
  return *this;
}

// += : assegnazione con somma (scalare)
template <class Type>
Point3D<Type>& Point3D<Type>::operator +=(Type v) {
  x += v; y += v; z += v;
  return *this;
}

// -= : assegnazione con differenza (scalare)
template <class Type>
Point3D<Type>& Point3D<Type>::operator -=(Type v) {
  x -= v; y -= v; z -= v;
  return *this;
}

// *= : assegnazione con prodotto (scalare)
template <class Type>
Point3D<Type>& Point3D<Type>::operator *=(Type v) {
  x *= v; y *= v; z *= v;
  return *this;
}

// /= : assegnazione con rapporto (scalare)
template <class Type>
Point3D<Type>& Point3D<Type>::operator /=(Type v) throw(std::domain_error) {
  if (v == 0) throw std::domain_error("Class Point3D: Divisione per zero.");
  x /= v; y /= v; z /= v;
  return *this;
}

// += : assegnazione con somma (punto)
template <class Type>
Point3D<Type>& Point3D<Type>::operator +=(const Point3D<Type> &p) {
  x += p.x; y += p.y; z += p.z;
  return *this;
}

// -= : assegnazione con differenza (punto)
template <class Type>
Point3D<Type>& Point3D<Type>::operator -=(const Point3D<Type> &p) {
  x -= p.x; y -= p.y; z -= p.z;
  return *this;
}

// conversiona a stringa
// vengono tenute 2 cifre decimali per i tipi floating point
template <class Type>
Point3D<Type>::operator std::string() const {
  char numbuf[50];
  if (typeid(Type) == typeid(double) || 
      typeid(Type) == typeid(float) ||
      typeid(Type) == typeid(long double) )
    sprintf(numbuf, "(%.2f, %.2f, %.2f)", 
      static_cast<double>(x),
      static_cast<double>(y),
      static_cast<double>(z) );
  else
    sprintf(numbuf, "(%d, %d, %d)", 
      static_cast<int>(x),
      static_cast<int>(y),
      static_cast<int>(z) );
  return std::string(numbuf);
}

// output su ostream
template <class Type>
std::ostream &operator << (std::ostream& os, const Point3D<Type> &p) {
  return os << std::string(p);
}


// ---- METODI PUBBLICI ------------------------------------------------------

// norma (distanza dall'origine)
template <class Type>
Type Point3D<Type>::norm() const {
  return ( sqrt(x*x + y*y + z*z) );
}

// normalizza (divide per la norma)
template <class Type>
Point3D<Type>& Point3D<Type>::normalize() {
  Type n = norm();
  x /= n; y /= n; z /= n;
  return *this;
}

// distanza tra 2 punti
template <class Type>
Type Point3D<Type>::dist(const Point3D<Type> &p) const {
  if (&p == this) return 0;
  return ( sqrt( (x-p.x)*(x-p.x) + (y-p.y)*(y-p.y) + (z-p.z)*(z-p.z) ) );
}

// distanza punto-piano
template <class Type>
Type Point3D<Type>::plane_dist(const Point3D<Type> &p1, const Point3D<Type> &p2, const Point3D<Type> &p3) const {
  Point3D<Type> v1 = p1-p2, v2 = p1-p3;
  return fabs(det(p1-*this,v1,v2)) / (v1.cross(v2)).norm();
}

// punto medio tra 2 punti
template <class Type>
Point3D<Type> Point3D<Type>::midpoint(const Point3D<Type> &p) const {
  // meno conti se il punto è lo stesso oggetto
  // si potrebbe anche controllare se il punto coincide, pur essendo un
  //  oggetto diverso
  if (&p == this) return *this;
  return (*this+p)/2;
}

// prodotto vettoriale
template <class Type>
Point3D<Type> Point3D<Type>::cross(const Point3D<Type> &p) const {
  Point3D<Type> r(y*p.z-z*p.y, p.x*z-x*p.z, x*p.y-y*p.x);
  return r;
}

// ruota il punto attorno all'origine secondo gli angoli indicati
// ritorna sè stesso
template <class Type>
Point3D<Type>& Point3D<Type>::rotate(double alpha, double beta, double gamma) {
  double s[3],c[3], tx,ty, rx,ry,rz;
  #ifdef _GNU_SOURCE
    sincos(alpha, s+0, c+0);
    sincos(beta, s+1, c+1);
    sincos(gamma, s+2, c+2);
  #else
    s[0] = sin(alpha), c[0] = cos(alpha);
    s[1] = sin(beta) , c[1] = cos(beta);
    s[2] = sin(gamma), c[2] = cos(gamma);
  #endif
  rx = x, ry = y, rz = z;
  ty = ry;
  ry = ty*c[0]-rz*s[0];
  rz = rz*c[0]+ty*s[0];
  tx = rx;
  rx = tx*c[1]-rz*s[1];
  rz = rz*c[1]+tx*s[1];
  tx = rx;
  rx = tx*c[2]-ry*s[2];
  ry = ry*c[2]+tx*s[2];
  x = static_cast<Type> (rx);
  y = static_cast<Type> (ry);
  z = static_cast<Type> (rz);
  return *this;
}

// ruota il punto attorno il punto c secondo gli angoli indicati
template <class Type>
Point3D<Type>& Point3D<Type>::rotate(double alpha, double beta, double gamma, const Point3D<Type> &c) {
  x -= c.x;
  y -= c.y;
  z -= c.z;
  rotate(alpha, beta, gamma);
  x += c.x;
  y += c.y;
  z += c.z;
  return *this;
}

// Calcola l'area del triangolo p1,p2,p3 usando il prodotto vettoriale.
// E' più efficiente della formula di Erone
template <class Type>
Type Point3D<Type>::area3p(const Point3D<Type> &p1, const Point3D<Type> &p2, const Point3D<Type> &p3) {
  return ((p1-p2).cross(p1-p3)).norm()/2;
}


// ---- METODI PRIVATI -------------------------------------------------------

// determinante di 3 vettori u,v,w
template <class Type>
Type Point3D<Type>::det(const Point3D<Type> &u, const Point3D<Type> &v, const Point3D<Type> &w) {
  return (u.x*(v.y*w.z-v.z*w.y) - 
          u.y*(v.x*w.z-v.z*w.x) + 
          u.z*(v.x*w.y-v.y*w.x) );
}
