/* PolyAskDialog.cpp */

#include "PolyAskDialog.h"

// ---- CLASSE PARAMETERS ----------------------------------------------------

// costruttore
PolyAskDialog::parameters::parameters(
 std::string title,
 Contenitore<std::string>* labels,
 Contenitore<std::string>* names,
 Contenitore<Contenitore<Point3Dd> >* ptlists,
 Contenitore<int>* intlist,
 int *color):
 title(title), labels(labels), names(names), ptlists(ptlists), intlist(intlist), color(color)  {
  
}


// ---- CLASSE POLYASKDIALOG -------------------------------------------------


// ---- COSTRUTTORI ----------------------------------------------------------

// costruttore di default
PolyAskDialog::PolyAskDialog(PolyAskDialog::parameters* param, QWidget *parent):
 QDialog(parent), param(param) {
  int i, ofs;
  
  // impostazioni generali
  resize(400,200);
  setModal(true);
  setWindowTitle(QString::fromStdString(param->title));
  
  // creazione griglia
  qg = new QGridLayout(this);
  
  // i vari widget
  list_labels = (QLabel**)malloc(param->labels->length()*sizeof(QLabel*));
  list_names = (QLineEdit**)malloc(param->names->length()*sizeof(QLineEdit*));
  list_ptedit = (QLineEdit**)malloc(param->ptlists->length()*sizeof(QLineEdit*));
  list_int = (QSpinBox**)malloc(param->intlist->length()*sizeof(QSpinBox*));
  
  Contenitore<std::string>::const_iterator li = param->labels->cbegin();
  for (i=0; !li.is_null(); li++,i++) {
    list_labels[i] = new QLabel(li->c_str());
    qg->addWidget(list_labels[i], i,0);
  }

  li = param->names->cbegin();
  for (i=0; !li.is_null(); li++,i++) {
    list_names[i] = new QLineEdit(li->c_str());
    qg->addWidget(list_names[i], i,1);
  }
  ofs = i;

  Contenitore<Contenitore<Point3Dd> >::const_iterator li2 = param->ptlists->cbegin();
  for (i=0; !li2.is_null(); li2++,i++) {
    list_ptedit[i] = new QLineEdit( plist2qstring(*li2) );
    qg->addWidget(list_ptedit[i], i+ofs,1);
  }
  ofs += i;
  
  Contenitore<int>::const_iterator li3 = param->intlist->cbegin();
  for (i=0; !li3.is_null(); li3++,i++) {
    list_int[i] = new QSpinBox();
    list_int[i]->setValue(*li3);
    qg->addWidget(list_int[i], i+ofs, 1);
  }
  ofs += i;
  
  btn_color = new QColorButton(*(param->color));
  qg->addWidget(btn_color, ofs++, 1);
  
  btn_box = new QDialogButtonBox(QDialogButtonBox::Ok | QDialogButtonBox::Cancel);
  qg->addWidget(btn_box, ofs, 0, 1,2);
  
  // connections
  connect(btn_box, SIGNAL(accepted()), this, SLOT(accept()));
  connect(btn_box, SIGNAL(rejected()), this, SLOT(reject()));
}


// ---- DISTRUTTORE ----------------------------------------------------------

PolyAskDialog::~PolyAskDialog() {
  if (list_labels) {
    for (int i=0; i<param->labels->length(); i++) delete list_labels[i];
    free(list_labels);
  }
  if (list_names) {
    for (int i=0; i<param->names->length(); i++) delete list_names[i];
    free(list_names);
  }
  if (list_ptedit) {
    for (int i=0; i<param->ptlists->length(); i++) delete list_ptedit[i];
    free(list_ptedit);
  }
  if (list_int) {
    for (int i=0; i<param->intlist->length(); i++) delete list_int[i];
    free(list_int);
  }
  if (btn_color) delete btn_color;
  if (btn_box) delete btn_box;
  if (qg) delete qg;
}


// ---- SLOT PUBBLICI --------------------------------------------------------

// viene premuto "OK"
void PolyAskDialog::accept() {
  int i;
  
  Contenitore<std::string>::iterator li = param->names->begin();
  for (i=0; i<param->names->length(); ++li,++i) {
    *li = list_names[i]->text().toStdString();
  }

  Contenitore<Contenitore<Point3Dd> >::iterator li2 = param->ptlists->begin();
  for (i=0; i<param->ptlists->length(); ++li2,++i) {
    bool r = qstring2plist(list_ptedit[i]->text(), &(*li2));
    if (!r) {
      QMessageBox::warning(this, tr("Errore"), tr("La lista di vertici non e' valida."));
      return;
    }
  }
  
  Contenitore<int>::iterator li3 = param->intlist->begin();
  for (i=0; i<param->intlist->length(); ++li3,++i) {
    *li3 = list_int[i]->value();
  }
  
  *(param->color) = btn_color->getColor().rgb();
  
  QDialog::accept();
}


// ---- METODI PRIVATI -------------------------------------------------------

// converte una contenitore di punti in una stringa
QString PolyAskDialog::plist2qstring(const Contenitore<Point3Dd>& list) {
  QString ret;
  Contenitore<Point3Dd>::const_iterator i = list.cbegin();
  for (; !i.is_null(); i++) {
    ret += QString::fromStdString(std::string(*i))+QString(" ");
  }
  return ret;
}

// converte una stringa in un contenitore di punti; se fallisce ritorna false
bool PolyAskDialog::qstring2plist(const QString& s, Contenitore<Point3Dd> *list) {
  QString sl = s;
  sl.remove(QChar(' '));
  sl.remove(QChar('\n'));
  sl.remove(QChar('\t'));
  sl.remove(QChar('\v'));
  sl.remove(QChar('\r'));
  sl.prepend(')'); sl.append('(');
  QStringList pl = sl.split(tr(")("), QString::SkipEmptyParts);
  Contenitore<Point3Dd> tcont;
  for (int i=0; i<pl.size(); i++) {
    bool okx, oky, okz;
    QStringList cl = pl[i].split(',', QString::SkipEmptyParts);
    if (cl.size() != 3) return false;
    Point3Dd p(cl[0].toDouble(&okx), cl[1].toDouble(&oky), cl[2].toDouble(&okz));
    if (! (okx && oky && okz)) return false;
    tcont.add_tail(p);
  }
  *list = tcont;
  return true;
}

