/* Polygon.cpp */

#include "Polygon.h"
#include <math.h>

// macro SEGNO
#define SGN(a)		(((a)<0)?-1:(((a)>0)?+1:0))

// ---- COSTRUTTORI ----------------------------------------------------------

// vuoto o con nome
Polygon::Polygon(const std::string& name): 
 Polyhedron(name) {
}

// con lista di vertici
Polygon::Polygon(int numverts, const Point3Dd *listv, const std::string& name):
 Polyhedron(numverts, listv, name) {
 if (!Polyhedron::is_flat()) {  // non è piatto!
   if (verts) free(verts);
   vertices = 0;
   verts = NULL;
 }
}

// di copia
Polygon::Polygon(const Polygon& p):
 Polyhedron(p) {
}


// ---- DISTRUTTORE ----------------------------------------------------------

Polygon::~Polygon() {
}


// ---- OPERATORI ------------------------------------------------------------

// output su ostream
std::ostream& operator << (std::ostream &os, const Polygon& ph) {
  os << "Polygon:\n";
  for (int i=0; i<ph.vertices; i++)
    os << ph.verts[i] << ", ";
  return os << "\n";
}


// ---- METODI PUBBLICI ------------------------------------------------------

// ritorna vero se è convesso. ho usato il secondo metodo
// modi disponibili: 
// 1. somma angoli esterni convesso = 2pi 
// 2. conservazione del segno del (coseno+sqrt(1/2)) tra angoli adiacenti
bool Polygon::is_convex() const {
  double s;
  Point3Dd v,w;
  if (!verts || vertices <= 3) return true;
  v = verts[1]-verts[0];
  w = verts[2]-verts[1];
  s = SGN( (v*w)/(v.norm()*w.norm()) + M_SQRT1_2 );
  for (int i=1; i<vertices; i++) {
    v = w;
    w = verts[(i+2)%vertices]-verts[(i+1)%vertices];
    double sr = SGN( (v*w)/(v.norm()*w.norm()) + M_SQRT1_2 );
    if (sr != s) return false;
    s = sr;
  }
  return true;
}

// calcola l'area per poligoni convessi
double Polygon::get_area() const {
  double ret = 0;
  for (int i=1; i<=vertices-2; i++) {
    ret += Point3Dd::area3p(verts[0], verts[i], verts[i+1]);
  }
  return ret;
}

// calcola il perimetro
double Polygon::get_perimeter() const {
  double ret = 0;
  for (int i=0; i<vertices-1; i++) 
    ret += (verts[i]-verts[i+1]).norm();
  return ret+(verts[vertices-1]-verts[0]).norm();
}

