/* Prism.cpp */

#include "Prism.h"


// ----- COSTRUTTORI ---------------------------------------------------------

// Crea un prisma i cui vertici di base sono listvb, altezza h e direzione degli
//  spigoli laterali hdir. Non viene controllato se i punti della base stanno 
//  effettivamente su un piano.
Prism::Prism(int vertbase, const Point3Dd *listvb, double h, Point3Dd hdir, std::string n):
 Polyhedron(vertbase, listvb, n), height(h) {
  if (vertices > 0) {
    vertices *= 2;
    verts = (Point3Dd *)realloc(verts, vertices*sizeof(Point3Dd));
    // lunghezza dello spigolo
    Point3Dd v = verts[0]-verts[1], w = hdir;
    edge = h/sqrt(1-(v*w)*(v*w)/((v*v)*(w*w)));
    hdir = edge*hdir.normalize();
    for (int i=0; i<vertbase; i++) {
      // base 2:
      verts[i+vertbase] = Point3Dd(verts[i]+hdir);
    }
  }
}


// ----- OPERATORI -----------------------------------------------------------

// output su ostream
std::ostream& operator << (std::ostream &os, const Prism& pr) {
  os << "Prism \"" << pr.name << "\":\n";
  for (int i=0; i<pr.vertices; i++)
    os << pr.verts[i] << ", ";
  return os << "\n";
}


// ----- METODI PUBBLICI -----------------------------------------------------

// calcola l'area di base del prisma, scomponendola in triangoli
// funziona solo per poligoni convessi
double Prism::get_basearea() const {
  double ret = 0;
  for (int i=1; i<=(vertices/2)-2; i++) {
    ret += Point3Dd::area3p(verts[0], verts[i], verts[i+1]);
  }
  return ret;
}

// calcola il perimetro della base
double Prism::get_baseperimeter() const {
  double ret = 0;
  for (int i=0; i<vertices/2-1; i++) 
    ret += (verts[i]-verts[i+1]).norm();
  return ret+(verts[vertices/2-1]-verts[0]).norm();
}

// calcola l'area della superficie
double Prism::get_area() const {
  return 2*get_basearea()+get_baseperimeter()*edge;
}

// calcola il volume del prisma
double Prism::get_volume() const {
  return height*get_basearea();
}

// restituisce la base
Polygon Prism::get_base() const {
  return Polygon(vertices/2, verts, "Base of '"+name+"'");
}
