/* Pyramid.cpp */

#include "Pyramid.h"

// ---- COSTRUTTORI ----------------------------------------------------------

// crea una piramide con la lista di vertici listvb e vertice v
Pyramid::Pyramid(int vertbase, const Point3Dd *listvb, Point3Dd v, std::string n):
 Polyhedron(vertbase, listvb, n) {
  if (vertices > 0) {
    ++vertices;
    verts = (Point3Dd *)realloc(verts, vertices*sizeof(Point3Dd));
    // vertice
    verts[vertices-1] = v;
  }
  calc_height();
}


// ---- OPERATORI ------------------------------------------------------------

// output su ostream
std::ostream& operator << (std::ostream &os, const Pyramid& pr) {
  os << "Pyramid \"" << pr.name << "\":\n";
  for (int i=0; i<pr.vertices; i++)
    os << pr.verts[i] << ", ";
  return os << "\n";
}


// ---- METODI PUBBLICI ------------------------------------------------------

// Calcola l'area di base
double Pyramid::get_basearea() const {
  double ret = 0;
  for (int i=1; i<=vertices-3; i++) {
    ret += Point3Dd::area3p(verts[0], verts[i], verts[i+1]);
  }
  return ret;
}

// Calcola il perimetro della base
double Pyramid::get_baseperimeter() const {
  double ret = 0;
  for (int i=0; i<vertices-2; i++) 
    ret += (verts[i]-verts[i+1]).norm();
  return ret;
}

// Calcola l'area della superficie
double Pyramid::get_area() const {
  double sidearea = 0;
  for (int i=0; i<vertices-2; i++) {
    sidearea += Point3Dd::area3p(verts[i], verts[i+1], verts[vertices-1]);
  }
  return get_basearea()+sidearea;
}

// Calcola il volume
double Pyramid::get_volume() const {
  return height*get_basearea()/3.0;
}

// Restituisce la base
Polygon Pyramid::get_base() const {
  return Polygon(vertices-1, verts, "Base of '"+name+"'");
}


// ---- METODI PROTETTI ------------------------------------------------------

// calcola l'altezza della piramide
double Pyramid::calc_height() {
  if (vertices > 0) {
    height = verts[vertices-1].plane_dist(verts[0], verts[1], verts[2]);
  } else 
    height = 0;
  return height;
}
