/* RegularPolygon.cpp */

#include "RegularPolygon.h"
#include "Matrix3D.h"

// ---- COSTRUTTORI ----------------------------------------------------------

// vuoto
RegularPolygon::RegularPolygon(const std::string& name):
 Polygon(name) {
}

// default
// costruisce un poligono regolare usando come lato p1-p2
// nel piano descritto da p1,p2,pplane (todo: ambiguita' della base)
RegularPolygon::RegularPolygon(
  int numverts, 
  const Point3Dd& p1, const Point3Dd& p2, const Point3Dd& pplane,
  const std::string& name):
 Polygon(name) {
  vertices = numverts;
  if (vertices > 2) {
    verts = (Point3Dd*)malloc(sizeof(Point3Dd)*vertices);
    
    Point3Dd e1 = (p2-p1).normalize();
    Point3Dd e3 = e1.cross(pplane-p1).normalize();
    Point3Dd e2 = e3.cross(e1);
    Matrix3Dd mrot(e1,e2,e3);
    //mrot.invert();

    Point3Dd pt, s(sin(M_PI/vertices), cos(M_PI/vertices), 0.0);
    double rad = (p2-p1).norm()/(2.0*sin(M_PI/vertices));
    
    for (int i=0; i<vertices; i++) {
      int j = 0;
      switch (vertices & 0x03) {
        case 0x01:
        case 0x03:
          pt = Point3Dd(-sin(2*i*M_PI/vertices), cos(2*i*M_PI/vertices), 0.0);
          j = (i+vertices/2+1)%vertices;
          break;
        case 0x02:
          pt = Point3Dd(cos(2*i*M_PI/vertices), sin(2*i*M_PI/vertices), 0.0);
          j = (i+vertices/3)%vertices;
          break;
        case 0x00:
          pt = Point3Dd(cos((2*i-1)*M_PI/vertices), sin((2*i-1)*M_PI/vertices), 0.0);
          j = (i+vertices/4)%vertices;
          break;
      }
      verts[j] = mrot*(rad*(pt+s))+p1;
    }
  }
}

// costrutore di copia
RegularPolygon::RegularPolygon(const RegularPolygon& p):
 Polygon(p) {
}
  

// ---- DISTRUTTORE ----------------------------------------------------------
RegularPolygon::~RegularPolygon() {
}


// ---- OPERATORI ------------------------------------------------------------
  

// ---- METODI PUBBLICI ------------------------------------------------------
  
// calcola l'area
double RegularPolygon::get_area() const {
  double el = get_edge_length();
  return el*el*vertices/(4*tan(M_PI/vertices));
}

// calcola il perimetro
double RegularPolygon::get_perimeter() const {
  return get_edge_length()*vertices;
}

// calcola la lunghezza del lato
double RegularPolygon::get_edge_length() const {
  return (vertices < 2) ? 0 : (verts[0]-verts[1]).norm();
}

// calcola il raggio inscritto
double RegularPolygon::get_inscribed_radius() const {
  return get_edge_length()/(2*tan(M_PI/vertices));
}

// calcola il ragio circoscritto
double RegularPolygon::get_circumscribed_radius() const {
  return get_edge_length()/(2*sin(M_PI/vertices));
}

