/* bitvector */
/* Gestisce un vettore di bit con punteggio. Massimo 64bit */

#ifndef BITVECTOR_H
#define BITVECTOR_H

#include <stdio.h>

typedef struct {
  unsigned long long bits;
  int score;
} BITVEC;

typedef struct {
  int m;
  int n;
  int list[64];
} BITVEC_REC;

char get_bit(BITVEC *bv, int bit) {
  return (bv->bits >> bit) & 1;
}

void set_bit(BITVEC *bv, int bitpos, char bt) {
  unsigned long long mask = ~(1<<bitpos);
  unsigned long long bit = (bt & 1)<<bitpos;
  
  bv->bits &= mask; // clear bit
  bv->bits |= bit;  // set bit
}

void set_bits(BITVEC *bv, char bt) {
  if (bt & 1) bv->bits = 0xffffffffffffffff; else bv->bits = 0x0;
}

void print_bits(FILE *stream, BITVEC *bv) {
  int i;
  for (i=0; i<64; ++i) fprintf(stream, "%i", get_bit(bv, i));
  fprintf(stream, " -- %i\n", bv->score);
}

/* Confronta i bit sulla base del punteggio (crescente) */
int cmp_bits(const void *w1, const void *w2) {
  int s1 = ((const BITVEC*)w1)->score;
  int s2 = ((const BITVEC*)w2)->score;
  if (s1 == s2) return 0;
  if (s1 < s2) return -1;
  return +1;
}

/* Confronta i bit sulla base del punteggio (decrescente) */
int cmp_bits_reverse(const void *w1, const void *w2) {
  int s1 = ((const BITVEC*)w1)->score;
  int s2 = ((const BITVEC*)w2)->score;
  if (s1 == s2) return 0;
  if (s1 < s2) return +1;
  return -1;
}


/* Ad ogni chiamata genera la successibva permutazione di m bit su n posti.
   bvr mantiene le informazioni sulla ricorsione. Alla prima chiamata bvr.list[n] = -1
   ritorna 0 dopo l'ultima combinazione */
int gen_next_bitset(BITVEC *bv, BITVEC_REC *bvr) {
  int i;
  int nbits = bvr->list[bvr->n];
  
  if (nbits == -1) { // primo passo
    for (i=0; i<bvr->m; ++i) bvr->list[i] = i;
    bvr->list[bvr->n] = bvr->m;
    set_bits(bv, 0);
    for (i=0; i<bvr->m; ++i) set_bit(bv, bvr->list[i], 1);
    return 1;
  } else { // altri passi
    if (nbits == bvr->m) { // bit giusti
      while (1) {
        ++bvr->list[nbits-1];
        if (bvr->list[nbits-1] >= bvr->n) --nbits; else break;
      }
      bvr->list[bvr->n] = nbits;
      if (nbits < bvr->m) return gen_next_bitset(bv, bvr);
      // trasforma combinazione
      set_bits(bv, 0);
      for (i=0; i<bvr->m; ++i) set_bit(bv, bvr->list[i], 1);
      return 1;
    } else { // mancano bit
      int prevbit;
      if (nbits > 0) prevbit = bvr->list[nbits-1]; else prevbit = -1;
      bvr->list[nbits] = prevbit+1;
      bvr->list[bvr->n] = nbits+1;
      if (bvr->list[nbits] >= bvr->n) {
        if (nbits == 1) return 0; // fine delle combinazioni
        --nbits;
        ++bvr->list[nbits-1];
        bvr->list[bvr->n] = nbits;
        return gen_next_bitset(bv, bvr);
      }
      if (bvr->list[bvr->n] == bvr->m) {
        set_bits(bv, 0);
        for (i=0; i<bvr->m; ++i) set_bit(bv, bvr->list[i], 1);
        return 1;
      } else {
        return gen_next_bitset(bv, bvr);
      }
    }
  }
  return 0; // mai raggiunto
}

#endif