/*
  TODO:
  
  errori in un metodo apposta
  
*/

#include <QtGui>
#include <QFileInfo>
#include <QMessageBox>

#include "gui.h"


RapiWindow::RapiWindow(QWidget *parent): QMainWindow(parent), generated(false) {
  ui.setupUi(this);
  
  ui_check_avanzate = qFindChild<QCheckBox *>(this, "check_avanzate");
  ui_group_avanzate = qFindChild<QGroupBox *>(this, "group_avanzate");
  ui_text_top = qFindChild<QTextEdit *>(this, "text_top");
  ui_text_result = qFindChild<QTextEdit *>(this, "text_result");
  
  ui_spin_N = qFindChild<QSpinBox *>(this, "spin_N");
  ui_spin_turns = qFindChild<QSpinBox *>(this, "spin_turns");
  ui_spin_consec = qFindChild<QSpinBox *>(this, "spin_consec");
  ui_spin_m_consec = qFindChild<QSpinBox *>(this, "spin_m_consec");
  ui_spin_p_consec = qFindChild<QSpinBox *>(this, "spin_p_consec");
  ui_spin_pm_week = qFindChild<QSpinBox *>(this, "spin_pm_week");
  ui_spin_relax_min = qFindChild<QSpinBox *>(this, "spin_relax_min");
  ui_spin_relax_max = qFindChild<QSpinBox *>(this, "spin_relax_max");
  ui_spin_relax_min = qFindChild<QSpinBox *>(this, "spin_relax_min");
  ui_spin_M_G = qFindChild<QSpinBox *>(this, "spin_M_G");
  ui_spin_M_S = qFindChild<QSpinBox *>(this, "spin_M_S");
  ui_spin_M_D = qFindChild<QSpinBox *>(this, "spin_M_D");
  ui_spin_P_G = qFindChild<QSpinBox *>(this, "spin_P_G");
  ui_spin_P_S = qFindChild<QSpinBox *>(this, "spin_P_S");
  ui_spin_P_D = qFindChild<QSpinBox *>(this, "spin_P_D");
  ui_spin_deep = qFindChild<QSpinBox *>(this, "spin_deep");
  
  ui_group_avanzate->hide();
  this->adjustSize();
  
  logname = "err.log";
  checkExeName();
  
  QString versioninfo;
  if (!exename.isEmpty()) {
    QProcess rsched;
    rsched.start(exename, QStringList("--version"), QIODevice::ReadOnly);
    if (rsched.waitForStarted(5000)) {
      if (rsched.waitForFinished(5000)) {
        QString res = rsched.readAllStandardOutput();
        int in = res.indexOf(QRegExp("[0-9]"));
        int fin = res.lastIndexOf(QRegExp("[0-9]"));
        versioninfo += "\nVersione generatore: "+res.mid(in, fin-in+1);
      }
    }
  }

  versioninfo += QString("\nVersione interfaccia: %1.%2\n").arg(GUI_VERSION_MAJOR).arg(GUI_VERSION_MINOR);  
  ui_text_top->append(versioninfo);
  
}

void RapiWindow::on_check_avanzate_clicked(bool c) {
  if (c) {
     ui_group_avanzate->show();
  } else {
     ui_group_avanzate->hide();
     //this->setSizePolicy(QSizePolicy::Minimum);
     this->adjustSize();
  }
}

void RapiWindow::checkExeName() {
  #ifdef Q_OS_LINUX
    exename = "./rapisched";
  #elif defined Q_OS_WIN32
    exename = "rapisched.exe";
  #endif
  QFileInfo qf(exename);
  if (!qf.exists() || !qf.isExecutable()) {
    QMessageBox::warning(this, "Attenzione", QString::fromUtf8("Il programma rapisched non è stato trovato"));
  }
}

void RapiWindow::on_btn_generate_clicked() {

  this->statusBar()->clearMessage();
  ui_text_result->clear();
  this->statusBar()->showMessage("Avvio generatore...");
  
  QStringList args;
  args << "-n" << QString::number(ui_spin_N->value());
  args << "-t" << QString::number(ui_spin_turns->value());
  args << "-c" << QString::number(ui_spin_consec->value());
  args << "-r" << QString::number(ui_spin_relax_min->value());
  args << "-s" << QString::number(ui_spin_relax_max->value());
  args << "-a" << QString::number(ui_spin_pm_week->value());
  args << "-x" << QString::number(ui_spin_m_consec->value());
  args << "-y" << QString::number(ui_spin_p_consec->value());
  args << "-d" << QString::number(ui_spin_deep->value());
  args << "-m" << QString("%1,%2,%3").arg(ui_spin_M_G->value()).arg(ui_spin_M_S->value()).arg(ui_spin_M_D->value());
  args << "-p" << QString("%1,%2,%3").arg(ui_spin_P_G->value()).arg(ui_spin_P_S->value()).arg(ui_spin_P_D->value());
  args << "--html";
  args << "-o" << "out.html";
  
  QProcess rsched;
  QString stderror;
  rsched.start(exename, args, QIODevice::ReadOnly);
  if (rsched.waitForStarted(2000) && rsched.waitForFinished(10000)) {
    stderror = rsched.readAllStandardError();
  } else {
    QMessageBox::critical(this, "Errore", "Non e' stato possibile avviare il programma o ci impiega troppo tempo.");
    this->statusBar()->clearMessage();
    return;
  }
  this->statusBar()->showMessage("Lettura risultati...");
  
  if (!stderror.isEmpty()) {
    QMessageBox::warning(this, "Attenzione", "Il programma ha restituito il seguente messaggio:\n\n"+stderror);
  }
  
  
  QFile fout("out.html");
  if (!fout.exists() || fout.size() == 0 || !fout.open(QIODevice::ReadOnly)) {
    ui_text_result->setPlainText("Errore nell'esecuzione del programma");
  } else {
    ui_text_result->setHtml(QString(fout.read(200000)));
    fout.close();
    generated = true;
  }
  this->statusBar()->showMessage("Fatto.", 1000);
}

void RapiWindow::on_btn_save_clicked() {
  if (generated) {
    QString filename = QFileDialog::getSaveFileName(this, "Salva...", "", "Documento HTML (*.html *.htm);;Tutti i files (*.*)");
    QFile fout("out.html");
    if (!filename.isEmpty() && !fout.copy(filename)) {
      QMessageBox::warning(this, "Attenzione", QString::fromUtf8("Non è stato possibile salvare il file."));
    }
  } else {
    QMessageBox::warning(this, "Attenzione", "Bisogna prima generare i turni.");
  }
}


void RapiWindow::on_btn_exit_clicked() {
  close();
}