/* CLASS SPACEDIR */

#include "class_spacedir.h"

#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include <unistd.h>
#include <time.h>
#include "../include/mmx.h"

char *font_files = "Franklin_Gothic.ttf";

// initializes the globes
Spacedir::Spacedir(SDL_Surface *screen, const char *dir) {
  srand(time(NULL));
  
  iGlobes = 0;
  globes = NULL;
  globes2 = NULL;
  navMode = 1;  // enables navigation mode
  navSpeed=0.4;
  this->screen = screen;
  virscr = new VirtualScreenSDL(screen);
  virscr->Clear(0);
  virscr->LoadFont(font_files, 14);
  virscr->TextMode = VS_TM_SOLID | VS_TM_NORENDER;
  OSD_Print("Loading directory...");
  
  this->dir = (char*)malloc(sizeof(char)*strlen(dir)+2);
  strcpy(this->dir, dir);
  if ( this->dir[strlen(this->dir)-1] != '/') strcat(this->dir, "/");  
  
  // sets up the Projector
  pro = new Projector();
  pro->moveMe(0,0,-800);
  pro->setUnits(8,8);
  pro->setSizes(screen->w, screen->h);

  // opens directory
  int dirlen = strlen(this->dir);
  struct stat buf;
  DIR *d = opendir(this->dir);
  
  if (d != NULL) {
    struct dirent *de = readdir(d);
    while (de != NULL) {
      
      // skips common names
      if (strcmp(de->d_name, ".") == 0) {
        de = readdir(d);
        continue;
      }

      // allocates memory for globes
      globes = (Globe *)realloc(globes, sizeof(Globe)*(iGlobes+1));
      globes2 = (Globe2D *)realloc(globes2, sizeof(Globe2D)*(iGlobes+1));      

      // gets full path name
      char nam[dirlen+strlen(de->d_name)+2];
      strcpy(nam, dir);
      if (dir[dirlen-1] != '/') strcat(nam, "/");
      strcat(nam, de->d_name);
      
      // gets file/dir info
      stat(nam, &buf);
      
      // sets globe properties
      (globes+iGlobes)->pos.x = rand()%800-400;
      (globes+iGlobes)->pos.y = rand()%800-400;
      (globes+iGlobes)->pos.z = rand()%800-400;
      (globes+iGlobes)->name = strdup(de->d_name);
      (globes+iGlobes)->filetype = buf.st_mode;
      (globes+iGlobes)->selected = 0;
      // precalculates the text
      (globes+iGlobes)->text = virscr->Print(0,0, de->d_name, 0xF0D0B0, 0);
      
      // sets color
      lstat(nam, &buf);
      (globes+iGlobes)->color = 0xFFFFFFFF;
      if (S_ISREG(buf.st_mode)) (globes+iGlobes)->color = 0xFF00FF00;
      if (S_ISDIR(buf.st_mode)) (globes+iGlobes)->color = 0xFF6060FF;
      if (S_ISLNK(buf.st_mode)) (globes+iGlobes)->color = 0xFFFFFF00;
      if (S_ISBLK(buf.st_mode)) (globes+iGlobes)->color = 0xFFFF0000;
      if (S_ISFIFO(buf.st_mode)) (globes+iGlobes)->color = 0xFFFF00FF;
      if (S_ISSOCK(buf.st_mode)) (globes+iGlobes)->color = 0xFF00FFFF;

      iGlobes++;
      de = readdir(d);
    }
  } 
}

Spacedir::~Spacedir() {
  delete pro;
  delete virscr;
  if (globes != NULL) {
    for (int i=0; i<iGlobes; i++) {
      free((globes+i)->name);
      if ((globes+i)->text != NULL) SDL_FreeSurface((globes+i)->text);
    }
    free(globes);
  }
  if (globes2 != NULL) {
    free(globes2);
  }
  if (dir != NULL) free(dir);
}

int Spacedir::OSD_Print(char *text) {
  int oldtm = virscr->TextMode;
  virscr->TextMode = VS_TM_SOLID;
  virscr->Print(10,30, text, 0x00F020, 0);
  virscr->Show();
  virscr->TextMode = oldtm;
  SDL_Delay(200);
}

int cmpGlobe2D(const void *g1, const void *g2) {
  if (((Globe2D*)g1)->z == ((Globe2D*)g2)->z) return(0);
  if (((Globe2D*)g1)->z > ((Globe2D*)g2)->z) return(-1); // inverted!!
  return(1);
}

int Spacedir::drawOnly() {
  virscr->Clear(0);

  // draws globes
  for (int i=0; i<iGlobes2; i++) {
    Globe *g = globes+((globes2+i)->iglobe);
    
    // checks hidden files
    if ( *(g->name) == '.') {
      virscr->FCircle((globes2+i)->x,(globes2+i)->y, (globes2+i)->radius-1, 0xFF000000);
      virscr->Circle((globes2+i)->x,(globes2+i)->y, (globes2+i)->radius, g->color);
    } else {
      virscr->FCircle((globes2+i)->x,(globes2+i)->y, (globes2+i)->radius, g->color);
    }
    if (g->selected) {
      virscr->Circle((globes2+i)->x,(globes2+i)->y, (globes2+i)->radius+1, 0xFFFF0000);
      virscr->Circle((globes2+i)->x,(globes2+i)->y, (globes2+i)->radius+2, 0xFFFF0000);
      virscr->Circle((globes2+i)->x,(globes2+i)->y, (globes2+i)->radius+3, 0xFFFF0000);
    }
    // puts names
    virscr->Unlock();
    if ((globes2+i)->z < 300) {
      SDL_Rect rc = {(globes2+i)->x,(globes2+i)->y,0,0};
      SDL_BlitSurface(g->text, NULL, screen, &rc);
    }
  }
  virscr->Show();
  // clears fpu state ( ... mmx)
  _mm_empty();
}

int Spacedir::draw() {
  PROJECTOR_TYPE ux,uy, dist;
  int x,y, ret;

  iGlobes2 = 0;
  for (int i=0; i<iGlobes; i++) {
    
    // do the projection!
    ret = pro->trace(&((globes+i)->pos), &x, &y);
    if (ret != 0) continue;

    // calculates radius
    dist = pro->calcDistOrigin(&((globes+i)->pos));
    if (dist == 0) continue;
    pro->getUnits(&ux, &uy);
    int rad = (int)(25000.0/(dist*ux));
    
    // sets globes2D
    (globes2+iGlobes2)->z = dist;
    (globes2+iGlobes2)->x = x;
    (globes2+iGlobes2)->y = y;
    (globes2+iGlobes2)->radius = rad;
    (globes2+iGlobes2++)->iglobe = i;
  }

  // sorts globes back-to-front
  qsort(globes2, iGlobes2, sizeof(Globe2D), cmpGlobe2D);
  
  // and do the drawing
  drawOnly();
}

char *Spacedir::getUpDir() {
  char *newdir = (char*)malloc(sizeof(char)*(strlen(dir)+2));
  // checks if we are in the root dir
  if (strcmp(dir, "/") == 0) {
    // in this case just copy the "/"
    strcpy(newdir, "/");
  } else {
    // removes the final "/" from current dir
    this->dir[strlen(this->dir)-1] = '\0';
    // looks for the last slash
    char *bslash = strrchr(dir, '/');
    // should never happens
    if (bslash == NULL) return(NULL);
    strncpy(newdir, dir, bslash-dir+2);
    newdir[bslash-dir+1] = '\0';
  }
  return(newdir);
}

char *Spacedir::getNextDir(char *newdir) {
  char *nextdir;
  if (strcmp(newdir, "..") == 0) {
    return getUpDir();
  } else {
    nextdir = (char*)malloc(sizeof(char)*(strlen(dir)+strlen(newdir)+2));
    strcpy(nextdir, dir);
    strcat(nextdir, newdir);
  }
  return(nextdir);
}

// returns the globes2 index of the clicked planet
int Spacedir::getClicked(int x, int y) {
  for (int i=iGlobes-1; i>=0; i--) {
    int rad = (globes2+i)->radius;
    if (x > ((globes2+i)->x-rad) && x < ((globes2+i)->x+rad) &&
        y > ((globes2+i)->y-rad) && y < ((globes2+i)->y+rad) ) {
      return(i);
    }
  }
  return(-1);
}

int Spacedir::EditMenu(int x, int y) {
  char tostay = 1;
  SDL_Event ev;
  int sizex, sizey;
  VirtualScreenSDL *menu = new VirtualScreenSDL(16,16);
  
  sizex = 150;
  sizey = 250;
  
  virscr->Rect(x,y,x+sizex,y+sizey, 0x80B050);
  virscr->Show();
  while (tostay) {
    if (SDL_WaitEvent(&ev)) {
      switch (ev.type) {
        case SDL_QUIT: tostay=0; break;
        case SDL_MOUSEMOTION: 
          if ((ev.motion.x < x) || (ev.motion.y < y) || (ev.motion.x > x+sizex) || (ev.motion.y > y+sizey)) tostay = 0;
          else {
          }
          break;
      }
    }
  }
  delete menu;
}

/* return values:
	0: nothing important
	1: change dir, must exit loop
	2: quit program
*/
int Spacedir::CatchEvent(char **newdir) {
  int evret, i, mousex, mousey;
  SDL_Event ev;
  char buff[50];

  evret = (navMode)?SDL_PollEvent(&ev):SDL_WaitEvent(&ev);
  if (evret != 0) {
    switch (ev.type) {
      case SDL_KEYDOWN:
        switch (ev.key.keysym.sym) {
          case SDLK_ESCAPE: return(2); break;
          //case SDLK_PAGEUP: pro->moveOriginCloser(10); break;
          //case SDLK_PAGEDOWN: pro->moveOriginCloser(-10); break;
          case SDLK_PAGEUP: 
            if (navMode) {
              navSpeed += 0.4;
              sprintf(buff, "New speed: %f", navSpeed);
              OSD_Print(buff);
            }
            break;
          case SDLK_PAGEDOWN:
            if (navMode) {
              navSpeed -= 0.4;
              sprintf(buff, "New speed: %f", navSpeed);
              OSD_Print(buff);
            }
            break;
          //case SDLK_PLUS: pro->moveUnits(-1,-1); break;
          //case SDLK_MINUS: pro->moveUnits(1,1); break;
          case SDLK_f: SDL_WM_ToggleFullScreen(screen); break;
          case SDLK_BACKSPACE: 
            *newdir = getUpDir();
            return(1);
        }
        break;
      case SDL_QUIT: return(2);
      case SDL_MOUSEBUTTONDOWN: {
        if (ev.button.button == SDL_BUTTON_LEFT) if (!navMode) EditMenu(ev.button.x, ev.button.y);
        //if (ev.button.button == SDL_BUTTON_LEFT) if (!navMode) 
        //if (ev.button.button == SDL_BUTTON_RIGHT) navSpeed -= 0.4;
        if (ev.button.button == SDL_BUTTON_RIGHT) navMode ^= 1;
        if (ev.button.button == SDL_BUTTON_LEFT) {
          if (!navMode) {
            if ((i = getClicked(ev.button.x, ev.button.y)) > 0) {
              Globe *g = globes+(globes2+i)->iglobe;
              if (g->selected && S_ISDIR(g->filetype)) {
                *newdir = getNextDir(g->name);
                return(1);
              } else {                
                navSpeed = 0.0;
                g->selected ^= 1;
              }  
            }
          }
          drawOnly();
        }
        break;
      }
    }
  }
  return(0);
}

int Spacedir::mainLoop(char **newdir) {
  char tostay = 1;  
  int mousex,mousey;
  PROJECTOR_TYPE x,y;
  
  // moves the mouse in the center
  SDL_WarpMouse(screen->w/2, screen->h/2);
  SDL_ShowCursor(SDL_ENABLE);

  while (tostay) {
    
    // poll events
    int evret = CatchEvent(newdir);
    if (evret == 1) return(evret);
    if (evret == 2) tostay = 0;
    
    if (navMode) {
      // rotates view
      SDL_GetMouseState(&mousex, &mousey);
      x = (mousex-(screen->w>>1))/40000.0;
      y = (mousey-(screen->h>>1))/40000.0;
      pro->rotateMeRel(y,x, 0);

      // move ahead
      pro->moveMeRel(0,0,navSpeed);
    
      // draw the scene
      draw();
    } else {
      SDL_GetMouseState(&mousex, &mousey);
      int i = getClicked(mousex, mousey);
      if (i > 0) {
        Globe *g = globes+((globes2+i)->iglobe);
        SDL_Rect rc = {(globes2+i)->x,(globes2+i)->y,0,0};
        SDL_BlitSurface(g->text, NULL, screen, &rc);
        virscr->Show();
        //draw();
      } else {
        drawOnly();
      }
    }
  }
  
  return(0);
}
