/*
ID: ziodf121
LANG: C++
TASK: ariprog
*/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#define MIN(a,b)	(((a)<=(b))?(a):(b))
#define MAX(a,b)	(((a)>=(b))?(a):(b))

#define MAXBISQ		32000
// massimo num di bisquares: 22864
// massimo bisquare: 125000

// se N == 3, delta = qualsiasi
// se N == 4..5 delta = 4*k
// se N > 5 delta = 12*k

typedef struct {
  int start, delta, len;
} PROG;

int N, M;

int bisquares[MAXBISQ];
int nbisq = 0;

PROG *progs = NULL;
int nprogs = 0;

void print_bisquares() {
  for (int i=0; i<nbisq; ++i) printf("%i ", bisquares[i]);
  printf("\n");
}

void print_progs() {
  for (int i=0; i<nprogs; ++i) printf("%i %i\n", bisquares[progs[i].start], progs[i].delta);
  printf("\n");
}

void add_prog(int istart, int delta, int len) {
  if ((nprogs & 0x3ff) == 0) progs = (PROG *)realloc(progs, sizeof(PROG)*(nprogs+0x400));
  if (!progs) exit(5);
  progs[nprogs].start = istart;
  progs[nprogs].delta = delta;
  progs[nprogs].len = len;
  //progs[nprogs].end = end;
  ++nprogs;
}

int findn(int b, int limit1, int limit2) { // cerca un numero nella lista (ordinata) dei bisquares
  while (limit1 < limit2-1) {
    register int pos = (limit1+limit2)>>1;
    if (bisquares[pos] == b) return pos;
    ((bisquares[pos] > b)?limit2:limit1) = pos;
  }
  if (bisquares[limit1] == b) return limit1;
  if (bisquares[limit2] == b) return limit2;
  return -1;
}

void find_progs(int limit) {
  int i,j,k;
  //                      0     3   5             10             15             20                   25
  const int deltamod[] = {0,0,0,1,4,4,12,12,12,12,12,12,12,12,84,84,84,84,84,84,84,84,2772,4096,4096,4096};
  
  if (limit >= 23) return;
  
  for (i=0; i<nbisq; ++i) {
    int l = (limit > 15)?MIN((nbisq>>3)+i,nbisq):nbisq;
    for (j=i+1; j<l; ++j) {
      int next = bisquares[j];
      int delta = next-bisquares[i];
      if (limit > 3 && (delta%deltamod[limit]) != 0) continue;
      int pos = j+((j-i)>>1)-1;
      for (k=3; k<=limit; ++k) {
        next += delta;
        //if (findn(next, j+1, nbisq-1) == -1) break;
        if ((pos = findn(next, pos+1, nbisq-1)) == -1) break;
      }
      if (k > limit) add_prog(i, delta, limit);
    }
    if (nprogs > 11000) break; // tricky!
  }
}

int intcomp(const void *e1, const void *e2) {
  if (*(const int*)e1 > *(const int*)e2) return +1;
  if (*(const int*)e1 < *(const int*)e2) return -1;
  return 0;
}

int progcomp(const void *e1, const void *e2) {
  const PROG *p1 = (const PROG *)e1;
  const PROG *p2 = (const PROG *)e2;
  if (p1->delta < p2->delta) return -1;
  if (p1->delta > p2->delta) return +1;
  if (p1->start < p2->start) return -1;
  if (p1->start > p2->start) return +1;
  return 0;
}

int main() {
  FILE *f = fopen("ariprog.in", "r");
  fscanf(f, "%i\n%i\n", &N, &M);
  fclose(f);
  
  // calc bisquares
  for (int j=0; j<=M; ++j) {
    for (int i=j; i<=M; ++i) {
      bisquares[nbisq++] = i*i+j*j;
    }
  }
  // sort them
  qsort(bisquares, nbisq, sizeof(int), intcomp);
  // remove duplicates   TODO: faster method
  int lastb = bisquares[nbisq-1];
  for (int i=nbisq-2; i>=0; --i) {
    if (lastb == bisquares[i]) bisquares[i] = bisquares[--nbisq];
    else lastb = bisquares[i];
  }
  qsort(bisquares, nbisq, sizeof(int), intcomp);
  
  // analyze:
  int maxdelta = bisquares[1]-bisquares[0];
  for (int i=2; i<nbisq; ++i) {
    if (bisquares[i]-bisquares[i-1] > maxdelta) maxdelta = bisquares[i]-bisquares[i-1];
  }
  //printf("Massima differenza tra 2 consecutivi: %i\n", maxdelta);
  
  //printf("Nr of bisquares: %i\nMax bisquare: %i\n", nbisq, bisquares[nbisq-1]);
  //print_bisquares();
  
  // TOOOOOO SLOW!!
  find_progs(N);
  
  // sort progs
  qsort(progs, nprogs, sizeof(PROG), progcomp);
  
  f = fopen("ariprog.out", "w");
  for (int i = 0; i<nprogs; ++i) {
    if (progs[i].delta == 0) continue;
    fprintf(f, "%i %i\n", bisquares[progs[i].start], progs[i].delta);
  }
  if (nprogs == 0) fprintf(f, "NONE\n");
  fclose(f);
  
  //printf("Nr of progressions: %i\n", nprogs);
  //print_progs();
 
  if (progs) free(progs);
  return 0;
}