/*
ID: ziodf121
LANG: C++
TASK: calfflac
*/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>

#define MAXSIZE		24000

char text[MAXSIZE];
int tlen;

typedef struct {
  unsigned short start;
  unsigned short len;
  unsigned short reallen;
  unsigned short final;
} PALI;

PALI pals[50000]; // = NULL;
int palcount = 0;

void print_pal(const PALI &p) {
  char buf[60];
  sprintf(buf, "%%4i (%%4i): %%.%is\n", p.reallen);
  printf(buf, p.len, p.start, text+p.start);
}

// reverse a string
char *strrev(char *str) {
  int t, i, l = strlen(str);
  for (i=0; i<(l>>1); ++i) {
    t = str[i];
    str[i] = str[l-i-1];
    str[l-i-1] = t;
  }
  return str;
}

bool is_pali(const char *s) {
  char *rs = strdup(s);
  bool ret = !(strcmp(s, strrev(rs)));
  free(rs);
  return ret;
}

// is a palindrome of len?
// ret: -1 = no, n = real len
int is_pal(int start, int len) {
  char *buf;
  int oldstart = start, ret = -1;
  buf = (char*)malloc(sizeof(char)*(len+1));
  int p = 0;
  while (p<len && start < tlen) {
    if (isalpha(text[start])) buf[p++] = tolower(text[start]);
    ++start;
  }
  buf[p] = 0;
  if (p >= len) {
    if (is_pali(buf)) ret = start-oldstart;
  }
  free(buf);
  return ret;
}

void find_pals(int len) {
  for (int i=0; i<tlen-len; ++i) {
    int r = is_pal(i, len);
    //printf("%i  ##%s\n", r, text+i);
    if (r > 0 && isalpha(text[i])) { // only starting with letter
      //if ((palcount&0x3ff) == 00) pals = (PALI*)realloc(pals, sizeof(PALI)*(palcount+0x400));
      pals[palcount].start = i;
      pals[palcount].len = len;
      pals[palcount].reallen = r;
      pals[palcount].final = 0;
      ++palcount;
    }
  }
}

void expand_pals() {
  int done = 1;
  int longest = 0;
  while (done) {
    done = 0;
    for (int i=0; i<palcount; ++i) {
      int ns, p = pals[i].start-1;
      char c1, c2;
      if (pals[i].final) {
        // del from list
        if (pals[i].len < longest) {
          pals[i] = pals[--palcount];
          --i;
        } else longest = pals[i].len;
        continue;
      }
      done = 1;
      // search for alpha char
      while (p >= 0 && !isalpha(text[p])) --p;
      if (p < 0) {
        pals[i].final = 1;
        continue;
      }
      c1 = tolower(text[p]);
      ns = p;
      p = pals[i].start + pals[i].reallen;
      while (p < tlen && !isalpha(text[p])) ++p;
      if (p >= tlen) {
        pals[i].final = 1;
        continue;
      }
      c2 = tolower(text[p]);
      if (c1 == c2) { // new palindrome!!
        pals[i].start = ns;
        pals[i].len += 2;
        pals[i].reallen = p-ns+1;
      } else {
        pals[i].final = 1;
      }
    }
  }
}

int pali_cmp(const void *e1, const void *e2) {
  const PALI *p1 = (const PALI *)e1;
  const PALI *p2 = (const PALI *)e2;
  if (p1->len > p2->len) return -1;
  if (p1->len < p2->len) return +1;
  if (p1->start < p2->start) return -1;
  if (p1->start > p2->start) return 1;
  return 0;
}

int main() {
  FILE *f = fopen("calfflac.in", "r");
  tlen = fread(text, 1, MAXSIZE, f);
  fclose(f);
   
  find_pals(3);
  find_pals(4);
  expand_pals();
  qsort(pals, palcount, sizeof(PALI), pali_cmp);
  //for (int i=0; i<palcount; ++i) print_pal(pals[i]);
  //printf("%i\n", palcount);
  
  f = fopen("calfflac.out", "w");
  char buf[60];
  sprintf(buf, "%%i\n%%.%is\n", pals[0].reallen);
  fprintf(f, buf, pals[0].len, text+pals[0].start);
  fclose(f);
  
  //if (pals) free(pals);
  
  return 0;
}
