/*
ID: ziodf121
LANG: C++
TASK: packrec
*/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#define MAX(a,b)		(((a)>=(b))?(a):(b))
#define MAX3(a,b,c)		MAX((a),MAX((b),(c)))
#define MAX4(a,b,c,d)	MAX(MAX3((a),(b),(c)),(d))

#define MIN(a,b)		(((a)<=(b))?(a):(b))
#define MIN3(a,b,c)		MIN((a),MIN((b),(c)))
#define MIN4(a,b,c,d)	MIN(MIN3((a),(b),(c)),(d))

#define MAXSIZEX		200
#define MAXSIZEY		200

//#define DEBUG

typedef struct {
  short x, y;
  short posx, posy;
} RECT;

// punto nella griglia
typedef struct {
  short x, y;
  short counter; // da quanti rettangoli è condiviso questo bordo
  short angle_counter; // angolo?
} POS;

int maxarea = 0;
RECT rects[4];

POS *border = NULL;
POS *lastborder = NULL;
int blen = 0; // can be a short
int grid[MAXSIZEY+1][MAXSIZEX+1];

RECT *sol = NULL;
int nsol = 0;

int limitx = MAXSIZEX, 
    limity = MAXSIZEY;

#ifdef DEBUG
void print_border() {
  printf("BORDER: ");
  for (int i=0; i<blen; ++i) {
    printf("(%u,%u - %u) ", border[i].x, border[i].y, border[i].counter); 
  }
  printf("\n");
}

void print_rects() {
  printf("RECTS: ");
  for (int i=0; i<4; i++) {
    printf("(%u,%u, %i,%i) ", rects[i].x, rects[i].y, rects[i].posx, rects[i].posy);
  }
  printf("\n");
}
#endif

template <class T>
void xchg(T &e1, T &e2) {
  register T et = e1;
  e1 = e2;
  e2 = et;
}

// true if r1 and r2 overlap
bool overlap(const RECT &r1, const RECT &r2) {
  // sort:
  const RECT *p1 = &r1;
  const RECT *p2 = &r2;
  //printf("OVERLAP: (%i,%i %i,%i) vs (%i,%i %i,%i) ... ", r1.x,r1.y, r1.posx,r1.posy, r2.x,r2.y, r2.posx,r2.posy);
  if (r2.posy < r1.posy) {
    p1 = &r2;
    p2 = &r1;
  } else if (r1.posy == r2.posy && r2.posx < r1.posx) {
    p1 = &r2;
    p2 = &r1;
  }
  //p1 higher or at left of r2
  if (p1->posy+p1->y <= p2->posy) { /*printf("FALSE\n"); */return false; }
  if (p1->posx+p1->x <= p2->posx) { /*printf("FALSE\n"); */return false; }
  if (p2->posx+p2->x <= p1->posx) { /*printf("FALSE\n"); */return false; }
  //printf("TRUE\n");
  return true;
}

// rect is out of limits
bool outside(const RECT &r) {
  if (r.posx+r.x > limitx || r.posy+r.y > limity) return true;
  return false;
}

void add_sol(const RECT &r) {
  if ((nsol & 0xff) == 0) sol = (RECT*)realloc(sol, sizeof(RECT)*(nsol+0x100));
  #ifdef DEBUG
  if (sol == NULL) printf("Out of memory.\n");
  #endif
  sol[nsol++] = r;
}


// calc boundary rectangle for given rects (uses positions in rects, if posx==-1, not used)
RECT calc_boundary_auto() {
  short x0=MAXSIZEX,y0=MAXSIZEY, x1=0,y1=0;
  for (int i=0; i<4; ++i) {
    if (rects[i].posx == -1) continue;
    x0 = MIN(x0, rects[i].posx);
    y0 = MIN(y0, rects[i].posy);
    x1 = MAX(x1, rects[i].posx+rects[i].x);
    y1 = MAX(y1, rects[i].posy+rects[i].y);
  }
  RECT ret = {x1-x0,y1-y0, x0,y0};
  return ret;
}

// aggiunge un punto al vettore del bordo (in coda al vettore bordi)
void addborder(unsigned short x, unsigned short y, bool corner) {
  if (y < MAXSIZEY && x < MAXSIZEX && grid[y][x] == -1) { // add it!
    if ((blen & 0x3ff) == 0) border = (POS*)realloc(border, sizeof(POS)*(blen+0x400));
    border[blen].x = x;
    border[blen].y = y;
    border[blen].counter = 1;
    border[blen].angle_counter = corner;
    grid[y][x] = blen++;
  } else { // esiste gia, incremento il contatore
    if (y < MAXSIZEY && x < MAXSIZEX) {
      ++border[grid[y][x]].counter;
      border[grid[y][x]].angle_counter += corner;
    }
  }
}

// toglie un punto al vettore bordo (sostituisce con l'ultimo)
void delborder(unsigned short x, unsigned short y, bool corner) {
  if (y < MAXSIZEY && x < MAXSIZEX && grid[y][x] != -1) {
    register int idx = grid[y][x];
    --border[idx].counter;
    border[idx].angle_counter -= corner;
    if (border[idx].counter == 0) { // non era condiviso
      border[idx] = border[--blen];
      grid[border[idx].y][border[idx].x] = idx;
      grid[y][x] = -1;
      if (blen == 0) {
        free(border);
        border = NULL;
      } else {
        if ((blen & 0x3ff) == 0) border = (POS*)realloc(border, sizeof(POS)*(blen));
      }
    }
  }
}


// imposta (aggiunge o toglie il bordo del rettangolo
// addbdr true per aggiungere, false per togliere
// TODO: rifinire per bene (tenere i vertici, + controlli
void calcborder(const RECT &r, bool addbdr = true) {
  int i;
  void(*funcs[2])(unsigned short, unsigned short, bool) = {delborder, addborder};
  
  // orizzontale sopra
  funcs[addbdr](r.posx, r.posy, 1);
  for (i=r.posx+1; i<r.posx+r.x; ++i) funcs[addbdr](i, r.posy, false);
  
  // orizzontale sotto
  funcs[addbdr](r.posx, r.posy+r.y, 1);
  for (i=r.posx+1; i<r.posx+r.x; ++i) funcs[addbdr](i, r.posy+r.y, false);
  
  // verticali sx
  funcs[addbdr](r.posx+r.x, r.posy, 1);
  for (i=r.posy+1; i<r.posy+r.y; ++i) funcs[addbdr](r.posx, i, false);
  
  // verticale dx
  funcs[addbdr](r.posx+r.x, r.posy+r.y, 1);
  for (i=r.posy+1; i<r.posy+r.y; ++i) funcs[addbdr](r.posx+r.x, i, false);
}

RECT last_boundary;

void place(int level) {
  int i,j,k;
  #ifdef DEBUG
  printf("\nLEVEL: %i\n", level);
  print_border();
  print_rects();
  #endif
  if (level == 0) { // first rect
    for (i=0; i<4; ++i) {
      rects[i].posx = rects[i].posy = 0;
      //int giri = (rects[i].x==rects[i].y)?1:2;
      int giri = 1;
      for (int sw=0; sw<giri; ++sw) { // scambio coordinate
        calcborder(rects[i], true);
        place(level+1);
        calcborder(rects[i], false); // remove old borders
        xchg(rects[i].x, rects[i].y);
      }
      rects[i].posx = rects[i].posy = -1; // rect not placed
    }
    return;
  } else if (level >= 4) {
    
    int area = last_boundary.x*last_boundary.y;
    if (area <= maxarea) { // good!
      add_sol(last_boundary);
      maxarea = area;
    }
    
    return;
  }

  // next levels
  for (j=0; j<4; ++j) { // scegli rett.
    if (rects[j].posx != -1) continue; // already placed
    int giri = (rects[j].x==rects[j].y)?1:2;
    for (int sw=0; sw<giri; ++sw) { // scambio coordinate
      int oldblen = blen;
      int oldarea = MAXSIZEX*MAXSIZEY+1;
      for (i=0; i<oldblen; ++i) {
        // controllo del bordo
        if (border[i].counter >= 2 && border[i].angle_counter == 0) continue;
        // ok lo uso come posizione
        rects[j].posx = border[i].x;
        rects[j].posy = border[i].y;
        if (outside(rects[j])) continue;
        for (k=0; k<4; ++k) {
          if (k != j && rects[k].posx != -1 && overlap(rects[k], rects[j])) break;
        }
        if (k<4) continue;
      
        // good placement!
        last_boundary = calc_boundary_auto();
        int area = last_boundary.x*last_boundary.y;
        if (area <= maxarea) { // seems good
          if (level < 4) calcborder(rects[level], true);
          place(level+1);
          if (level < 4) calcborder(rects[level], false);
        } else { // area too big          
          if (area > oldarea) { // area is growing
            while (++i < oldblen && border[i].angle_counter == 0) ; // go to next angle
            --i;
            area = MAXSIZEX*MAXSIZEY+1;
          }
        }
        oldarea = area;
      }
      xchg(rects[j].x, rects[j].y);
    }
    rects[j].posx = rects[j].posy = -1;
  }
}

int solcomp(const void *e1, const void *e2) {
  const RECT *r1 = (const RECT *)e1;
  const RECT *r2 = (const RECT *)e2;
  if (r1->x < r2->x) return -1;
  if (r1->x > r2->x) return +1;
  return 0;
}

int main() {
  
  FILE *f = fopen("packrec.in", "r");
  for (int i=0; i<4; ++i) {
    fscanf(f, "%u %u\n", &(rects[i].x), &(rects[i].y));
    rects[i].posx = rects[i].posy = -1;
    // ordino, rettangoli distesi, x > y
    if (rects[i].x < rects[i].y) xchg(rects[i].x, rects[i].y);
  }
  fclose(f);
  
  qsort(rects, 4, sizeof(RECT), solcomp);
  
  // calc stupid max area
  int max1, max2;
  max1 = MAX4(rects[0].x,rects[1].x,rects[2].x,rects[3].x)*(rects[0].y+rects[1].y+rects[2].y+rects[3].y);
  max2 = MAX4(rects[0].y,rects[1].y,rects[2].y,rects[3].y)*(rects[0].x+rects[1].x+rects[2].x+rects[3].x);
  maxarea = MIN(max1, max2);
  
  limitx = MIN(MAXSIZEX, rects[0].x+rects[1].x+rects[2].x+rects[3].x);
  limity = MIN(MAXSIZEY, rects[0].y+rects[1].y+rects[2].y+rects[3].y);

  #ifdef DEBUG
  printf("MAXAREA: %i\n", maxarea);
  #endif
  
  memset(grid, 0xff, sizeof(grid));
  place(0);
  
  // pulisco soluzioni non ottime
  for (int i=nsol-1; i>=0; --i) {
    if (sol[i].x > sol[i].y) xchg(sol[i].x, sol[i].y);
    if (sol[i].x*sol[i].y > maxarea) sol[i] = sol[--nsol];
  }
  
  // ordino soluzioni
  qsort(sol, nsol, sizeof(RECT), solcomp);
  
  f = fopen("packrec.out", "w");
  fprintf(f, "%i\n", maxarea);
  int lastx = -1;
  for (int i=0; i<nsol; ++i) {
    if (lastx != sol[i].x) fprintf(f, "%u %u\n", sol[i].x, sol[i].y);
    lastx = sol[i].x;
  }
  fclose(f);
  
  
  if (border) free(border);
  if (sol) free(sol);
  return 0;
}